@@ -8,8 +8,6 @@
  *
  *          Implementation of the Intel PCISet chips from 420TX to 440GX.
  *
- *
- *
  * Authors: Miran Grca, <mgrca8@gmail.com>
  *
  *          Copyright 2019-2020 Miran Grca.
@@ -1013,7 +1011,8 @@ i4x0_write(int func, int addr, uint8_t val, void *priv)
                     case INTEL_430TX:
                         if (!dev->smram_locked) {
                             i4x0_smram_handler_phase0(dev);
-                            regs[0x71] = (regs[0x71] & 0x20) | (val & 0xdf);
+                            regs[0x71] = (regs[0x71] & 0x60) | (val & 0x9f);
+                            regs[0x71] &= (val & 0x40);
                             i4x0_smram_handler_phase1(dev);
                         }
                         break;
@@ -1041,9 +1040,11 @@ i4x0_write(int func, int addr, uint8_t val, void *priv)
                             regs[0x72] = (val & 0x7f);
                         else
                             regs[0x72] = (regs[0x72] & 0x87) | (val & 0x78);
-                        dev->smram_locked = (val & 0x10);
-                        if (dev->smram_locked)
-                            regs[0x72] &= 0xbf;
+                        if (val & 0x08) {
+                            dev->smram_locked = (val & 0x10);
+                            if (dev->smram_locked)
+                                regs[0x72] &= 0xbf;
+                        }
                     }
                 } else {
                     if (dev->smram_locked)
@@ -1546,7 +1547,7 @@ i4x0_read(int func, int addr, void *priv)
            with the addition of bits 3 and 0. */
         if ((func == 0) && (addr == 0x93) && ((dev->type == INTEL_440FX) || (dev->type == INTEL_440LX) || (dev->type == INTEL_440EX)))
             ret = (ret & 0xf9) | (pci_read(0x0cf9, NULL) & 0x06);
-        else if ((func == 0) && (addr == 0x52) && (dev->type == INTEL_430TX) && !strcmp(machine_get_internal_name(), "tomahawk"))
+        else if ((func == 0) && (addr == 0x52) && (dev->type == INTEL_430TX) && (machines[machine].init == machine_at_tomahawk_init))
             ret = 0xb2;
     }
 
@@ -1577,6 +1578,8 @@ i4x0_reset(void *priv)
             dev->regs[0x68 + i] = 0x00;
     }
 
+    dev->smram_locked = 0;
+
     if (dev->type >= INTEL_430FX) {
         dev->regs[0x72] &= 0xef; /* Forcibly unlock the SMRAM register. */
         i4x0_write(0, 0x72, 0x02, priv);
@@ -1608,11 +1611,9 @@ i4x0_close(void *priv)
 static void *
 i4x0_init(const device_t *info)
 {
-    i4x0_t  *dev = (i4x0_t *) malloc(sizeof(i4x0_t));
+    i4x0_t  *dev = (i4x0_t *) calloc(1, sizeof(i4x0_t));
     uint8_t *regs;
 
-    memset(dev, 0, sizeof(i4x0_t));
-
     dev->smram_low  = smram_add();
     dev->smram_high = smram_add();
 
@@ -1658,7 +1659,12 @@ i4x0_init(const device_t *info)
             regs[0x57] = 0x31;
             regs[0x59] = 0x0f;
             regs[0x60] = regs[0x61] = regs[0x62] = regs[0x63] = 0x02;
-            dev->max_drb                                      = 3;
+            /* At the very least the 420ZX seems to read to 0x64, per the SB486PV. */
+            if (dev->type == INTEL_420ZX) {
+                regs[0x64]   = 0x02;
+                dev->max_drb = 4;
+            } else
+                dev->max_drb                                      = 3;
             dev->drb_unit                                     = 1;
             dev->drb_default                                  = 0x02;
             break;
@@ -1967,7 +1973,7 @@ const device_t i420tx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1981,7 +1987,7 @@ const device_t i420zx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1995,7 +2001,7 @@ const device_t i430lx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2009,7 +2015,7 @@ const device_t i430nx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2023,7 +2029,7 @@ const device_t i430fx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2037,7 +2043,7 @@ const device_t i430fx_rev02_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2051,7 +2057,7 @@ const device_t i430hx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2065,7 +2071,7 @@ const device_t i430vx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2079,7 +2085,7 @@ const device_t i430tx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2093,7 +2099,7 @@ const device_t i440fx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2107,7 +2113,7 @@ const device_t i440lx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2121,7 +2127,7 @@ const device_t i440ex_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2135,7 +2141,7 @@ const device_t i440bx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2149,7 +2155,7 @@ const device_t i440bx_no_agp_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2163,7 +2169,7 @@ const device_t i440gx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2177,7 +2183,7 @@ const device_t i440zx_device = {
     .init          = i4x0_init,
     .close         = i4x0_close,
     .reset         = i4x0_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
