@@ -1,5 +1,48 @@
+
+#ifdef X87_INLINE_ASM
+static inline double float_add(double src, double val, int round)
+{
+    int rounding_mode_orig;
+
+    __m128d xmm_src = _mm_load_sd(&src);
+    __m128d xmm_dst = _mm_load_sd(&val);
+    __m128d xmm_res;
+
+    rounding_mode_orig = _MM_GET_ROUNDING_MODE();
+    if (round == 0) _MM_SET_ROUNDING_MODE(_MM_ROUND_NEAREST);
+    if (round == 1) _MM_SET_ROUNDING_MODE(_MM_ROUND_DOWN);
+    if (round == 2) _MM_SET_ROUNDING_MODE(_MM_ROUND_UP);
+    if (round == 3) _MM_SET_ROUNDING_MODE(_MM_ROUND_TOWARD_ZERO);
+
+    xmm_res = _mm_add_sd(xmm_src, xmm_dst);
+
+    _MM_SET_ROUNDING_MODE(rounding_mode_orig);
+
+    return _mm_cvtsd_f64(xmm_res);
+}
+
+#define DO_FADD(use_var)                                               \
+    do                                                                 \
+    {                                                                  \
+        ST(0) = float_add(ST(0), use_var, (cpu_state.npxc >> 10) & 3); \
+    }                                                                  \
+    while (0)
+
+#else
+#define DO_FADD(use_var)                                               \
+    do                                                                 \
+    {                                                                  \
+        if ((cpu_state.npxc >> 10) & 3)                                \
+            fesetround(rounding_modes[(cpu_state.npxc >> 10) & 3]);    \
+        ST(0) += use_var;                                              \
+        if ((cpu_state.npxc >> 10) & 3)                                \
+            fesetround(FE_TONEAREST);                                  \
+    }                                                                  \
+    while (0)
+#endif
+
 #define opFPU(name, optype, a_size, load_var, get, use_var, cycle_postfix)                                                                         \
-    static int opFADD##name##_a##a_size(uint32_t fetchdat)                                                                                         \
+    static int opFADD##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                 \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -8,17 +51,13 @@
         load_var = get();                                                                                                                          \
         if (cpu_state.abrt)                                                                                                                        \
             return 1;                                                                                                                              \
-        if ((cpu_state.npxc >> 10) & 3)                                                                                                            \
-            fesetround(rounding_modes[(cpu_state.npxc >> 10) & 3]);                                                                                \
-        ST(0) += use_var;                                                                                                                          \
-        if ((cpu_state.npxc >> 10) & 3)                                                                                                            \
-            fesetround(FE_TONEAREST);                                                                                                              \
+        DO_FADD(use_var);                                                                                                                          \
         FP_TAG_VALID;                                                                                                                              \
         CLOCK_CYCLES_FPU((fpu_type >= FPU_487SX) ? (x87_timings.fadd##cycle_postfix) : ((x87_timings.fadd##cycle_postfix) * cpu_multi));           \
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fadd##cycle_postfix) : ((x87_concurrency.fadd##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFCOM##name##_a##a_size(uint32_t fetchdat)                                                                                         \
+    static int opFCOM##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                 \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -27,13 +66,13 @@
         load_var = get();                                                                                                                          \
         if (cpu_state.abrt)                                                                                                                        \
             return 1;                                                                                                                              \
-        cpu_state.npxs &= ~(FPU_SW_C0 | FPU_SW_C2 | FPU_SW_C3);                                                                                                         \
+        cpu_state.npxs &= ~(FPU_SW_C0 | FPU_SW_C2 | FPU_SW_C3);                                                                                    \
         cpu_state.npxs |= x87_compare(ST(0), (double) use_var);                                                                                    \
         CLOCK_CYCLES_FPU((fpu_type >= FPU_487SX) ? (x87_timings.fcom##cycle_postfix) : ((x87_timings.fcom##cycle_postfix) * cpu_multi));           \
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fcom##cycle_postfix) : ((x87_concurrency.fcom##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFCOMP##name##_a##a_size(uint32_t fetchdat)                                                                                        \
+    static int opFCOMP##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -42,14 +81,14 @@
         load_var = get();                                                                                                                          \
         if (cpu_state.abrt)                                                                                                                        \
             return 1;                                                                                                                              \
-        cpu_state.npxs &= ~(FPU_SW_C0 | FPU_SW_C2 | FPU_SW_C3);                                                                                                         \
+        cpu_state.npxs &= ~(FPU_SW_C0 | FPU_SW_C2 | FPU_SW_C3);                                                                                    \
         cpu_state.npxs |= x87_compare(ST(0), (double) use_var);                                                                                    \
         x87_pop();                                                                                                                                 \
         CLOCK_CYCLES_FPU((fpu_type >= FPU_487SX) ? (x87_timings.fcom##cycle_postfix) : ((x87_timings.fcom##cycle_postfix) * cpu_multi));           \
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fcom##cycle_postfix) : ((x87_concurrency.fcom##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFDIV##name##_a##a_size(uint32_t fetchdat)                                                                                         \
+    static int opFDIV##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                 \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -64,7 +103,7 @@
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fadd##cycle_postfix) : ((x87_concurrency.fadd##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFDIVR##name##_a##a_size(uint32_t fetchdat)                                                                                        \
+    static int opFDIVR##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -79,7 +118,7 @@
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fdiv##cycle_postfix) : ((x87_concurrency.fdiv##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFMUL##name##_a##a_size(uint32_t fetchdat)                                                                                         \
+    static int opFMUL##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                 \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -94,7 +133,7 @@
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fmul##cycle_postfix) : ((x87_concurrency.fmul##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFSUB##name##_a##a_size(uint32_t fetchdat)                                                                                         \
+    static int opFSUB##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                 \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -109,7 +148,7 @@
         CONCURRENCY_CYCLES((fpu_type >= FPU_487SX) ? (x87_concurrency.fadd##cycle_postfix) : ((x87_concurrency.fadd##cycle_postfix) * cpu_multi)); \
         return 0;                                                                                                                                  \
     }                                                                                                                                              \
-    static int opFSUBR##name##_a##a_size(uint32_t fetchdat)                                                                                        \
+    static int opFSUBR##name##_a##a_size(UNUSED(uint32_t fetchdat))                                                                                        \
     {                                                                                                                                              \
         optype t;                                                                                                                                  \
         FP_ENTER();                                                                                                                                \
@@ -208,7 +247,7 @@ opFCOMP(uint32_t fetchdat)
 }
 
 static int
-opFCOMPP(uint32_t fetchdat)
+opFCOMPP(UNUSED(uint32_t fetchdat))
 {
     uint64_t *p, *q;
     FP_ENTER();
@@ -229,7 +268,7 @@ opFCOMPP(uint32_t fetchdat)
 }
 #ifndef FPU_8087
 static int
-opFUCOMPP(uint32_t fetchdat)
+opFUCOMPP(UNUSED(uint32_t fetchdat))
 {
     FP_ENTER();
     cpu_state.pc++;
