@@ -9,8 +9,6 @@
  *          Implementation of the generic device interface to handle
  *          all devices attached to the emulator.
  *
- *
- *
  * Authors: Fred N. van Kempen, <decwiz@yahoo.com>
  *          Miran Grca, <mgrca8@gmail.com>
  *          Sarah Walker, <https://pcem-emulator.co.uk/>
@@ -19,7 +17,7 @@
  *          Copyright 2016-2019 Miran Grca.
  *          Copyright 2008-2019 Sarah Walker.
  *          Copyright 2021      Andreas J. Reichel.
- *          Copyright 2021-2022 Jasmine Iwanek.
+ *          Copyright 2021-2025 Jasmine Iwanek.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +43,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <wchar.h>
 #define HAVE_STDARG_H
 #include <86box/86box.h>
 #include <86box/ini.h>
@@ -52,8 +51,10 @@
 #include <86box/device.h>
 #include <86box/machine.h>
 #include <86box/mem.h>
+#include <86box/plat.h>
 #include <86box/rom.h>
 #include <86box/sound.h>
+#include <86box/ui.h>
 
 #define DEVICE_MAX 256 /* max # of devices */
 
@@ -89,22 +90,38 @@ device_init(void)
 }
 
 void
-device_set_context(device_context_t *c, const device_t *dev, int inst)
+device_set_context(device_context_t *ctx, const device_t *dev, int inst)
 {
-    memset(c, 0, sizeof(device_context_t));
-    c->dev      = dev;
-    c->instance = inst;
+    memset(ctx, 0, sizeof(device_context_t));
+    ctx->dev      = dev;
+    ctx->instance = inst;
     if (inst) {
-        sprintf(c->name, "%s #%i", dev->name, inst);
+        sprintf(ctx->name, "%s #%i", dev->name, inst);
 
         /* If a numbered section is not present, but a non-numbered of the same name
            is, rename the non-numbered section to numbered. */
-        const void *sec        = config_find_section(c->name);
+        const void *sec        = config_find_section(ctx->name);
         void *      single_sec = config_find_section((char *) dev->name);
         if ((sec == NULL) && (single_sec != NULL))
-            config_rename_section(single_sec, c->name);
+            config_rename_section(single_sec, ctx->name);
+    } else if (!strcmp(dev->name, "PS/2 Mouse")) {
+        sprintf(ctx->name, "%s", dev->name);
+
+        /* Migrate the old "Standard PS/2 Mouse" section */
+        const void *sec        = config_find_section(ctx->name);
+        void *      old_sec    = config_find_section("Standard PS/2 Mouse");
+        if ((sec == NULL) && (old_sec != NULL))
+            config_rename_section(old_sec, ctx->name);
+    } else if (!strcmp(dev->name, "Microsoft RAMCard")) {
+        sprintf(ctx->name, "%s", dev->name);
+
+        /* Migrate the old "Microsoft RAMCard for IBM PC" section */
+        const void *sec        = config_find_section(ctx->name);
+        void *      old_sec    = config_find_section("Microsoft RAMCard for IBM PC");
+        if ((sec == NULL) && (old_sec != NULL))
+            config_rename_section(old_sec, ctx->name);
     } else
-        sprintf(c->name, "%s", dev->name);
+        sprintf(ctx->name, "%s", dev->name);
 }
 
 static void
@@ -137,7 +154,25 @@ device_add_common(const device_t *dev, void *p, void *params, int inst)
 {
     device_t *init_dev = NULL;
     void     *priv     = NULL;
-    int       c;
+    int16_t   c;
+
+    /*
+       IMPORTANT: This is needed to gracefully handle machine
+                  device addition if the relevant device is NULL.
+     */
+    if (dev == NULL)
+        return NULL;
+
+    if (!device_available(dev)) {
+        wchar_t temp[512] = { 0 };
+        swprintf(temp, sizeof_w(temp),
+                 plat_get_string(STRING_HW_NOT_AVAILABLE_DEVICE),
+                 dev->name);
+        ui_msgbox_header(MBX_INFO,
+                         plat_get_string(STRING_HW_NOT_AVAILABLE_TITLE),
+                         temp);
+        return ((void *) dev->name);
+    }
 
     if (params != NULL) {
         init_dev = calloc(1, sizeof(device_t));
@@ -146,7 +181,7 @@ device_add_common(const device_t *dev, void *p, void *params, int inst)
     } else
         init_dev = (device_t *) dev;
 
-    for (c = 0; c < 256; c++) {
+    for (c = 0; c < DEVICE_MAX; c++) {
         if (!inst && (devices[c] == dev)) {
             device_log("DEVICE: device already exists!\n");
             return (NULL);
@@ -155,7 +190,8 @@ device_add_common(const device_t *dev, void *p, void *params, int inst)
             break;
     }
     if (c >= DEVICE_MAX) {
-        fatal("DEVICE: too many devices\n");
+        fatal("Attempting to initialize more than the maximum "
+              "limit of %i devices\n", DEVICE_MAX);
         return NULL;
     }
 
@@ -228,6 +264,7 @@ void *
 device_add_linked(const device_t *dev, void *priv)
 {
     void *ret;
+
     device_common_priv = priv;
     ret = device_add_common(dev, NULL, NULL, 0);
     device_common_priv = NULL;
@@ -295,7 +332,8 @@ device_close_all(void)
 #endif
             if (devices[c]->close != NULL)
                 devices[c]->close(device_priv[c]);
-            devices[c] = device_priv[c] = NULL;
+            devices[c]     = NULL;
+            device_priv[c] = NULL;
         }
     }
 }
@@ -309,12 +347,6 @@ device_reset_all(uint32_t match_flags)
                 devices[c]->reset(device_priv[c]);
         }
     }
-
-#ifdef UNCOMMENT_LATER
-    /* TODO: Actually convert the LPT devices to device_t's. */
-    if ((match_flags == DEVICE_ALL) || (match_flags == DEVICE_PCI))
-        lpt_reset();
-#endif
 }
 
 void *
@@ -350,76 +382,83 @@ device_get_priv(const device_t *dev)
 int
 device_available(const device_t *dev)
 {
-    const device_config_t      *config       = NULL;
-    const device_config_bios_t *bios         = NULL;
-
-    if (dev != NULL) {
-        config = dev->config;
-        if (config != NULL) {
-            while (config->type != -1) {
-                if (config->type == CONFIG_BIOS) {
-                    int roms_present = 0;
-
-                    bios = (const device_config_bios_t *) config->bios;
-
-                    /* Go through the ROM's in the device configuration. */
-                    while (bios->files_no != 0) {
-                        int i = 0;
-                        for (int bf = 0; bf < bios->files_no; bf++)
-                            i += !!rom_present(bios->files[bf]);
-                        if (i == bios->files_no)
-                            roms_present++;
-                        bios++;
-                    }
-
-                    return (roms_present ? -1 : 0);
-                }
-                config++;
-            }
-        }
+    int ret = machine_device_available(dev);
 
+    if (ret == 0) {
         /* No CONFIG_BIOS field present, use the classic available(). */
         if (dev->available != NULL)
-            return (dev->available());
+            ret = (dev->available());
         else
-            return 1;
+            ret = (dev != NULL);
+    } else
+        ret = (ret == -1);
+
+    return ret;
+}
+
+static const device_config_bios_t *
+device_get_bios(const device_t *dev, const char *internal_name)
+{
+    if (dev != NULL) {
+        const device_config_t *config = dev->config;
+        while (config && (config->type != CONFIG_END)) {
+            if (config->type == CONFIG_BIOS) {
+                const device_config_bios_t *bios = (const device_config_bios_t *) config->bios;
+
+                /* Go through the ROMs in the device configuration. */
+                while ((bios != NULL) &&
+                       (bios->name != NULL) &&
+                       (bios->internal_name != NULL) &&
+                       (bios->files_no != 0)) {
+                    if (!strcmp(internal_name, bios->internal_name))
+                        return bios;
+                    bios++;
+                }
+
+                /* Unknown value, fall back to the default ROMs. */
+                if (strcmp(internal_name, config->default_string))
+                    return device_get_bios(dev, config->default_string);
+            }
+            config++;
+        }
     }
 
-    /* A NULL device is never available. */
-    return 0;
+    return NULL;
+}
+
+uint8_t
+device_get_bios_type(const device_t *dev, const char *internal_name)
+{
+    const device_config_bios_t *bios = device_get_bios(dev, internal_name);
+    return bios ? bios->bios_type : 0;
+}
+
+uint8_t
+device_get_bios_num_files(const device_t *dev, const char *internal_name)
+{
+    const device_config_bios_t *bios = device_get_bios(dev, internal_name);
+    return bios ? bios->files_no : 0;
+}
+
+uint32_t
+device_get_bios_local(const device_t *dev, const char *internal_name)
+{
+    const device_config_bios_t *bios = device_get_bios(dev, internal_name);
+    return bios ? bios->local : 0;
+}
+
+uint32_t
+device_get_bios_file_size(const device_t *dev, const char *internal_name)
+{
+    const device_config_bios_t *bios = device_get_bios(dev, internal_name);
+    return bios ? bios->size : 0;
 }
 
 const char *
-device_get_bios_file(const device_t *dev, const char *internal_name, int file_no)
+device_get_bios_file(const device_t *dev, const char *internal_name, unsigned int file_no)
 {
-    const device_config_t      *config = NULL;
-    const device_config_bios_t *bios   = NULL;
-
-    if (dev != NULL) {
-        config = dev->config;
-        if (config != NULL) {
-            while (config->type != -1) {
-                if (config->type == CONFIG_BIOS) {
-                    bios = config->bios;
-
-                    /* Go through the ROM's in the device configuration. */
-                    while (bios->files_no != 0) {
-                        if (!strcmp(internal_name, bios->internal_name)) {
-                            if (file_no < bios->files_no)
-                                return bios->files[file_no];
-                            else
-                                return NULL;
-                        }
-                        bios++;
-                    }
-                }
-                config++;
-            }
-        }
-    }
-
-    /* A NULL device is never available. */
-    return (NULL);
+    const device_config_bios_t *bios = device_get_bios(dev, internal_name);
+    return (bios && (file_no < bios->files_no)) ? bios->files[file_no] : NULL;
 }
 
 int
@@ -436,7 +475,7 @@ device_has_config(const device_t *dev)
 
     config = dev->config;
 
-    while (config->type != -1) {
+    while (config->type != CONFIG_END) {
         c++;
         config++;
     }
@@ -444,28 +483,13 @@ device_has_config(const device_t *dev)
     return (c > 0) ? 1 : 0;
 }
 
-int
-device_poll(const device_t *dev)
-{
-    for (uint16_t c = 0; c < DEVICE_MAX; c++) {
-        if (devices[c] != NULL) {
-            if (devices[c] == dev) {
-                if (devices[c]->poll)
-                    return (devices[c]->poll(device_priv[c]));
-            }
-        }
-    }
-
-    return 0;
-}
-
 void
 device_get_name(const device_t *dev, int bus, char *name)
 {
     const char *sbus = NULL;
     const char *fbus;
     char       *tname;
-    char        pbus[8] = { 0 };
+    char        pbus[16] = { 0 };
 
     if (dev == NULL)
         return;
@@ -473,18 +497,45 @@ device_get_name(const device_t *dev, int bus, char *name)
     name[0] = 0x00;
 
     if (bus) {
-        if (dev->flags & DEVICE_ISA)
-            sbus = (dev->flags & DEVICE_AT) ? "ISA16" : "ISA";
+        if ((dev->flags & (DEVICE_SIDECAR | DEVICE_ISA)) ==
+            (DEVICE_SIDECAR | DEVICE_ISA))
+            sbus = "ISA/Sidecar";
+        else if (dev->flags & DEVICE_SIDECAR)
+            sbus = "Sidecar";
+        else if (dev->flags & DEVICE_XT_KBC)
+            sbus = "XT KBC";
+        else if (dev->flags & DEVICE_ISA16)
+            sbus = "ISA16";
+        else if (dev->flags & DEVICE_AT_KBC)
+            sbus = "AT KBC";
+        else if (dev->flags & DEVICE_PS2_KBC)
+            sbus = "PS/2 KBC";
+        else if (dev->flags & DEVICE_ISA)
+            sbus = "ISA";
         else if (dev->flags & DEVICE_CBUS)
             sbus = "C-BUS";
+        else if (dev->flags & DEVICE_PCMCIA)
+            sbus = "PCMCIA";
         else if (dev->flags & DEVICE_MCA)
             sbus = "MCA";
+        else if (dev->flags & DEVICE_MCA32)
+            sbus = "MCA32";
+        else if (dev->flags & DEVICE_HIL)
+            sbus = "HP HIL";
         else if (dev->flags & DEVICE_EISA)
             sbus = "EISA";
+        else if (dev->flags & DEVICE_AT32)
+            sbus = "AT/32";
+        else if (dev->flags & DEVICE_OLB)
+            sbus = "OLB";
         else if (dev->flags & DEVICE_VLB)
             sbus = "VLB";
         else if (dev->flags & DEVICE_PCI)
             sbus = "PCI";
+        else if (dev->flags & DEVICE_CARDBUS)
+            sbus = "CardBus";
+        else if (dev->flags & DEVICE_USB)
+            sbus = "USB";
         else if (dev->flags & DEVICE_AGP)
             sbus = "AGP";
         else if (dev->flags & DEVICE_AC97)
@@ -515,8 +566,7 @@ device_get_name(const device_t *dev, int bus, char *name)
             strcat(pbus, ")");
 
             /* Allocate the temporary device name string and set it to all zeroes. */
-            tname = (char *) malloc(strlen(dev->name) + 1);
-            memset(tname, 0x00, strlen(dev->name) + 1);
+            tname = (char *) calloc(1, strlen(dev->name) + 1);
 
             /* First strip the bus string with parentheses. */
             fbus = strstr(dev->name, pbus);
@@ -533,9 +583,9 @@ device_get_name(const device_t *dev, int bus, char *name)
             fbus = strstr(tname, sbus);
             if (fbus == tname)
                 strcat(name, tname + strlen(sbus) + 1);
-            /* Special case to not strip the "oPCI" from "Ensoniq AudioPCI" or
-               the "-ISA" from "AMD PCnet-ISA". */
-            else if ((fbus == NULL) || (*(fbus - 1) == 'o') || (*(fbus - 1) == '-') || (*(fbus - 2) == 'r'))
+            /* Special case to not strip the "oPCI" from "Ensoniq AudioPCI",
+               the "-ISA" from "AMD PCnet-ISA" or the " PCI" from "CMD PCI-064x". */
+            else if ((fbus == NULL) || (*(fbus - 1) == 'o') || (*(fbus - 1) == '-') || (*(fbus - 2) == 'r') || ((fbus[0] == 'P') && (fbus[1] == 'C') && (fbus[2] == 'I') && (fbus[3] == '-')))
                 strcat(name, tname);
             else {
                 strncat(name, tname, fbus - tname - 1);
@@ -584,259 +634,271 @@ device_get_instance(void)
 }
 
 const char *
-device_get_config_string(const char *s)
+device_get_config_string(const char *str)
 {
-    const device_config_t *c = device_current.dev->config;
+    const char *ret = "";
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_string((char *) device_current.name, (char *) s, (char *) c->default_string));
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-        c++;
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                const char *s = (config_get_string((char *) device_current.name,
+                                 (char *) str, (char *) cfg->default_string));
+                ret = (s == NULL) ? "" : s;
+                break;
+            }
+
+            cfg++;
+        }
     }
 
-    return (NULL);
+    return ret;
 }
 
 int
-device_get_config_int(const char *s)
+device_get_config_int(const char *str)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_int((char *) device_current.name, (char *) s, c->default_int));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_int((char *) device_current.name, (char *) str, cfg->default_int));
 
-        c++;
+            cfg++;
+        }
     }
 
     return 0;
 }
 
 int
-device_get_config_int_ex(const char *s, int def)
+device_get_config_int_ex(const char *str, int def)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_int((char *) device_current.name, (char *) s, def));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_int((char *) device_current.name, (char *) str, def));
 
-        c++;
+            cfg++;
+        }
     }
 
     return def;
 }
 
 int
-device_get_config_hex16(const char *s)
+device_get_config_hex16(const char *str)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_hex16((char *) device_current.name, (char *) s, c->default_int));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_hex16((char *) device_current.name, (char *) str, cfg->default_int));
 
-        c++;
+            cfg++;
+        }
     }
 
     return 0;
 }
 
 int
-device_get_config_hex20(const char *s)
+device_get_config_hex20(const char *str)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_hex20((char *) device_current.name, (char *) s, c->default_int));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_hex20((char *) device_current.name, (char *) str, cfg->default_int));
 
-        c++;
+            cfg++;
+        }
     }
 
     return 0;
 }
 
 int
-device_get_config_mac(const char *s, int def)
+device_get_config_mac(const char *str, int def)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_mac((char *) device_current.name, (char *) s, def));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_mac((char *) device_current.name, (char *) str, def));
 
-        c++;
+            cfg++;
+        }
     }
 
     return def;
 }
 
 void
-device_set_config_int(const char *s, int val)
+device_set_config_int(const char *str, int val)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name)) {
-            config_set_int((char *) device_current.name, (char *) s, val);
-            break;
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                config_set_int((char *) device_current.name, (char *) str, val);
+                break;
+            }
+
+            cfg++;
         }
-
-        c++;
     }
 }
 
 void
-device_set_config_hex16(const char *s, int val)
+device_set_config_hex16(const char *str, int val)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name)) {
-            config_set_hex16((char *) device_current.name, (char *) s, val);
-            break;
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                config_set_hex16((char *) device_current.name, (char *) str, val);
+                break;
+            }
+
+            cfg++;
         }
-
-        c++;
     }
 }
 
 void
-device_set_config_hex20(const char *s, int val)
+device_set_config_hex20(const char *str, int val)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name)) {
-            config_set_hex20((char *) device_current.name, (char *) s, val);
-            break;
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                config_set_hex20((char *) device_current.name, (char *) str, val);
+                break;
+            }
+
+        cfg++;
         }
-
-        c++;
     }
 }
 
 void
-device_set_config_mac(const char *s, int val)
+device_set_config_mac(const char *str, int val)
 {
-    const device_config_t *c = device_current.dev->config;
+    if (device_current.dev != NULL) {
+        const device_config_t *cfg = device_current.dev->config;
 
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name)) {
-            config_set_mac((char *) device_current.name, (char *) s, val);
-            break;
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                config_set_mac((char *) device_current.name, (char *) str, val);
+                break;
+            }
+
+            cfg++;
         }
-
-        c++;
     }
 }
 
 int
-device_is_valid(const device_t *device, int m)
+device_is_valid(const device_t *device, int mch)
 {
-    if (device == NULL)
-        return 1;
+    int ret = 1;
 
-    if ((device->flags & DEVICE_PCJR) && !machine_has_bus(m, MACHINE_BUS_PCJR))
-        return 0;
+    if ((device != NULL) && ((device->flags & DEVICE_BUS) != 0)) {
+        /* Hide PCI devices on machines with only an internal PCI bus. */
+        if ((device->flags & DEVICE_PCI) &&
+            machine_has_flags(mch, MACHINE_PCI_INTERNAL))
+            ret = 0;
+        else
+            ret = machine_has_bus(mch, device->flags & DEVICE_BUS);
+    }
 
-    if ((device->flags & DEVICE_XTKBC) && machine_has_bus(m, MACHINE_BUS_ISA16) && !machine_has_bus(m, MACHINE_BUS_DM_KBC))
-        return 0;
-
-    if ((device->flags & DEVICE_AT) && !machine_has_bus(m, MACHINE_BUS_ISA16))
-        return 0;
-
-    if ((device->flags & DEVICE_ATKBC) && !machine_has_bus(m, MACHINE_BUS_ISA16) && !machine_has_bus(m, MACHINE_BUS_DM_KBC))
-        return 0;
-
-    if ((device->flags & DEVICE_ISA) && !machine_has_bus(m, MACHINE_BUS_ISA))
-        return 0;
-
-    if ((device->flags & DEVICE_CBUS) && !machine_has_bus(m, MACHINE_BUS_CBUS))
-        return 0;
-
-    if ((device->flags & DEVICE_PCMCIA) && !machine_has_bus(m, MACHINE_BUS_PCMCIA))
-        return 0;
-
-    if ((device->flags & DEVICE_MCA) && !machine_has_bus(m, MACHINE_BUS_MCA))
-        return 0;
-
-    if ((device->flags & DEVICE_HIL) && !machine_has_bus(m, MACHINE_BUS_HIL))
-        return 0;
-
-    if ((device->flags & DEVICE_EISA) && !machine_has_bus(m, MACHINE_BUS_EISA))
-        return 0;
-
-    if ((device->flags & DEVICE_OLB) && !machine_has_bus(m, MACHINE_BUS_OLB))
-        return 0;
-
-    if ((device->flags & DEVICE_VLB) && !machine_has_bus(m, MACHINE_BUS_VLB))
-        return 0;
-
-    if ((device->flags & DEVICE_PCI) && !machine_has_bus(m, MACHINE_BUS_PCI))
-        return 0;
-
-    if ((device->flags & DEVICE_CARDBUS) && !machine_has_bus(m, MACHINE_BUS_CARDBUS))
-        return 0;
-
-    if ((device->flags & DEVICE_USB) && !machine_has_bus(m, MACHINE_BUS_USB))
-        return 0;
-
-    if ((device->flags & DEVICE_AGP) && !machine_has_bus(m, MACHINE_BUS_AGP))
-        return 0;
-
-    if ((device->flags & DEVICE_PS2) && !machine_has_bus(m, MACHINE_BUS_PS2_PORTS))
-        return 0;
-
-    if ((device->flags & DEVICE_AC97) && !machine_has_bus(m, MACHINE_BUS_AC97))
-        return 0;
-
-    return 1;
+    return ret;
 }
 
 int
-machine_get_config_int(char *s)
+machine_get_config_int(char *str)
 {
-    const device_t        *d = machine_get_device(machine);
-    const device_config_t *c;
+    const device_t *dev = machine_get_device(machine);
 
-    if (d == NULL)
-        return 0;
+    if (dev != NULL) {
+        const device_config_t *cfg = dev->config;
 
-    c = d->config;
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_int((char *) d->name, s, c->default_int));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name))
+                return (config_get_int((char *) dev->name, str, cfg->default_int));
 
-        c++;
+            cfg++;
+        }
     }
-
     return 0;
 }
 
-char *
-machine_get_config_string(char *s)
+const char *
+machine_get_config_string(char *str)
 {
-    const device_t        *d = machine_get_device(machine);
-    const device_config_t *c;
+    const device_t *dev = machine_get_device(machine);
+    const char     *ret = "";
 
-    if (d == NULL)
-        return 0;
+    if (dev != NULL) {
+        const device_config_t *cfg = dev->config;
 
-    c = d->config;
-    while (c && c->type != -1) {
-        if (!strcmp(s, c->name))
-            return (config_get_string((char *) d->name, s, (char *) c->default_string));
+        while ((cfg != NULL) && (cfg->type != CONFIG_END)) {
+            if (!strcmp(str, cfg->name)) {
+                const char *s = config_get_string((char *) dev->name, str,
+                                                  (char *) cfg->default_string);
+                ret = (s == NULL) ? "" : s;
+                break;
+            }
 
-        c++;
+            cfg++;
+        }
     }
 
-    return NULL;
+    return ret;
 }
 
-const device_t*
+int
+machine_device_available(const device_t *dev)
+{
+    if (dev != NULL) {
+        const device_config_t *config = dev->config;
+        if ((config != NULL) && (config->type == CONFIG_BIOS)) {
+            int roms_present = 0;
+            const device_config_bios_t *bios = (const device_config_bios_t *) config->bios;
+
+            /* Go through the ROM's in the device configuration. */
+            while ((bios != NULL) &&
+                   (bios->name != NULL) &&
+                   (bios->internal_name != NULL) &&
+                   (bios->files_no != 0)) {
+                int i = 0;
+                for (uint8_t bf = 0; bf < bios->files_no; bf++)
+                    i += !!rom_present(bios->files[bf]);
+                if (i == bios->files_no)
+                    roms_present++;
+                bios++;
+            }
+
+            return (roms_present ? -1 : -2);
+        }
+    }
+
+    /* NULL device or no CONFIG_BIOS field, return 0. */
+    return 0;
+}
+
+const device_t *
 device_context_get_device(void)
 {
     return device_current.dev;
@@ -850,7 +912,7 @@ const device_t device_none = {
     .init          = NULL,
     .close         = NULL,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -864,7 +926,7 @@ const device_t device_internal = {
     .init          = NULL,
     .close         = NULL,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
