@@ -8,8 +8,6 @@
  *
  *          Implementation of ISA Plug and Play.
  *
- *
- *
  * Authors: Miran Grca, <mgrca8@gmail.com>
  *          RichardG, <richardg867@gmail.com>
  *
@@ -27,6 +25,7 @@
 #include <86box/io.h>
 #include <86box/isapnp.h>
 #include <86box/plat_unused.h>
+#include "cpu.h"
 
 #define CHECK_CURRENT_LD()                                  \
     if (!ld) {                                              \
@@ -44,6 +43,11 @@ const uint8_t         isapnp_init_key[32] = { 0x6A, 0xB5, 0xDA, 0xED, 0xF6, 0xFB
                                               0xDF, 0x6F, 0x37, 0x1B, 0x0D, 0x86, 0xC3, 0x61,
                                               0xB0, 0x58, 0x2C, 0x16, 0x8B, 0x45, 0xA2, 0xD1,
                                               0xE8, 0x74, 0x3A, 0x9D, 0xCE, 0xE7, 0x73, 0x39 };
+/* Required for the RTL8019AS. */
+const uint8_t         isapnp_init_key2[32] = { 0xDA, 0x6D, 0x36, 0x1B, 0x8D, 0x46, 0x23, 0x91,
+                                               0x48, 0xA4, 0xD2, 0x69, 0x34, 0x9A, 0x4D, 0x26,
+                                               0x13, 0x89, 0x44, 0xA2, 0x51, 0x28, 0x94, 0xCA,
+                                               0x65, 0x32, 0x19, 0x0C, 0x86, 0x43, 0xA1, 0x50 };
 static const device_t isapnp_device;
 
 #ifdef ENABLE_ISAPNP_LOG
@@ -73,6 +77,7 @@ enum {
 
 typedef struct _isapnp_device_ {
     uint8_t                       number;
+    uint8_t                       defs[256];
     uint8_t                       regs[256];
     uint8_t                       mem_upperlimit;
     uint8_t                       irq_types;
@@ -87,11 +92,15 @@ typedef struct _isapnp_card_ {
     uint8_t  enable;
     uint8_t  state;
     uint8_t  csn;
+    uint8_t  csnsav;
     uint8_t  ld;
     uint8_t  id_checksum;
     uint8_t  serial_read;
     uint8_t  serial_read_pair;
     uint8_t  serial_read_pos;
+    uint8_t  is_rt;
+    uint8_t  normal;
+    uint8_t  multiple_lds;
     uint8_t *rom;
     uint16_t rom_pos;
     uint16_t rom_size;
@@ -111,6 +120,7 @@ typedef struct _isapnp_card_ {
 
 typedef struct {
     uint8_t  in_isolation;
+    uint8_t  using_key2;
     uint8_t  reg;
     uint8_t  key_pos : 5;
     uint16_t read_data_addr;
@@ -168,7 +178,7 @@ isapnp_device_config_changed(isapnp_card_t *card, isapnp_device_t *ld)
     }
     for (uint8_t i = 0; i < 8; i++) {
         reg_base = 0x60 + (2 * i);
-        if (ld->regs[0x31] & 0x02)
+        if (!(ld->regs[0x30] & 0x01) && (ld->regs[0x31] & 0x02))
             card->config.io[i].base = 0; /* let us handle I/O range check reads */
         else
             card->config.io[i].base = (ld->regs[reg_base] << 8) | ld->regs[reg_base + 1];
@@ -176,7 +186,7 @@ isapnp_device_config_changed(isapnp_card_t *card, isapnp_device_t *ld)
     for (uint8_t i = 0; i < 2; i++) {
         reg_base                  = 0x70 + (2 * i);
         card->config.irq[i].irq   = ld->regs[reg_base];
-        card->config.irq[i].level = ld->regs[reg_base + 1] & 0x02;
+        card->config.irq[i].level = !!(ld->regs[reg_base + 1] & 0x02);
         card->config.irq[i].type  = ld->regs[reg_base + 1] & 0x01;
     }
     for (uint8_t i = 0; i < 2; i++) {
@@ -258,17 +268,29 @@ isapnp_reset_ld_regs(isapnp_device_t *ld)
     /* Set the default IRQ type bits. */
     for (uint8_t i = 0; i < 2; i++) {
         if (ld->irq_types & (0x1 << (4 * i)))
-            ld->regs[0x70 + (2 * i)] = 0x02;
+            ld->regs[0x71 + (2 * i)] = 0x02;
         else if (ld->irq_types & (0x2 << (4 * i)))
-            ld->regs[0x70 + (2 * i)] = 0x00;
+            ld->regs[0x71 + (2 * i)] = 0x00;
         else if (ld->irq_types & (0x4 << (4 * i)))
-            ld->regs[0x70 + (2 * i)] = 0x03;
+            ld->regs[0x71 + (2 * i)] = 0x03;
         else if (ld->irq_types & (0x8 << (4 * i)))
-            ld->regs[0x70 + (2 * i)] = 0x01;
+            ld->regs[0x71 + (2 * i)] = 0x01;
     }
 
     /* Reset configuration registers to match the default configuration. */
     isapnp_reset_ld_config(ld);
+
+    if (ld->defs[0x30] != 0x00)
+        ld->regs[0x30] = ld->defs[0x30];
+
+    if (ld->defs[0x60] != 0x00)
+        ld->regs[0x60] = ld->defs[0x60];
+
+    if (ld->defs[0x61] != 0x00)
+        ld->regs[0x61] = ld->defs[0x61];
+
+    if (ld->defs[0x70] != 0x00)
+        ld->regs[0x70] = ld->defs[0x70];
 }
 
 static uint8_t
@@ -290,7 +312,7 @@ isapnp_read_common(isapnp_t *dev, isapnp_card_t *card, isapnp_device_t *ld, uint
         case 0x01: /* Serial Isolation */
             card = dev->first_card;
             while (card) {
-                if (card->enable && card->rom && (card->state == PNP_STATE_ISOLATION))
+                if (card->enable && (card->rom != NULL) && (card->state == PNP_STATE_ISOLATION))
                     break;
                 card = card->next;
             }
@@ -376,6 +398,7 @@ vendor_defined:
 
         default:
             if (reg >= 0x30) {
+                CHECK_CURRENT_CARD();
                 CHECK_CURRENT_LD();
                 isapnp_log("ISAPnP: Read register %02X from CSN %02X device %02X\n", reg, card->csn, ld->number);
                 ret = ld->regs[reg];
@@ -399,7 +422,8 @@ isapnp_read_data(UNUSED(uint16_t addr), void *priv)
         card = card->next;
     }
 
-    isapnp_log("ISAPnP: read_data() => ");
+    // isapnp_log("ISAPnP: read_data() => ");
+    isapnp_log("[%04X:%08X] ISAPnP: read_data() => ", CS, cpu_state.pc);
     return isapnp_read_common(dev, card, dev->current_ld, dev->reg);
 }
 
@@ -434,26 +458,60 @@ isapnp_write_addr(UNUSED(uint16_t addr), uint8_t val, void *priv)
 
     if (card->state == PNP_STATE_WAIT_FOR_KEY) { /* checking only the first card should be fine */
         /* Check written value against LFSR key. */
-        if (val == isapnp_init_key[dev->key_pos]) {
+        if ((val == isapnp_init_key[dev->key_pos]) || (val == isapnp_init_key2[dev->key_pos])) {
+            dev->using_key2 = (val == isapnp_init_key2[dev->key_pos]);
             dev->key_pos++;
             if (!dev->key_pos) {
                 isapnp_log("ISAPnP: Key unlocked, putting cards to SLEEP\n");
                 while (card) {
-                    if (card->enable && (card->enable != ISAPNP_CARD_NO_KEY) && (card->state == PNP_STATE_WAIT_FOR_KEY))
+                    int match_rt = (dev->using_key2 && card->is_rt);
+                    int match_normal = (!dev->using_key2 && card->normal);
+                    if (card->enable && (match_rt || match_normal) &&
+                        (card->enable != ISAPNP_CARD_NO_KEY) && (card->state == PNP_STATE_WAIT_FOR_KEY))
                         card->state = PNP_STATE_SLEEP;
                     card = card->next;
                 }
             }
-        } else {
+        } else
             dev->key_pos = 0;
-        }
+    }
+}
+
+static void
+isapnp_ld_io_remove(isapnp_device_t *ld)
+{
+    uint16_t io_addr;
+
+    for (uint8_t i = 0; i < 8; i++) {
+        if (!ld->io_len[i])
+            continue;
+
+        io_addr = (ld->regs[0x60 + (2 * i)] << 8) | ld->regs[0x61 + (2 * i)];
+
+        io_removehandler(io_addr, ld->io_len[i], isapnp_read_rangecheck, NULL, NULL, NULL, NULL, NULL, ld);
+    }
+}
+
+static void
+isapnp_ld_io_set(isapnp_device_t *ld)
+{
+    uint16_t io_addr;
+
+    for (uint8_t i = 0; i < 8; i++) {
+        if (!ld->io_len[i])
+            continue;
+
+        io_addr = (ld->regs[0x60 + (2 * i)] << 8) | ld->regs[0x61 + (2 * i)];
+        int ior = !(ld->regs[0x30] & 0x01) && (ld->regs[0x31] & 0x02);
+
+        if (ior)
+            io_sethandler(io_addr, ld->io_len[i], isapnp_read_rangecheck, NULL, NULL, NULL, NULL, NULL, ld);
     }
 }
 
 static void
 isapnp_write_common(isapnp_t *dev, isapnp_card_t *card, isapnp_device_t *ld, uint8_t reg, uint8_t val)
 {
-    uint16_t io_addr;
     uint16_t reset_cards = 0;
 
     isapnp_log("ISAPnP: write_common(%02X, %02X)\n", reg, val);
@@ -495,6 +553,7 @@ isapnp_write_common(isapnp_t *dev, isapnp_card_t *card, isapnp_device_t *ld, uin
                     card->state = PNP_STATE_WAIT_FOR_KEY;
                     card        = card->next;
                 }
+                dev->key_pos = 0;
             }
             if (val & 0x04) {
                 isapnp_log("ISAPnP: Reset CSN\n");
@@ -515,8 +574,22 @@ isapnp_write_common(isapnp_t *dev, isapnp_card_t *card, isapnp_device_t *ld, uin
                 if (card->csn == val) {
                     card->rom_pos     = 0;
                     card->id_checksum = isapnp_init_key[0];
-                    if (card->state == PNP_STATE_SLEEP)
+                    if (card->state == PNP_STATE_SLEEP) {
                         card->state = (val == 0) ? PNP_STATE_ISOLATION : PNP_STATE_CONFIG;
+
+                        if (!card->multiple_lds) {
+                            ld = card->first_ld;
+                            while (ld) {
+                                if (ld->number == 0x00) {
+                                    isapnp_log("ISAPnP: Select CSN %02X device 00\n", card->csn);
+                                    dev->current_ld_card = card;
+                                    dev->current_ld      = ld;
+                                    break;
+                                }
+                                ld = ld->next;
+                            }
+                        }
+                    }
                 } else
                     card->state = PNP_STATE_SLEEP;
 
@@ -537,56 +610,47 @@ isapnp_write_common(isapnp_t *dev, isapnp_card_t *card, isapnp_device_t *ld, uin
             break;
 
         case 0x07: /* Logical Device Number */
-            CHECK_CURRENT_CARD();
+            if (card->multiple_lds) {
+                CHECK_CURRENT_CARD();
 
-            card->ld = val;
-            ld = card->first_ld;
-            while (ld) {
-                if (ld->number == val) {
-                    isapnp_log("ISAPnP: Select CSN %02X device %02X\n", card->csn, val);
-                    dev->current_ld_card = card;
-                    dev->current_ld      = ld;
-                    break;
+                card->ld = val;
+                ld = card->first_ld;
+                while (ld) {
+                    if (ld->number == val) {
+                        isapnp_log("ISAPnP: Select CSN %02X device %02X\n", card->csn, val);
+                        dev->current_ld_card = card;
+                        dev->current_ld      = ld;
+                        break;
+                    }
+                    ld = ld->next;
                 }
-                ld = ld->next;
-            }
 
-            if (!ld) {
-                isapnp_log("ISAPnP: CSN %02X has no device %02X, creating one\n", card->csn, val);
-                dev->current_ld_card    = card;
-                dev->current_ld         = isapnp_create_ld(card);
-                dev->current_ld->number = val;
+                if (!ld) {
+                    isapnp_log("ISAPnP: CSN %02X has no device %02X, creating one\n", card->csn, val);
+                    dev->current_ld_card    = card;
+                    dev->current_ld         = isapnp_create_ld(card);
+                    dev->current_ld->number = val;
+                }
             }
-
             break;
 
         case 0x30: /* Activate */
-            CHECK_CURRENT_LD();
-
-            isapnp_log("ISAPnP: %sctivate CSN %02X device %02X\n", (val & 0x01) ? "A" : "Dea", card->csn, ld->number);
-
-            ld->regs[reg] = val & 0x01;
-            isapnp_device_config_changed(card, ld);
-
-            break;
-
         case 0x31: /* I/O Range Check */
             CHECK_CURRENT_LD();
 
-            for (uint8_t i = 0; i < 8; i++) {
-                if (!ld->io_len[i])
-                    continue;
+            isapnp_log("ISAPnP: %sctivate CSN %02X device %02X\n", (ld->regs[0x30] & 0x01) ? "A" : "Dea",
+                       card->csn, ld->number);
 
-                io_addr = (ld->regs[0x60 + (2 * i)] << 8) | ld->regs[0x61 + (2 * i)];
-                if (ld->regs[reg] & 0x02)
-                    io_removehandler(io_addr, ld->io_len[i], isapnp_read_rangecheck, NULL, NULL, NULL, NULL, NULL, ld);
-                if (val & 0x02)
-                    io_sethandler(io_addr, ld->io_len[i], isapnp_read_rangecheck, NULL, NULL, NULL, NULL, NULL, ld);
-            }
+            isapnp_ld_io_remove(ld);
+
+            if (reg == 0x30)
+                ld->regs[reg] = val & 0x01;
+            else
+                ld->regs[reg] = val & 0x03;
+
+            isapnp_ld_io_set(ld);
 
-            ld->regs[reg] = val & 0x03;
             isapnp_device_config_changed(card, ld);
-
             break;
 
         case 0x20 ... 0x2f:
@@ -603,6 +667,7 @@ vendor_defined:
 
         default:
             if (reg >= 0x40) {
+                CHECK_CURRENT_CARD();
                 CHECK_CURRENT_LD();
                 isapnp_log("ISAPnP: Write %02X to register %02X on CSN %02X device %02X\n", val, reg, card->csn, ld->number);
 
@@ -651,7 +716,9 @@ vendor_defined:
                         break;
                 }
 
+                isapnp_ld_io_remove(ld);
                 ld->regs[reg] = val;
+                isapnp_ld_io_set(ld);
                 isapnp_device_config_changed(card, ld);
             }
             break;
@@ -672,7 +739,8 @@ isapnp_write_data(UNUSED(uint16_t addr), uint8_t val, void *priv)
         }
     }
 
-    isapnp_log("ISAPnP: write_data(%02X) => ", val);
+    // isapnp_log("ISAPnP: write_data(%02X) => ", val);
+    isapnp_log("[%04X:%08X] ISAPnP: write_data(%02X) => ", CS, cpu_state.pc, val);
     isapnp_write_common(dev, card, dev->current_ld, dev->reg, val);
 }
 
@@ -731,11 +799,13 @@ isapnp_add_card(uint8_t *rom, uint16_t rom_size,
     isapnp_card_t *card = (isapnp_card_t *) calloc(1, sizeof(isapnp_card_t));
 
     card->enable           = 1;
+    card->normal           = 1;
     card->priv             = priv;
     card->config_changed   = config_changed;
     card->csn_changed      = csn_changed;
     card->read_vendor_reg  = read_vendor_reg;
     card->write_vendor_reg = write_vendor_reg;
+    card->multiple_lds     = 1;
 
     if (!dev->first_card) {
         dev->first_card = card;
@@ -815,22 +885,22 @@ isapnp_update_card_rom(void *priv, uint8_t *rom, uint16_t rom_size)
                         isapnp_log("ISAPnP: >>%s Memory range %d with %d bytes at %06X-%06X to %06X-%06X, align %d",
                                    /* %s   */ in_df ? ">" : "",
                                    /* %d   */ mem_range,
-                                   /* %d   */ *((uint16_t *) &card->rom[i + 8]),
-                                   /* %06X */ *((uint16_t *) &card->rom[i + 4]) << 8,
+                                   /* %d   */ AS_U16(card->rom[i + 8]),
+                                   /* %06X */ AS_U16(card->rom[i + 4]) << 8,
                                    /* %06X */ ((card->rom[i + 3] & 0x4) ?
                                               /* High address. */
-                                              (*((uint16_t *) &card->rom[i + 10]) << 8) :
+                                              (AS_U16(card->rom[i + 10]) << 8) :
                                               /* Range. */
-                                              (*((uint16_t *) &card->rom[i + 4]) << 8)) +
-                                              (*((uint16_t *) &card->rom[i + 10]) << 8),
-                                   /* %06X */ *((uint16_t *) &card->rom[i + 6]) << 8,
+                                              (AS_U16(card->rom[i + 4]) << 8)) +
+                                              (AS_U16(card->rom[i + 10]) << 8),
+                                   /* %06X */ AS_U16(card->rom[i + 6]) << 8,
                                    /* %06X */ ((card->rom[i + 3] & 0x4) ?
                                               /* High address. */
-                                              (*((uint16_t *) &card->rom[i + 10]) << 8) :
+                                              (AS_U16(card->rom[i + 10]) << 8) :
                                               /* Range. */
-                                              (*((uint16_t *) &card->rom[i + 6]) << 8)) +
-                                              (*((uint16_t *) &card->rom[i + 10]) << 8),
-                                   /* %d   */ *((uint16_t *) &card->rom[i + 8]));
+                                              (AS_U16(card->rom[i + 6]) << 8)) +
+                                              (AS_U16(card->rom[i + 10]) << 8),
+                                   /* %d   */ AS_U16(card->rom[i + 8]));
                         res = 1 << mem_range;
                         mem_range++;
                     } else {
@@ -847,22 +917,22 @@ isapnp_update_card_rom(void *priv, uint8_t *rom, uint16_t rom_size)
                         isapnp_log("ISAPnP: >>%s 32-bit memory range %d with %d bytes at %08X-%08X, align %d",
                                    /* %s   */ in_df ? ">" : "",
                                    /* %d   */ mem_range_32,
-                                   /* %d   */ *((uint32_t *) &card->rom[i + 12]),
-                                   /* %08X */ *((uint32_t *) &card->rom[i + 4]),
+                                   /* %d   */ AS_U32(card->rom[i + 12]),
+                                   /* %08X */ AS_U32(card->rom[i + 4]),
                                    /* %08X */ ((card->rom[i + 3] & 0x4) ?
                                               /* High address. */
-                                              *((uint32_t *) &card->rom[i + 16]) :
+                                              AS_U32(card->rom[i + 16]) :
                                               /* Range. */
-                                              *((uint32_t *) &card->rom[i + 4])) +
-                                              *((uint32_t *) &card->rom[i + 16]),
-                                   /* %08X */ *((uint32_t *) &card->rom[i + 8]),
+                                              AS_U32(card->rom[i + 4])) +
+                                              AS_U32(card->rom[i + 16]),
+                                   /* %08X */ AS_U32(card->rom[i + 8]),
                                    /* %08X */ ((card->rom[i + 3] & 0x4) ?
                                               /* High address. */
-                                              *((uint32_t *) &card->rom[i + 16]) :
+                                              AS_U32(card->rom[i + 16]) :
                                               /* Range. */
-                                              *((uint32_t *) &card->rom[i + 8])) +
-                                              *((uint32_t *) &card->rom[i + 16]),
-                                   /* %d   */ *((uint32_t *) &card->rom[i + 12]));
+                                              AS_U32(card->rom[i + 8])) +
+                                              AS_U32(card->rom[i + 16]),
+                                   /* %d   */ AS_U32(card->rom[i + 12]));
                         res = 1 << (4 + mem_range_32);
                         mem_range_32++;
                     }
@@ -972,7 +1042,7 @@ isapnp_update_card_rom(void *priv, uint8_t *rom, uint16_t rom_size)
                     else            /* specific */
                         res = card->rom[i + 3] & 0x0f;
 
-                    isapnp_log("ISAPnP: >>%s IRQ index %d with mask %04X, types %01X\n", in_df ? ">" : "", irq, *((uint16_t *) &card->rom[i + 1]), res);
+                    isapnp_log("ISAPnP: >>%s IRQ index %d with mask %04X, types %01X\n", in_df ? ">" : "", irq, AS_U16(card->rom[i + 1]), res);
 
                     ld->irq_types &= ~(0x0f << (4 * irq));
                     ld->irq_types |= res << (4 * irq);
@@ -1038,7 +1108,7 @@ isapnp_update_card_rom(void *priv, uint8_t *rom, uint16_t rom_size)
                         break;
                     }
 
-                    isapnp_log("ISAPnP: >>%s I/O range %d with %d ports at %04X-%04X, align %d, %d-bit decode\n", in_df ? ">" : "", io, card->rom[i + 7], *((uint16_t *) &card->rom[i + 2]), *((uint16_t *) &card->rom[i + 4]), card->rom[i + 6], (card->rom[i + 1] & 0x01) ? 16 : 10);
+                    isapnp_log("ISAPnP: >>%s I/O range %d with %d ports at %04X-%04X, align %d, %d-bit decode\n", in_df ? ">" : "", io, card->rom[i + 7], AS_U16(card->rom[i + 2]), AS_U16(card->rom[i + 4]), card->rom[i + 6], (card->rom[i + 1] & 0x01) ? 16 : 10);
 
                     if (card->rom[i + 1] & 0x01)
                         ld->io_16bit |= 1 << io;
@@ -1063,7 +1133,7 @@ isapnp_update_card_rom(void *priv, uint8_t *rom, uint16_t rom_size)
                         break;
                     }
 
-                    isapnp_log("ISAPnP: >>%s Fixed I/O range %d with %d ports at %04X\n", in_df ? ">" : "", io, card->rom[i + 3], *((uint16_t *) &card->rom[i + 1]));
+                    isapnp_log("ISAPnP: >>%s Fixed I/O range %d with %d ports at %04X\n", in_df ? ">" : "", io, card->rom[i + 3], AS_U16(card->rom[i + 1]));
 
                     /* Fixed I/O port ranges of this kind are always 10-bit. */
                     ld->io_16bit &= ~(1 << io);
@@ -1117,6 +1187,8 @@ isapnp_enable_card(void *priv, uint8_t enable)
             /* Enable or disable the card. */
             if (!!enable ^ !!card->enable)
                 card->state = (enable == ISAPNP_CARD_FORCE_CONFIG) ? PNP_STATE_CONFIG : PNP_STATE_WAIT_FOR_KEY;
+            if (enable == ISAPNP_CARD_FORCE_SLEEP)
+                card->state = PNP_STATE_SLEEP;
             int old_enable = card->enable;
             card->enable = enable;
 
@@ -1163,6 +1235,63 @@ isapnp_read_reg(void *priv, uint8_t ldn, uint8_t reg)
     return isapnp_read_common(device_get_priv(&isapnp_device), card, ld, reg);
 }
 
+void
+isapnp_set_rt(void *priv, uint8_t is_rt)
+{
+    isapnp_card_t *card = (isapnp_card_t *) priv;
+
+    card->is_rt = is_rt;
+}
+
+void
+isapnp_set_normal(void *priv, uint8_t normal)
+{
+    isapnp_card_t *card = (isapnp_card_t *) priv;
+
+    card->normal = normal;
+}
+
+void
+isapnp_activate(void *priv, uint16_t base, uint8_t irq, int active)
+{
+    isapnp_card_t   *card = (isapnp_card_t *) priv;
+    isapnp_device_t *ld   = card->first_ld;
+
+    while (ld) {
+        if (ld->number == 0x00)
+            break;
+        ld = ld->next;
+    }
+
+    if (ld != NULL) {
+        ld->defs[0x30] = active;
+        ld->defs[0x60] = base >> 8;
+        if (!(ld->io_16bit & (1 << ((0x60 >> 1) & 0x07))))
+            ld->defs[0x60] &= 0x03;
+        ld->defs[0x61] = base & 0xff;
+        ld->defs[0x70] = irq;
+
+        isapnp_reset_ld_regs(ld);
+    }
+}
+
+void
+isapnp_set_single_ld(void *priv)
+{
+    isapnp_card_t   *card = (isapnp_card_t *) priv;
+
+    card->multiple_lds    = 0;
+    card->ld              = 0x00;
+}
+
+uint8_t *
+isapnp_get_csnsav(void *priv)
+{
+    isapnp_card_t *card = (isapnp_card_t *) priv;
+
+    return &card->csnsav;
+}
+
 void
 isapnp_write_reg(void *priv, uint8_t ldn, uint8_t reg, uint8_t val)
 {
