@@ -8,21 +8,23 @@
  *
  *          Implementation of the floppy drive emulation.
  *
- *
- *
  * Authors: Sarah Walker, <https://pcem-emulator.co.uk/>
  *          Miran Grca, <mgrca8@gmail.com>
  *          Fred N. van Kempen, <decwiz@yahoo.com>
+ *          Toni Riikonen, <riikonen.toni@gmail.com>
  *
  *          Copyright 2008-2019 Sarah Walker.
  *          Copyright 2016-2019 Miran Grca.
  *          Copyright 2018-2019 Fred N. van Kempen.
+ *          Copyright 2025 Toni Riikonen.
  */
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
 #include <wchar.h>
+#include <stdlib.h>
+
 #define HAVE_STDARG_H
 #include <86box/86box.h>
 #include <86box/timer.h>
@@ -39,6 +41,7 @@
 #include <86box/fdd_mfm.h>
 #include <86box/fdd_td0.h>
 #include <86box/fdc.h>
+#include <86box/fdd_audio.h>
 
 /* Flags:
    Bit  0:  300 rpm supported;
@@ -76,19 +79,44 @@ typedef struct fdd_t {
 
 fdd_t fdd[FDD_NUM];
 
+enum {
+    FDD_OP_NONE = 0,
+    FDD_OP_READ,
+    FDD_OP_WRITE,
+    FDD_OP_COMPARE,
+    FDD_OP_READADDR,
+    FDD_OP_FORMAT
+};
+
+typedef struct fdd_pending_op_t {
+    int     pending;
+    int     op;
+    int     sector;
+    int     track;
+    int     side;
+    int     density;
+    int     sector_size;
+    uint8_t fill;
+} fdd_pending_op_t;
+
+static fdd_pending_op_t fdd_pending[FDD_NUM];
+
 char  floppyfns[FDD_NUM][512];
 char *fdd_image_history[FDD_NUM][FLOPPY_IMAGE_HISTORY];
 
 pc_timer_t fdd_poll_time[FDD_NUM];
+pc_timer_t fdd_seek_timer[FDD_NUM];
+int        fdd_seek_in_progress[FDD_NUM] = { 0, 0, 0, 0 };
 
 static int fdd_notfound = 0;
 static int driveloaders[FDD_NUM];
+static int fdd_audio_profile[FDD_NUM] = { 0 };
 
 int writeprot[FDD_NUM];
 int fwriteprot[FDD_NUM];
 int fdd_changed[FDD_NUM];
 int ui_writeprot[FDD_NUM] = { 0, 0, 0, 0 };
-int drive_empty[FDD_NUM] = { 1, 1, 1, 1 };
+int drive_empty[FDD_NUM]  = { 1, 1, 1, 1 };
 
 DRIVE drives[FDD_NUM];
 
@@ -101,43 +129,43 @@ d86f_handler_t d86f_handler[FDD_NUM];
 static const struct
 {
     const char *ext;
-    void        (*load)(int drive, char *fn);
-    void        (*close)(int drive);
-    int         size;
+    void (*load)(int drive, char *fn);
+    void (*close)(int drive);
+    int size;
 } loaders[] = {
-    { "001",  img_load,  img_close,  -1},
-    { "002",  img_load,  img_close,  -1},
-    { "003",  img_load,  img_close,  -1},
-    { "004",  img_load,  img_close,  -1},
-    { "005",  img_load,  img_close,  -1},
-    { "006",  img_load,  img_close,  -1},
-    { "007",  img_load,  img_close,  -1},
-    { "008",  img_load,  img_close,  -1},
-    { "009",  img_load,  img_close,  -1},
-    { "010",  img_load,  img_close,  -1},
-    { "12",   img_load,  img_close,  -1},
-    { "144",  img_load,  img_close,  -1},
-    { "360",  img_load,  img_close,  -1},
-    { "720",  img_load,  img_close,  -1},
-    { "86F",  d86f_load, d86f_close, -1},
-    { "BIN",  img_load,  img_close,  -1},
-    { "CQ",   img_load,  img_close,  -1},
-    { "CQM",  img_load,  img_close,  -1},
-    { "DDI",  img_load,  img_close,  -1},
-    { "DSK",  img_load,  img_close,  -1},
-    { "FDI",  fdi_load,  fdi_close,  -1},
-    { "FDF",  img_load,  img_close,  -1},
-    { "FLP",  img_load,  img_close,  -1},
-    { "HDM",  img_load,  img_close,  -1},
-    { "IMA",  img_load,  img_close,  -1},
-    { "IMD",  imd_load,  imd_close,  -1},
-    { "IMG",  img_load,  img_close,  -1},
-    { "JSON", pcjs_load, pcjs_close, -1},
-    { "MFM",  mfm_load,  mfm_close,  -1},
-    { "TD0",  td0_load,  td0_close,  -1},
-    { "VFD",  img_load,  img_close,  -1},
-    { "XDF",  img_load,  img_close,  -1},
-    { 0,      0,         0,          0 }
+    { "001",  img_load,  img_close,  -1 },
+    { "002",  img_load,  img_close,  -1 },
+    { "003",  img_load,  img_close,  -1 },
+    { "004",  img_load,  img_close,  -1 },
+    { "005",  img_load,  img_close,  -1 },
+    { "006",  img_load,  img_close,  -1 },
+    { "007",  img_load,  img_close,  -1 },
+    { "008",  img_load,  img_close,  -1 },
+    { "009",  img_load,  img_close,  -1 },
+    { "010",  img_load,  img_close,  -1 },
+    { "12",   img_load,  img_close,  -1 },
+    { "144",  img_load,  img_close,  -1 },
+    { "360",  img_load,  img_close,  -1 },
+    { "720",  img_load,  img_close,  -1 },
+    { "86F",  d86f_load, d86f_close, -1 },
+    { "BIN",  img_load,  img_close,  -1 },
+    { "CQ",   img_load,  img_close,  -1 },
+    { "CQM",  img_load,  img_close,  -1 },
+    { "DDI",  img_load,  img_close,  -1 },
+    { "DSK",  img_load,  img_close,  -1 },
+    { "FDI",  fdi_load,  fdi_close,  -1 },
+    { "FDF",  img_load,  img_close,  -1 },
+    { "FLP",  img_load,  img_close,  -1 },
+    { "HDM",  img_load,  img_close,  -1 },
+    { "IMA",  img_load,  img_close,  -1 },
+    { "IMD",  imd_load,  imd_close,  -1 },
+    { "IMG",  img_load,  img_close,  -1 },
+    { "JSON", pcjs_load, pcjs_close, -1 },
+    { "MFM",  mfm_load,  mfm_close,  -1 },
+    { "TD0",  td0_load,  td0_close,  -1 },
+    { "VFD",  img_load,  img_close,  -1 },
+    { "XDF",  img_load,  img_close,  -1 },
+    { 0,      0,         0,          0  }
 };
 
 static const struct {
@@ -147,35 +175,35 @@ static const struct {
     const char *internal_name;
 } drive_types[] = {
     /* None */
-    { 0, 0, "None", "none" },
+    { 0,  0,                                                                                                       "None",                    "none"            },
     /* 5.25" 1DD */
-    { 43, FLAG_RPM_300 | FLAG_525 | FLAG_HOLE0, "5.25\" 180k", "525_1dd" },
+    { 43, FLAG_RPM_300 | FLAG_525 | FLAG_HOLE0,                                                                    "5.25\" 180k",             "525_1dd"         },
     /* 5.25" DD */
-    { 43, FLAG_RPM_300 | FLAG_525 | FLAG_DS | FLAG_HOLE0, "5.25\" 360k", "525_2dd" },
+    { 43, FLAG_RPM_300 | FLAG_525 | FLAG_DS | FLAG_HOLE0,                                                          "5.25\" 360k",             "525_2dd"         },
     /* 5.25" QD */
-    { 86, FLAG_RPM_300 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_DOUBLE_STEP, "5.25\" 720k", "525_2qd" },
+    { 86, FLAG_RPM_300 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_DOUBLE_STEP,                                       "5.25\" 720k",             "525_2qd"         },
     /* 5.25" HD */
-    { 86, FLAG_RPM_360 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_PS2, "5.25\" 1.2M", "525_2hd" },
+    { 86, FLAG_RPM_360 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_PS2,               "5.25\" 1.2M",             "525_2hd"         },
     /* 5.25" HD Dual RPM */
-    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP, "5.25\" 1.2M 300/360 RPM", "525_2hd_dualrpm" },
+    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_525 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP,           "5.25\" 1.2M 300/360 RPM", "525_2hd_dualrpm" },
     /* 3.5" 1DD */
-    { 86, FLAG_RPM_300 | FLAG_HOLE0 | FLAG_DOUBLE_STEP, "3.5\" 360k", "35_1dd" },
+    { 86, FLAG_RPM_300 | FLAG_HOLE0 | FLAG_DOUBLE_STEP,                                                            "3.5\" 360k",              "35_1dd"          },
     /* 3.5" DD, Equivalent to TEAC FD-235F */
-    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_DOUBLE_STEP, "3.5\" 720k", "35_2dd" },
+    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_DOUBLE_STEP,                                                  "3.5\" 720k",              "35_2dd"          },
     /* 3.5" HD, Equivalent to TEAC FD-235HF */
-    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_PS2, "3.5\" 1.44M", "35_2hd" },
+    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_PS2,                          "3.5\" 1.44M",             "35_2hd"          },
     /* TODO: 3.5" DD, Equivalent to TEAC FD-235GF */
-//    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP, "3.5\" 1.25M", "35_2hd_2mode" },
+    //    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP, "3.5\" 1.25M", "35_2hd_2mode" },
     /* 3.5" HD PC-98 */
-    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_INVERT_DENSEL, "3.5\" 1.25M PC-98", "35_2hd_nec" },
+    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP | FLAG_INVERT_DENSEL, "3.5\" 1.25M PC-98",       "35_2hd_nec"      },
     /* 3.5" HD 3-Mode, Equivalent to TEAC FD-235HG */
-    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP, "3.5\" 1.44M 300/360 RPM", "35_2hd_3mode" },
+    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_DOUBLE_STEP,                      "3.5\" 1.44M 300/360 RPM", "35_2hd_3mode"    },
     /* 3.5" ED, Equivalent to TEAC FD-235J */
-    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_HOLE2 | FLAG_DOUBLE_STEP, "3.5\" 2.88M", "35_2ed" },
+    { 86, FLAG_RPM_300 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_HOLE2 | FLAG_DOUBLE_STEP,                        "3.5\" 2.88M",             "35_2ed"          },
     /* 3.5" ED Dual RPM, Equivalent to TEAC FD-335J */
-    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_HOLE2 | FLAG_DOUBLE_STEP, "3.5\" 2.88M 300/360 RPM", "35_2ed_dualrpm" },
+    { 86, FLAG_RPM_300 | FLAG_RPM_360 | FLAG_DS | FLAG_HOLE0 | FLAG_HOLE1 | FLAG_HOLE2 | FLAG_DOUBLE_STEP,         "3.5\" 2.88M 300/360 RPM", "35_2ed_dualrpm"  },
     /* End of list */
-    { -1, -1, "", "" }
+    { -1, -1,                                                                                                      "",                        ""                }
 };
 
 #ifdef ENABLE_FDD_LOG
@@ -184,11 +212,32 @@ int fdd_do_log = ENABLE_FDD_LOG;
 static void
 fdd_log(const char *fmt, ...)
 {
-    va_list ap;
+    va_list ap, ap_copy;
+    char    timebuf[32];
+    char    fullbuf[1056]; // 32 + 1024 bytes for timestamp + message
 
     if (fdd_do_log) {
+        uint32_t ticks        = plat_get_ticks();
+        uint32_t seconds      = ticks / 1000;
+        uint32_t milliseconds = ticks % 1000;
+
+        snprintf(timebuf, sizeof(timebuf), "[%07u.%03u] ", seconds, milliseconds);
+
         va_start(ap, fmt);
-        pclog_ex(fmt, ap);
+        va_copy(ap_copy, ap);
+
+        strcpy(fullbuf, timebuf);
+        vsnprintf(fullbuf + strlen(timebuf), sizeof(fullbuf) - strlen(timebuf), fmt, ap_copy);
+
+        va_end(ap_copy);
+        va_end(ap);
+
+        va_start(ap, fmt);
+        va_end(ap);
+
+        char *msg = fullbuf;
+        va_start(ap, fmt);
+        pclog_ex("%s", (va_list) &msg);
         va_end(ap);
     }
 }
@@ -196,6 +245,24 @@ fdd_log(const char *fmt, ...)
 #    define fdd_log(fmt, ...)
 #endif
 
+void
+fdd_set_audio_profile(int drive, int profile)
+{
+    if (drive < 0 || drive >= FDD_NUM)
+        return;
+    if (profile < 0 || profile >= FDD_AUDIO_PROFILE_MAX)
+        profile = 0;
+    fdd_audio_profile[drive] = profile;
+}
+
+int
+fdd_get_audio_profile(int drive)
+{
+    if (drive < 0 || drive >= FDD_NUM)
+        return 0;
+    return fdd_audio_profile[drive];
+}
+
 char *
 fdd_getname(int type)
 {
@@ -211,7 +278,7 @@ fdd_get_internal_name(int type)
 int
 fdd_get_from_internal_name(char *s)
 {
-    int   c = 0;
+    int c = 0;
 
     while (strlen(drive_types[c].internal_name)) {
         if (!strcmp((char *) drive_types[c].internal_name, s))
@@ -244,12 +311,70 @@ fdd_forced_seek(int drive, int track_diff)
     fdd_do_seek(drive, fdd[drive].track);
 }
 
+static void
+fdd_seek_complete_callback(void *priv)
+{
+    DRIVE *drive = (DRIVE *) priv;
+
+    fdd_seek_in_progress[drive->id] = 0;
+
+    fdd_log("fdd_seek_complete_callback(drive=%d) - TIMER FIRED! seek_in_progress=1\n", drive->id);
+    fdd_log("Notifying FDC of seek completion\n");
+    fdd_do_seek(drive->id, fdd[drive->id].track);
+
+    int had_pending = fdd_pending[drive->id].pending;
+    if (had_pending) {
+        fdd_pending_op_t *po = &fdd_pending[drive->id];
+        fdd_log("Starting deferred op %d after seek on drive %d (trk=%d, side=%d, sec=%d)\n",
+                po->op, drive->id, po->track, po->side, po->sector);
+
+        switch (po->op) {
+            case FDD_OP_READ:
+                if (drives[drive->id].readsector)
+                    drives[drive->id].readsector(drive->id, po->sector, po->track, po->side, po->density, po->sector_size);
+                break;
+            case FDD_OP_WRITE:
+                if (drives[drive->id].writesector)
+                    drives[drive->id].writesector(drive->id, po->sector, po->track, po->side, po->density, po->sector_size);
+                break;
+            case FDD_OP_COMPARE:
+                if (drives[drive->id].comparesector)
+                    drives[drive->id].comparesector(drive->id, po->sector, po->track, po->side, po->density, po->sector_size);
+                break;
+            case FDD_OP_READADDR:
+                if (drives[drive->id].readaddress)
+                    drives[drive->id].readaddress(drive->id, po->side, po->density);
+                break;
+            case FDD_OP_FORMAT:
+                if (drives[drive->id].format)
+                    drives[drive->id].format(drive->id, po->side, po->density, po->fill);
+                break;
+            default:
+                break;
+        }
+
+        po->pending = 0;
+        po->op      = FDD_OP_NONE;
+    }
+
+    if (!had_pending)
+        fdc_seek_complete_interrupt(fdd_fdc, drive->id);
+}
+
 void
 fdd_seek(int drive, int track_diff)
 {
+    fdd_log("fdd_seek(drive=%d, track_diff=%d)\n", drive, track_diff);
     if (!track_diff)
         return;
 
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek already in progress for drive %d, ignoring new seek request\n", drive);
+        return;
+    }
+
+    int old_track = fdd[drive].track;
+
     fdd[drive].track += track_diff;
 
     if (fdd[drive].track < 0)
@@ -260,12 +385,44 @@ fdd_seek(int drive, int track_diff)
 
     fdd_changed[drive] = 0;
 
-    fdd_do_seek(drive, fdd[drive].track);
+    if (fdd[drive].turbo)
+        fdd_do_seek(drive, fdd[drive].track);
+    else {
+        /* Trigger appropriate audio for track movements */
+        int actual_track_diff = abs(old_track - fdd[drive].track);
+        if (actual_track_diff == 1) {
+            /* Single track movement */
+            fdd_audio_play_single_track_step(drive, old_track, fdd[drive].track);
+        } else if (actual_track_diff > 1) {
+            /* Multi-track seek */
+            fdd_audio_play_multi_track_seek(drive, old_track, fdd[drive].track);
+        }
+
+        if (old_track + track_diff < 0) {
+            fdd_do_seek(drive, fdd[drive].track);
+            return;
+        }
+
+        fdd_seek_in_progress[drive] = 1;
+
+        if (!fdd_seek_timer[drive].callback) {
+            timer_add(&(fdd_seek_timer[drive]), fdd_seek_complete_callback, &drives[drive], 0);
+        }
+
+        /* Get seek timings from audio profile configuration */
+        double   initial_seek_time = fdd_audio_get_seek_time(drive, 1, actual_track_diff);
+        double   track_seek_time   = fdd_audio_get_seek_time(drive, 0, actual_track_diff);
+        fdd_log("Seek timing for drive %d: initial %.2f ms, per track %.2f ms\n", drive, initial_seek_time, track_seek_time);
+        uint64_t seek_time_us      = (initial_seek_time + (abs(actual_track_diff) * track_seek_time)) * TIMER_USEC;
+        timer_set_delay_u64(&fdd_seek_timer[drive], seek_time_us);
+    }
 }
 
 int
 fdd_track0(int drive)
 {
+    fdd_log("fdd_track0(drive=%d)\n", drive);
+
     /* If drive is disabled, TRK0 never gets set. */
     if (!drive_types[fdd[drive].type].max_track)
         return 0;
@@ -408,6 +565,7 @@ fdd_is_double_sided(int drive)
 void
 fdd_set_head(int drive, int head)
 {
+    fdd_log("fdd_set_head(%d, %d)\n", drive, head);
     if (head && !fdd_is_double_sided(drive))
         fdd[drive].head = 0;
     else
@@ -455,18 +613,17 @@ fdd_get_densel(int drive)
 void
 fdd_load(int drive, char *fn)
 {
+    fdd_log("fdd_load(%d, %s)\n", drive, fn);
     int         c = 0;
     int         size;
     const char *p;
     FILE       *fp;
     int         offs = 0;
 
-    fdd_log("FDD: loading drive %d with '%s'\n", drive, fn);
-
     if (!fn)
         return;
     if (strstr(fn, "wp://") == fn) {
-        offs = 5;
+        offs                = 5;
         ui_writeprot[drive] = 1;
     }
     fn += offs;
@@ -495,7 +652,6 @@ fdd_load(int drive, char *fn)
             c++;
         }
     }
-    fdd_log("FDD: could not load '%s' %s\n", fn, p);
     drive_empty[drive] = 1;
     fdd_set_head(drive, 0);
     memset(floppyfns[drive], 0, sizeof(floppyfns[drive]));
@@ -505,8 +661,6 @@ fdd_load(int drive, char *fn)
 void
 fdd_close(int drive)
 {
-    fdd_log("FDD: closing drive %d\n", drive);
-
     d86f_stop(drive); /* Call this first of all to make sure the 86F poll is back to idle state. */
     if (loaders[driveloaders[drive]].close)
         loaders[driveloaders[drive]].close(drive);
@@ -548,11 +702,14 @@ fdd_byteperiod(int drive)
 void
 fdd_set_motor_enable(int drive, int motor_enable)
 {
-    /* I think here is where spin-up and spin-down should be implemented. */
-    if (motor_enable && !motoron[drive])
+    fdd_log("fdd_set_motor_enable(%d, %d)\n", drive, motor_enable);
+    fdd_audio_set_motor_enable(drive, motor_enable);
+
+    if (motor_enable && !motoron[drive]) {
         timer_set_delay_u64(&fdd_poll_time[drive], fdd_byteperiod(drive));
-    else if (!motor_enable)
+    } else if (!motor_enable && motoron[drive]) {
         timer_disable(&fdd_poll_time[drive]);
+    }
     motoron[drive] = motor_enable;
 }
 
@@ -617,6 +774,23 @@ fdd_reset(void)
 void
 fdd_readsector(int drive, int sector, int track, int side, int density, int sector_size)
 {
+    fdd_log("fdd_readsector(%d, %d, %d, %d, %d, %d)\n", drive, sector, track, side, density, sector_size);
+
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek in progress on drive %d, deferring READ (trk=%d->%d, side=%d, sec=%d)\n",
+                drive, fdd[drive].track, track, side, sector);
+        fdd_pending[drive] = (fdd_pending_op_t) {
+            .pending     = 1,
+            .op          = FDD_OP_READ,
+            .sector      = sector,
+            .track       = track,
+            .side        = side,
+            .density     = density,
+            .sector_size = sector_size
+        };
+        return;
+    }
+
     if (drives[drive].readsector)
         drives[drive].readsector(drive, sector, track, side, density, sector_size);
     else
@@ -626,6 +800,23 @@ fdd_readsector(int drive, int sector, int track, int side, int density, int sect
 void
 fdd_writesector(int drive, int sector, int track, int side, int density, int sector_size)
 {
+    fdd_log("fdd_writesector(%d, %d, %d, %d, %d, %d)\n", drive, sector, track, side, density, sector_size);
+
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek in progress on drive %d, deferring WRITE (trk=%d->%d, side=%d, sec=%d)\n",
+                drive, fdd[drive].track, track, side, sector);
+        fdd_pending[drive] = (fdd_pending_op_t) {
+            .pending     = 1,
+            .op          = FDD_OP_WRITE,
+            .sector      = sector,
+            .track       = track,
+            .side        = side,
+            .density     = density,
+            .sector_size = sector_size
+        };
+        return;
+    }
+
     if (drives[drive].writesector)
         drives[drive].writesector(drive, sector, track, side, density, sector_size);
     else
@@ -635,6 +826,21 @@ fdd_writesector(int drive, int sector, int track, int side, int density, int sec
 void
 fdd_comparesector(int drive, int sector, int track, int side, int density, int sector_size)
 {
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek in progress on drive %d, deferring COMPARE (trk=%d->%d, side=%d, sec=%d)\n",
+                drive, fdd[drive].track, track, side, sector);
+        fdd_pending[drive] = (fdd_pending_op_t) {
+            .pending     = 1,
+            .op          = FDD_OP_COMPARE,
+            .sector      = sector,
+            .track       = track,
+            .side        = side,
+            .density     = density,
+            .sector_size = sector_size
+        };
+        return;
+    }
+
     if (drives[drive].comparesector)
         drives[drive].comparesector(drive, sector, track, side, density, sector_size);
     else
@@ -644,6 +850,19 @@ fdd_comparesector(int drive, int sector, int track, int side, int density, int s
 void
 fdd_readaddress(int drive, int side, int density)
 {
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek in progress on drive %d, deferring READADDRESS (trk=%d, side=%d)\n",
+                drive, fdd[drive].track, side);
+        fdd_pending[drive] = (fdd_pending_op_t) {
+            .pending = 1,
+            .op      = FDD_OP_READADDR,
+            .track   = fdd[drive].track,
+            .side    = side,
+            .density = density
+        };
+        return;
+    }
+
     if (drives[drive].readaddress)
         drives[drive].readaddress(drive, side, density);
 }
@@ -651,6 +870,20 @@ fdd_readaddress(int drive, int side, int density)
 void
 fdd_format(int drive, int side, int density, uint8_t fill)
 {
+    if (fdd_seek_in_progress[drive]) {
+        fdd_log("Seek in progress on drive %d, deferring FORMAT (trk=%d, side=%d)\n",
+                drive, fdd[drive].track, side);
+        fdd_pending[drive] = (fdd_pending_op_t) {
+            .pending = 1,
+            .op      = FDD_OP_FORMAT,
+            .track   = fdd[drive].track,
+            .side    = side,
+            .density = density,
+            .fill    = fill
+        };
+        return;
+    }
+
     if (drives[drive].format)
         drives[drive].format(drive, side, density, fill);
     else
@@ -690,10 +923,14 @@ fdd_init(void)
     for (i = 0; i < FDD_NUM; i++) {
         fdd_load(i, floppyfns[i]);
     }
+
+    if (fdd_sounds_enabled) {
+        fdd_audio_init();
+    }
 }
 
 void
 fdd_do_writeback(int drive)
 {
     d86f_handler[drive].writeback(drive);
-}
+}
\ No newline at end of file
