@@ -42,6 +42,8 @@
 #define GLOBAL_CONFIG_FILE "86box_global.cfg"
 #define NVR_PATH           "nvr"
 #define SCREENSHOT_PATH    "screenshots"
+#define VMM_PATH		   "Virtual Machines"
+#define VMM_PATH_WINDOWS   "86Box VMs"
 
 /* Recently used images */
 #define MAX_PREV_IMAGES    10
@@ -77,6 +79,17 @@
 #define BCD16(x)  ((((x) / 1000) << 12) | (((x) / 100) << 8) | BCD8(x))
 #define BCD32(x)  ((((x) / 10000000) << 28) | (((x) / 1000000) << 24) | (((x) / 100000) << 20) | (((x) / 10000) << 16) | BCD16(x))
 
+#define AS_U8(x)     (*((uint8_t *) &(x)))
+#define AS_U16(x)    (*((uint16_t *) &(x)))
+#define AS_U32(x)    (*((uint32_t *) &(x)))
+#define AS_U64(x)    (*((uint64_t *) &(x)))
+#define AS_I8(x)     (*((int8_t *) &(x)))
+#define AS_I16(x)    (*((int16_t *) &(x)))
+#define AS_I32(x)    (*((int32_t *) &(x)))
+#define AS_I64(x)    (*((int64_t *) &(x)))
+#define AS_FLOAT(x)  (*((float *) &(x)))
+#define AS_DOUBLE(x) (*((double *) &(x)))
+
 #if defined(__GNUC__) || defined(__clang__)
 #    define UNLIKELY(x) __builtin_expect((x), 0)
 #    define LIKELY(x)   __builtin_expect((x), 1)
@@ -138,6 +151,8 @@ extern int      force_43;                   /* (C) video */
 extern int      video_filter_method;        /* (C) video */
 extern int      video_vsync;                /* (C) video */
 extern int      video_framerate;            /* (C) video */
+extern double   video_gl_input_scale;       /* (C) OpenGL 3.x input scale */
+extern int      video_gl_input_scale_mode;  /* (C) OpenGL 3.x input stretch mode */
 extern int      gfxcard[GFXCARD_MAX];       /* (C) graphics/video card */
 extern int      bugger_enabled;             /* (C) enable ISAbugger */
 extern int      novell_keycard_enabled;     /* (C) enable Novell NetWare 2.x key card emulation. */
@@ -188,14 +203,24 @@ extern int    hook_enabled;                 /* (C) Keyboard hook is enabled */
 extern int    vmm_disabled;                 /* (G) disable built-in manager */
 extern char   vmm_path_cfg[1024];           /* (G) VMs path (unless -E is used) */
 
-extern char exe_path[2048];     /* path (dir) of executable */
-extern char usr_path[1024];     /* path (dir) of user data */
-extern char cfg_path[1024];     /* full path of config file */
+extern char exe_path[2048];        /* path (dir) of executable */
+extern char usr_path[1024];        /* path (dir) of user data */
+extern char cfg_path[1024];        /* full path of config file */
 extern char global_cfg_path[1024]; /* full path of global config file */
-extern int  open_dir_usr_path;  /* default file open dialog directory of usr_path */
-extern char uuid[MAX_UUID_LEN]; /* UUID or machine identifier */
-extern char vmm_path[1024];       /* VM Manager path to scan */
-extern int  start_vmm;
+extern int  open_dir_usr_path;     /* default file open dialog directory of usr_path */
+extern char uuid[MAX_UUID_LEN];    /* UUID or machine identifier */
+extern char vmm_path[1024];        /* VM Manager path to scan */
+extern int  start_vmm;             /* the current execution will start the manager */
+extern int  portable_mode;         /* we are running in portable mode 
+                                      (global dirs = exe path) */
+extern int global_cfg_overridden;  /* global config file was overriden on command line */
+
+extern int  monitor_edid;                   /* (C) Which EDID to use. 0=default, 1=custom. */
+extern char monitor_edid_path[1024];        /* (C) Path to custom EDID */
+
+extern int color_scheme;                    /* (C) Color scheme of UI (Windows-only) */
+extern int fdd_sounds_enabled;              /* (C) Enable floppy drive sounds */
+
 #ifndef USE_NEW_DYNAREC
 extern FILE *stdlog; /* file to log output to */
 #endif
