@@ -61,6 +61,7 @@ machine_at_ap61_init(const machine_t *model)
     pci_register_slot(0x04, PCI_CARD_NORMAL,          2, 3, 4, 1);
     pci_register_slot(0x05, PCI_CARD_NORMAL,          3, 4, 1, 2);
     pci_register_slot(0x06, PCI_CARD_NORMAL,          4, 1, 2, 3);
+
     device_add(&i450kx_device);
     device_add(&sio_zb_device);
     device_add(&ide_cmd646_device);
@@ -94,6 +95,7 @@ machine_at_p6rp4_init(const machine_t *model)
     pci_register_slot(0x06, PCI_CARD_NORMAL,          2, 3, 4, 1);
     pci_register_slot(0x05, PCI_CARD_NORMAL,          3, 4, 1, 2);
     pci_register_slot(0x04, PCI_CARD_NORMAL,          4, 1, 2, 3);
+
     device_add(&i450kx_device);
     device_add(&sio_zb_device);
     device_add(&ide_cmd646_device);
@@ -108,20 +110,35 @@ machine_at_p6rp4_init(const machine_t *model)
 static const device_config_t ficpo6000_config[] = {
     // clang-format off
     {
-        .name = "bios",
-        .description = "BIOS Version",
-        .type = CONFIG_BIOS,
-        .default_string = "405F03C",
-        .default_int = 0,
-        .file_filter = "",
-        .spinner = { 0 }, /*W1*/
-        .bios = {
-            { .name = "PhoenixBIOS 4.05 - Revision 405F03C (CD-ROM Boot support)", .internal_name = "405F03C", .bios_type = BIOS_NORMAL,
-              .files_no = 1, .local = 0, .size = 131072, .files = { "roms/machines/ficpo6000/405F03C.ROM", "" } },
-            { .name = "PhoenixBIOS 4.05 - Revision 405F05C (No CD-ROM Boot support)", .internal_name = "405F05C", .bios_type = BIOS_NORMAL,
-              .files_no = 1, .local = 0, .size = 131072, .files = { "roms/machines/ficpo6000/405F05C.ROM", "" } },
+        .name           = "bios",
+        .description    = "BIOS Version",
+        .type           = CONFIG_BIOS,
+        .default_string = "405F05C",
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = {
+            {
+                .name          = "PhoenixBIOS 4.05 - Revision 405F03C (CD-ROM Boot support)",
+                .internal_name = "405F03C",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 1,
+                .local         = 0,
+                .size          = 131072,
+                .files         = { "roms/machines/ficpo6000/405F03C.ROM", "" }
+            },
+            {
+                .name          = "PhoenixBIOS 4.05 - Revision 405F05C (No CD-ROM Boot support)",
+                .internal_name = "405F05C",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 1,
+                .local         = 0,
+                .size          = 131072,
+                .files         = { "roms/machines/ficpo6000/405F05C.ROM", "" }
+            },
             { .files_no = 0 }
-        },
+        }
     },
     { .name = "", .description = "", .type = CONFIG_END }
     // clang-format on
@@ -135,7 +152,7 @@ const device_t ficpo6000_device = {
     .init          = NULL,
     .close         = NULL,
     .reset         = NULL,
-    .available 	   = NULL,
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = ficpo6000_config
@@ -144,15 +161,15 @@ const device_t ficpo6000_device = {
 int
 machine_at_ficpo6000_init(const machine_t *model)
 {
-    int ret = 0;
-    const char* fn;
+    int         ret = 0;
+    const char *fn;
 
     /* No ROMs available */
     if (!device_available(model->device))
         return ret;
 
     device_context(model->device);
-    fn = device_get_bios_file(machine_get_device(machine), device_get_config_bios("bios"), 0);
+    fn  = device_get_bios_file(machine_get_device(machine), device_get_config_bios("bios"), 0);
     ret = bios_load_linear(fn, 0x000e0000, 131072, 0);
     device_context_restore();
 
@@ -167,6 +184,7 @@ machine_at_ficpo6000_init(const machine_t *model)
     pci_register_slot(0x05, PCI_CARD_NORMAL,          3, 4, 1, 2);
     pci_register_slot(0x06, PCI_CARD_NORMAL,          4, 1, 2, 3);
     pci_register_slot(0x0c, PCI_CARD_IDE,             0, 0, 0, 0);
+
     device_add(&i450kx_device);
     device_add(&sio_zb_device);
     device_add(&ide_cmd646_device);
@@ -189,7 +207,7 @@ machine_at_acerv60n_init(const machine_t *model)
     if (bios_only || !ret)
         return ret;
 
-    machine_at_common_init_ex(model,2 );
+    machine_at_common_init_ex(model, 2);
 
     pci_init(PCI_CONFIG_TYPE_1);
     pci_register_slot(0x00, PCI_CARD_NORTHBRIDGE, 0, 0, 0, 0);
@@ -199,6 +217,7 @@ machine_at_acerv60n_init(const machine_t *model)
     pci_register_slot(0x10, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x12, PCI_CARD_NORMAL,      4, 1, 2, 3);
     pci_register_slot(0x0C, PCI_CARD_NORMAL,      2, 3, 4, 1);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(&fdc37c93x_device, (void *) (FDC37XXX5 | FDC37C93X_NORMAL));
@@ -244,6 +263,7 @@ machine_at_8600ttc_init(const machine_t *model)
     pci_register_slot(0x09, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x0A, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x0B, PCI_CARD_NORMAL,      4, 1, 2, 3);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(machine_get_kbc_device(machine), (void *) model->kbc_params);
@@ -273,6 +293,7 @@ machine_at_686nx_init(const machine_t *model)
     pci_register_slot(0x09, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x0A, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x0B, PCI_CARD_NORMAL,      4, 1, 2, 3);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(machine_get_kbc_device(machine), (void *) model->kbc_params);
@@ -282,6 +303,15 @@ machine_at_686nx_init(const machine_t *model)
     return ret;
 }
 
+uint32_t
+machine_ap440fx_vs440fx_gpio_handler(uint8_t write, uint32_t val)
+{
+    if (!write)
+        return 0xff7f;
+
+    return val; /* Writes are ignored. */
+}
+
 int
 machine_at_ap440fx_init(const machine_t *model)
 {
@@ -306,6 +336,7 @@ machine_at_ap440fx_init(const machine_t *model)
     pci_register_slot(0x13, PCI_CARD_NORMAL,      2, 1, 3, 4);
     pci_register_slot(0x0B, PCI_CARD_NORMAL,      3, 2, 1, 4);
     pci_register_slot(0x07, PCI_CARD_SOUTHBRIDGE, 0, 0, 0, 4);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(&pc87307_device, (void *) (PCX730X_AMI | PCX7307_PC87307));
@@ -323,28 +354,50 @@ machine_at_ap440fx_init(const machine_t *model)
 static const device_config_t vs440fx_config[] = {
     // clang-format off
     {
-        .name = "bios",
-        .description = "BIOS Version",
-        .type = CONFIG_BIOS,
+        .name           = "bios",
+        .description    = "BIOS Version",
+        .type           = CONFIG_BIOS,
         .default_string = "vs440fx",
-        .default_int = 0,
-        .file_filter = "",
-        .spinner = { 0 },
-        .bios = {
-            { .name = "Intel AMIBIOS - Revision 1.00.06.CS1J (Dell Dimension XPS Pro___n)", .internal_name = "dellvenus", .bios_type = BIOS_NORMAL, 
-              .files_no = 5, .local = 0, .size = 262144, .files = { "roms/machines/vs440fx/1006CS1J.BIO", "roms/machines/vs440fx/1006CS1J.BI1",
-                                                                    "roms/machines/vs440fx/1006CS1J.BI2", "roms/machines/vs440fx/1006CS1J.BI3",
-                                                                    "roms/machines/vs440fx/1006CS1J.RCV", "" } },
-            { .name = "Intel AMIBIOS - Revision 1.00.11.CS1T (Gateway 2000)", .internal_name = "gw2kvenus", .bios_type = BIOS_NORMAL, 
-              .files_no = 5, .local = 0, .size = 262144, .files = { "roms/machines/vs440fx/1011CS1T.BIO", "roms/machines/vs440fx/1011CS1T.BI1",
-                                                                    "roms/machines/vs440fx/1011CS1T.BI2", "roms/machines/vs440fx/1011CS1T.BI3",
-                                                                    "roms/machines/vs440fx/1011CS1T.RCV", "" } },
-            { .name = "Intel AMIBIOS - Revision 1.00.18.CS1", .internal_name = "vs440fx", .bios_type = BIOS_NORMAL, 
-              .files_no = 5, .local = 0, .size = 262144, .files = { "roms/machines/vs440fx/1018CS1_.BIO", "roms/machines/vs440fx/1018CS1_.BI1",
-                                                                    "roms/machines/vs440fx/1018CS1_.BI2", "roms/machines/vs440fx/1018CS1_.BI3",
-                                                                    "roms/machines/vs440fx/1018CS1_.RCV", "" } },
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = {
+            {
+                .name          = "Intel AMIBIOS - Revision 1.00.06.CS1J (Dell Dimension XPS Pro___n)",
+                .internal_name = "dellvenus",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 5,
+                .local         = 0,
+                .size          = 262144,
+                .files         = { "roms/machines/vs440fx/1006CS1J.BIO", "roms/machines/vs440fx/1006CS1J.BI1",
+                                   "roms/machines/vs440fx/1006CS1J.BI2", "roms/machines/vs440fx/1006CS1J.BI3",
+                                   "roms/machines/vs440fx/1006CS1J.RCV", "" }
+            },
+            {
+                .name          = "Intel AMIBIOS - Revision 1.00.11.CS1T (Gateway 2000)",
+                .internal_name = "gw2kvenus",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 5,
+                .local         = 0,
+                .size          = 262144,
+                .files         = { "roms/machines/vs440fx/1011CS1T.BIO", "roms/machines/vs440fx/1011CS1T.BI1",
+                                   "roms/machines/vs440fx/1011CS1T.BI2", "roms/machines/vs440fx/1011CS1T.BI3",
+                                   "roms/machines/vs440fx/1011CS1T.RCV", "" }
+            },
+            {
+                .name          = "Intel AMIBIOS - Revision 1.00.18.CS1",
+                .internal_name = "vs440fx",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 5,
+                .local         = 0,
+                .size          = 262144,
+                .files         = { "roms/machines/vs440fx/1018CS1_.BIO", "roms/machines/vs440fx/1018CS1_.BI1",
+                                   "roms/machines/vs440fx/1018CS1_.BI2", "roms/machines/vs440fx/1018CS1_.BI3",
+                                   "roms/machines/vs440fx/1018CS1_.RCV", "" }
+            },
             { .files_no = 0 }
-        },
+        }
     },
     { .name = "", .description = "", .type = CONFIG_END }
     // clang-format on
@@ -367,15 +420,15 @@ const device_t vs440fx_device = {
 int
 machine_at_vs440fx_init(const machine_t *model)
 {
-    int ret = 0;
-    const char* fn[5];
+    int         ret = 0;
+    const char *fn[5];
 
     /* No ROMs available */
     if (!device_available(model->device))
         return ret;
 
     device_context(model->device);
-    for (int i = 0; i < 5; i++)
+    for (uint8_t i = 0; i < 5; i++)
         fn[i] = device_get_bios_file(machine_get_device(machine), device_get_config_bios("bios"), i);
     ret = bios_load_linear_combined2(fn[0], fn[1], fn[2], fn[3], fn[4], 0x3a000, 128);
     device_context_restore();
@@ -389,6 +442,7 @@ machine_at_vs440fx_init(const machine_t *model)
     pci_register_slot(0x11, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x13, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x07, PCI_CARD_SOUTHBRIDGE, 0, 0, 0, 0);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(&pc87307_device, (void *) (PCX730X_AMI | PCX7307_PC87307));
@@ -421,6 +475,7 @@ machine_at_lgibmx61_init(const machine_t *model)
     pci_register_slot(0x0D, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x0E, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x0F, PCI_CARD_NORMAL,      4, 1, 2, 3);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(machine_get_kbc_device(machine), (void *) model->kbc_params);
@@ -450,6 +505,7 @@ machine_at_m6mi_init(const machine_t *model)
     pci_register_slot(0x11, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x10, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x0F, PCI_CARD_NORMAL,      4, 1, 2, 3);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(&fdc37c93x_device, (void *) (FDC37XXX5 | FDC37C93X_NORMAL));
@@ -478,6 +534,7 @@ machine_at_mb600n_init(const machine_t *model)
     pci_register_slot(0x12, PCI_CARD_NORMAL,      2, 3, 4, 1);
     pci_register_slot(0x13, PCI_CARD_NORMAL,      3, 4, 1, 2);
     pci_register_slot(0x14, PCI_CARD_NORMAL,      4, 1, 2, 3);
+
     device_add(&i440fx_device);
     device_add(&piix3_device);
     device_add_params(machine_get_kbc_device(machine), (void *) model->kbc_params);
