@@ -8,8 +8,6 @@
  *
  *          Emulation of the IBM PCjr.
  *
- *
- *
  * Authors: Sarah Walker, <https://pcem-emulator.co.uk/>
  *          Miran Grca, <mgrca8@gmail.com>
  *          Fred N. van Kempen, <decwiz@yahoo.com>
@@ -43,6 +41,7 @@
 #include <86box/rom.h>
 #include <86box/fdd.h>
 #include <86box/fdc.h>
+#include <86box/fdc_ext.h>
 #include <86box/sound.h>
 #include <86box/snd_speaker.h>
 #include <86box/snd_sn76489.h>
@@ -52,16 +51,14 @@
 #include <86box/machine.h>
 #include <86box/plat_unused.h>
 
-
-#define STAT_PARITY    0x80
-#define STAT_RTIMEOUT  0x40
-#define STAT_TTIMEOUT  0x20
-#define STAT_LOCK      0x10
-#define STAT_CD        0x08
-#define STAT_SYSFLAG   0x04
-#define STAT_IFULL     0x02
-#define STAT_OFULL     0x01
-
+#define STAT_PARITY   0x80
+#define STAT_RTIMEOUT 0x40
+#define STAT_TTIMEOUT 0x20
+#define STAT_LOCK     0x10
+#define STAT_CD       0x08
+#define STAT_SYSFLAG  0x04
+#define STAT_IFULL    0x02
+#define STAT_OFULL    0x01
 
 static uint8_t key_queue[16];
 static int     key_queue_start = 0;
@@ -70,7 +67,7 @@ static int     key_queue_end   = 0;
 /*PCjr keyboard has no escape scancodes, and no scancodes beyond 54
   Map right alt to 54h (FN) */
 const scancode scancode_pcjr[512] = {
-  // clang-format off
+    // clang-format off
     { .mk = {            0 }, .brk = {                   0 } }, /* 000 */
     { .mk = {      0x01, 0 }, .brk = {             0x81, 0 } }, /* 001 */
     { .mk = {      0x02, 0 }, .brk = {             0x82, 0 } }, /* 002 */
@@ -583,10 +580,9 @@ const scancode scancode_pcjr[512] = {
     { .mk = {            0 }, .brk = {                   0 } }, /* 1fd */
     { .mk = {            0 }, .brk = {                   0 } }, /* 1fe */
     { .mk = {            0 }, .brk = {                   0 } }  /* 1ff */
-  // clang-format on
+    // clang-format on
 };
 
-
 static void
 kbd_write(uint16_t port, uint8_t val, void *priv)
 {
@@ -657,7 +653,10 @@ kbd_read(uint16_t port, void *priv)
 
         case 0x62:
             ret = (pcjr->latched ? 1 : 0);
-            ret |= 0x02; /* Modem card not installed */
+            if (!pcjr->option_modem)
+                ret |= 0x02; /* Modem card not installed */
+            if (!pcjr->option_fdc)
+                ret |= 0x04; /* Diskette card not installed */
             if (mem_size < 128)
                 ret |= 0x08; /* 64k expansion card not installed */
             if ((pcjr->pb & 0x08) || (cassette == NULL))
@@ -669,6 +668,8 @@ kbd_read(uint16_t port, void *priv)
             ret |= (pcjr->data ? 0x40 : 0);
             if (pcjr->data)
                 ret |= 0x40;
+            if (pcjr->option_ir)
+                ret |= 0x80; /* Keyboard cable not connected */
             break;
 
         case 0xa0:
@@ -772,7 +773,7 @@ pit_irq0_timer_pcjr(int new_out, int old_out, UNUSED(void *priv))
 }
 
 static const device_config_t pcjr_config[] = {
-  // clang-format off
+    // clang-format off
     {
         .name           = "display_type",
         .description    = "Display type",
@@ -787,7 +788,8 @@ static const device_config_t pcjr_config[] = {
             { .description = "RGB (no brown)", .value = PCJR_RGB_NO_BROWN },
             { .description = "RGB (IBM 5153)", .value = PCJR_RGB_IBM_5153 },
             { .description = ""                                           }
-        }
+        },
+        .bios           = { { 0 } }
     },
     {
         .name           = "double_type",
@@ -811,10 +813,38 @@ static const device_config_t pcjr_config[] = {
         .description    = "Apply overscan deltas",
         .type           = CONFIG_BINARY,
         .default_string = NULL,
-        .default_int    = 1
+        .default_int    = 1,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = { { 0 } }
     },
+#if 0
+    {
+        .name           = "modem_slot",
+        .description    = "Enable Serial Port in Modem Slot",
+        .type           = CONFIG_BINARY,
+        .default_string = NULL,
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = { { 0 } }
+    },
+    {
+        .name           = "ir_receiver",
+        .description    = "Enable IR Receiver",
+        .type           = CONFIG_BINARY,
+        .default_string = NULL,
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = { { 0 } }
+    },
+#endif
     { .name = "", .description = "", .type = CONFIG_END }
-  // clang-format on
+    // clang-format on
 };
 
 const device_t pcjr_device = {
@@ -846,6 +876,18 @@ machine_pcjr_init(UNUSED(const machine_t *model))
 
     pcjr = calloc(1, sizeof(pcjr_t));
 
+#if 0
+    pcjr->option_modem = device_get_config_int("modem_slot");
+#else
+    pcjr->option_modem = 0;
+#endif
+    pcjr->option_fdc   = 0;
+#if 0
+    pcjr->option_ir    = device_get_config_int("ir_receiver");
+#else
+    pcjr->option_ir    = 0;
+#endif
+
     is_pcjr = 1;
 
     pic_init_pcjr();
@@ -865,9 +907,13 @@ machine_pcjr_init(UNUSED(const machine_t *model))
     keyboard_scan   = 1;
     key_queue_start = key_queue_end = 0;
     io_sethandler(0x0060, 4,
-                  kbd_read, NULL, NULL, kbd_write, NULL, NULL, pcjr);
+                  kbd_read, NULL, NULL,
+                  kbd_write, NULL, NULL,
+                  pcjr);
     io_sethandler(0x00a0, 8,
-                  kbd_read, NULL, NULL, kbd_write, NULL, NULL, pcjr);
+                  kbd_read, NULL, NULL,
+                  kbd_write, NULL, NULL,
+                  pcjr);
     timer_add(&pcjr->send_delay_timer, kbd_poll, pcjr, 1);
     keyboard_set_table(scancode_pcjr);
     keyboard_send = kbd_adddata_ex;
@@ -877,9 +923,18 @@ machine_pcjr_init(UNUSED(const machine_t *model))
 
     nmi_mask = 0x80;
 
-    device_add(&fdc_pcjr_device);
+    if (fdc_current[0] == FDC_INTERNAL) {
+        device_add(&fdc_pcjr_device);
+        pcjr->option_fdc = 1;
+    }
+
+    if (!pcjr->option_modem)
+        device_add(&ns8250_pcjr_2f8_device);
+    else {
+        device_add(&ns8250_pcjr_3f8_device);
+        device_add(&ns8250_pcjr_2f8_device);
+    }
 
-    device_add(&ns8250_pcjr_device);
     /* So that serial_standalone_init() won't do anything. */
     serial_set_next_inst(SERIAL_MAX - 1);
 
