@@ -8,8 +8,6 @@
  *
  *          Implementation of MCA-based PS/2 machines.
  *
- *
- *
  * Authors: Fred N. van Kempen, <decwiz@yahoo.com>
  *          Miran Grca, <mgrca8@gmail.com>
  *          Sarah Walker, <https://pcem-emulator.co.uk/>
@@ -107,8 +105,8 @@ static struct ps2_t {
     serial_t *uart;
     lpt_t    *lpt;
 
-    vga_t* mb_vga;
-    int has_e0000_hole;
+    vga_t *mb_vga;
+    int    has_e0000_hole;
 } ps2;
 
 /*The model 70 type 3/4 BIOS performs cache testing. Since 86Box doesn't have any
@@ -146,7 +144,7 @@ static struct ps2_t {
 
 static uint8_t ps2_cache[65536];
 static int     ps2_cache_valid[65536 / 8];
-static void mem_encoding_update(void);
+static void    mem_encoding_update(void);
 // #define ENABLE_PS2_MCA_LOG 1
 #ifdef ENABLE_PS2_MCA_LOG
 int ps2_mca_do_log = ENABLE_PS2_MCA_LOG;
@@ -376,45 +374,47 @@ ps55_model_50t_read(uint16_t port)
             return ps2.planar_id >> 8;
         case 0x102:
             return ps2.option[0];
-        case 0x103: {
-            uint8_t val = 0xff;
-            /*
-            I/O 103h - Bit 7-4: Memory Card ID (Connector 1 or 3)
-                       Bit 3-0: Memory Card ID (Connector 2)
+        case 0x103:
+            {
+                uint8_t val = 0xff;
+                /*
+                I/O 103h - Bit 7-4: Memory Card ID (Connector 1 or 3)
+                           Bit 3-0: Memory Card ID (Connector 2)
 
-            Memory Card ID: 7h = 2 MB Memory Card 2 or 3 Installed
-                            5h = 4 MB Memory Card 2 Installed
-            */
-            switch (mem_size / 1024) {
-                case 2:
-                    if (ps2.option[1] & 0x04)
-                        val = 0xff;
-                    else
-                        val = 0x7f;
-                    break;
-                case 4:
-                    if (ps2.option[1] & 0x04)
-                        val = 0xff;
-                    else
-                        val = 0x77;
-                    break;
-                case 6:
-                    if (ps2.option[1] & 0x04)
-                        val = 0x7f;
-                    else
-                        val = 0x77;
-                    break;
-                case 8:
-                default:
-                    if (ps2.option[1] & 0x04)
-                        val = 0x5f;
-                    else
-                        val = 0x77;
-                    break;
+                Memory Card ID: 7h = 2 MB Memory Card 2 or 3 Installed
+                                5h = 4 MB Memory Card 2 Installed
+                */
+                switch (mem_size / 1024) {
+                    case 2:
+                        if (ps2.option[1] & 0x04)
+                            val = 0xff;
+                        else
+                            val = 0x7f;
+                        break;
+                    case 4:
+                        if (ps2.option[1] & 0x04)
+                            val = 0xff;
+                        else
+                            val = 0x77;
+                        break;
+                    case 6:
+                        if (ps2.option[1] & 0x04)
+                            val = 0x7f;
+                        else
+                            val = 0x77;
+                        break;
+                    case 8:
+                    default:
+                        if (ps2.option[1] & 0x04)
+                            val = 0x5f;
+                        else
+                            val = 0x77;
+                        break;
+                }
+                ps2_mca_log(" Read MCA %04X %02X %04X:%04X mem_size = %d, ps2option1 = %2X\n", port, val, cs >> 4, cpu_state.pc, mem_size, ps2.option[1]);
+                return val;
             }
-            ps2_mca_log(" Read MCA %04X %02X %04X:%04X mem_size = %d, ps2option1 = %2X\n", port, val, cs >> 4, cpu_state.pc, mem_size, ps2.option[1]);
-            return val;
-        } case 0x104:
+        case 0x104:
             return ps2.option[2];
         case 0x105:
             return ps2.option[3];
@@ -436,32 +436,34 @@ ps55_model_50v_read(uint16_t port)
             return ps2.planar_id >> 8;
         case 0x102:
             return ps2.option[0];
-        case 0x103: {
-            uint8_t val = 0xff;
-            /*
-            I/O 103h - Bit 7-4: Reserved
-                       Bit 3-0: Memory Card ID (Connector 3 or 1)
+        case 0x103:
+            {
+                uint8_t val = 0xff;
+                /*
+                I/O 103h - Bit 7-4: Reserved
+                           Bit 3-0: Memory Card ID (Connector 3 or 1)
 
-            Memory Card ID: 8h = 4 MB Memory Card IV Installed
-                            Fh = No Card Installed
-            */
-            switch (mem_size / 1024) {
-                case 4:
-                    if (ps2.option[1] & 0x04)
-                        val = 0xff;
-                    else
-                        val = 0xf8;
-                    break;
-                case 8:
-                default:
-                    if (ps2.option[1] & 0x04)
-                        val = 0xf8;
-                    else
-                        val = 0xf8;
-                    break;
+                Memory Card ID: 8h = 4 MB Memory Card IV Installed
+                                Fh = No Card Installed
+                */
+                switch (mem_size / 1024) {
+                    case 4:
+                        if (ps2.option[1] & 0x04)
+                            val = 0xff;
+                        else
+                            val = 0xf8;
+                        break;
+                    case 8:
+                    default:
+                        if (ps2.option[1] & 0x04)
+                            val = 0xf8;
+                        else
+                            val = 0xf8;
+                        break;
+                }
+                return val;
             }
-            return val;
-        } case 0x104:
+        case 0x104:
             /* Reading cache ID (bit 3-2) always returns zero */
             return ps2.option[2] & 0xf3;
         case 0x105:
@@ -505,7 +507,7 @@ model_50_write(uint16_t port, uint8_t val)
             ps2.option[0] = val;
             break;
         case 0x103:
-            ps2.option[1] = val;
+            ps2.option[1] = (ps2.option[1] & 0xfe) | (val & 0x01);
             break;
         case 0x104:
             ps2.option[2] = val;
@@ -532,10 +534,10 @@ model_55sx_mem_recalc(void)
 #ifdef ENABLE_PS2_MCA_LOG
     int enabled_mem = 0;
 #endif
-    int base = 0;
-    int remap_size = (ps2.option[3] & 0x10) ? 384 : 256;
-    int bit_mask = 0x00;
-    int max_rows = 4;
+    int base             = 0;
+    int remap_size       = (ps2.option[3] & 0x10) ? 384 : 256;
+    int bit_mask         = 0x00;
+    int max_rows         = 4;
     int bank_to_rows[16] = { 4, 2, 1, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 2, 1, 0 };
 
     ps2_mca_log("%02X %02X\n", ps2.option[1], ps2.option[3]);
@@ -970,18 +972,15 @@ ps2_mca_write(uint16_t port, uint8_t val, UNUSED(void *priv))
         case 0x102:
             if (!(ps2.setup & PS2_SETUP_IO))
                 ps2.planar_write(port, val);
-            else if (!(ps2.setup & PS2_SETUP_VGA))
-            {
-                if (ps2.mb_vga)
-                {
+            else if (!(ps2.setup & PS2_SETUP_VGA)) {
+                if (ps2.mb_vga) {
                     if (vga_isenabled(ps2.mb_vga))
                         vga_disable(ps2.mb_vga);
                     if (val & 1)
                         vga_enable(ps2.mb_vga);
                 }
                 ps2.pos_vga = val;
-            }
-            else if (ps2.adapter_setup & PS2_ADAPTER_SETUP)
+            } else if (ps2.adapter_setup & PS2_ADAPTER_SETUP)
                 mca_write(port, val);
             break;
         case 0x103:
@@ -1164,6 +1163,29 @@ ps2_mca_board_model_50_init(void)
     ps2.planar_read  = model_50_read;
     ps2.planar_write = model_50_write;
 
+    /*
+     I/O 103h - Bit 3: Memory Presence Detect 2
+                Bit 2: Memory Presence Detect 1
+                Bit 1: Password Override Jumper
+                Bit 0: Enable System Board RAM
+    */
+
+    switch (mem_size / 1024) {
+        case 0: /*256Kx2*/
+            ps2.option[1] = 0xf8;
+            break;
+        case 1: /*512Kx2*/
+            ps2.option[1] = 0xf4;
+            break;
+        case 2: /*1Mx2*/
+        default:
+            ps2.option[1] = 0xf0;
+            break;
+    }
+
+    /* Enable password function */
+    ps2.option[1] |= 0x02;
+
     if (mem_size > 2048) {
         /* Only 2 MB supported on planar, create a memory expansion card for the rest */
         ps2_mca_mem_fffc_init(2);
@@ -1184,9 +1206,36 @@ ps2_mca_board_model_60_init(void)
     ps2.planar_read  = model_50_read;
     ps2.planar_write = model_50_write;
 
-    if (mem_size > 2048) {
-        /* Only 2 MB supported on planar, create a memory expansion card for the rest */
-        ps2_mca_mem_fffc_init(2);
+    /*
+     I/O 103h - Bit 3: Memory Presence Detect 2 (Reversed)
+                Bit 2: Memory Presence Detect 1 (Reversed)
+                Bit 1: Password Override Jumper
+                Bit 0: Enable System Board RAM
+    */
+
+    switch (mem_size / 1024) {
+        case 0: /*256Kx2*/
+            ps2.option[1] = 0xf0;
+            break;       
+        case 1: /*256Kx4*/
+            ps2.option[1] = 0xf4;
+            break;
+        case 2: /*512Kx4*/
+        case 3: /*Not supported*/
+            ps2.option[1] = 0xf8;
+            break;
+        case 4: /*1Mx4*/
+        default:
+            ps2.option[1] = 0xfc;
+            break;
+    }
+    
+    /* Enable password function */
+    ps2.option[1] |= 0x02;
+
+    if (mem_size > 4096) {
+        /* Only 4 MB supported on planar, create a memory expansion card for the rest */
+        ps2_mca_mem_fffc_init(4);
     }
 
     device_add(&ps2_nvr_55ls_device);
@@ -1597,27 +1646,88 @@ machine_ps2_common_init(const machine_t *model)
 
     ps2.uart = device_add_inst(&ns16550_device, 1);
 
-    ps2.lpt  = device_add_inst(&lpt_port_device, 1);
+    ps2.lpt = device_add_inst(&lpt_port_device, 1);
     lpt_set_ext(ps2.lpt, 1);
 
     ps2.has_e0000_hole = 0;
 }
 
+static const device_config_t ps2_model_50_config[] = {
+    // clang-format off
+    {
+        .name           = "bios",
+        .description    = "BIOS Version",
+        .type           = CONFIG_BIOS,
+        .default_string = "ibmps2_m50",
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = {
+            {
+                .name          = "IBM PS/2 model 50",
+                .internal_name = "ibmps2_m50",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 4,
+                .local         = 0,
+                .size          = 131072,
+                .files         = { "roms/machines/ibmps2_m50/90x7420.zm13", "roms/machines/ibmps2_m50/90x7429.zm18",
+                                   "roms/machines/ibmps2_m50/90x7423.zm14", "roms/machines/ibmps2_m50/90x7426.zm16", "" }
+            },
+            {
+                .name          = "IBM PS/2 model 50Z",
+                .internal_name = "ibmps2_m50z",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 2,
+                .local         = 0,
+                .size          = 131072,
+                .files         = { "roms/machines/ibmps2_m50/15F8366.BIN", "roms/machines/ibmps2_m50/15F8365.BIN", "" }
+            },
+            { .files_no = 0 }
+        }
+    },
+    { .name = "", .description = "", .type = CONFIG_END }
+    // clang-format on
+};
+
+const device_t ps2_model_50_device = {
+    .name          = "IBM PS/2 model 50",
+    .internal_name = "ps2_model_50_device",
+    .flags         = 0,
+    .local         = 0,
+    .init          = NULL,
+    .close         = NULL,
+    .reset         = NULL,
+    .available     = NULL,
+    .speed_changed = NULL,
+    .force_redraw  = NULL,
+    .config        = ps2_model_50_config
+};
+
 int
 machine_ps2_model_50_init(const machine_t *model)
 {
-    int ret;
+    int         ret = 0;
+    const char *fn[4];
 
-    ret = bios_load_interleaved("roms/machines/ibmps2_m50/90x7420.zm13",
-                                "roms/machines/ibmps2_m50/90x7429.zm18",
-                                0x000f0000, 131072, 0);
-    ret &= bios_load_aux_interleaved("roms/machines/ibmps2_m50/90x7423.zm14",
-                                     "roms/machines/ibmps2_m50/90x7426.zm16",
-                                     0x000e0000, 65536, 0);
-
-    if (bios_only || !ret)
+    /* No ROMs available */
+    if (!device_available(model->device))
         return ret;
 
+    device_context(model->device);
+    int is_50z = !strcmp(device_get_config_bios("bios"), "ibmps2_m50z");
+    if (is_50z) {
+        for (uint8_t i = 0; i < 2; i++)
+            fn[i] = device_get_bios_file(machine_get_device(machine), device_get_config_bios("bios"), i);
+        ret = bios_load_interleaved(fn[0], fn[1], 0x000e0000, 131072, 0);
+    } else {
+        for (uint8_t i = 0; i < 4; i++)
+            fn[i] = device_get_bios_file(machine_get_device(machine), device_get_config_bios("bios"), i);
+        ret = bios_load_interleaved(fn[0], fn[1], 0x000f0000, 131072, 0);
+        ret &= bios_load_aux_interleaved(fn[2], fn[3], 0x000e0000, 65536, 0);
+    }
+    device_context_restore();
+
     machine_ps2_common_init(model);
 
     ps2.planar_id = 0xfbff;
@@ -1794,7 +1904,7 @@ ps55_mca_board_model_50t_init(void)
     /* The slot 5 is reserved for the Integrated Fixed Disk II (an internal ESDI hard drive). */
     mca_init(5);
 
-    ps2.planar_read = ps55_model_50t_read;
+    ps2.planar_read  = ps55_model_50t_read;
     ps2.planar_write = ps55_model_50tv_write;
 
     device_add(&ps2_nvr_device);
@@ -1806,26 +1916,26 @@ ps55_mca_board_model_50t_init(void)
     ps2.has_e0000_hole = 1;
 
     mem_mapping_add(&ps2.split_mapping,
-        (mem_size + 256) * 1024,
-        256 * 1024,
-        ps2_read_split_ram,
-        ps2_read_split_ramw,
-        ps2_read_split_raml,
-        ps2_write_split_ram,
-        ps2_write_split_ramw,
-        ps2_write_split_raml,
-        &ram[0xa0000],
-        MEM_MAPPING_INTERNAL,
-        NULL);
+                    (mem_size + 256) * 1024,
+                    256 * 1024,
+                    ps2_read_split_ram,
+                    ps2_read_split_ramw,
+                    ps2_read_split_raml,
+                    ps2_write_split_ram,
+                    ps2_write_split_ramw,
+                    ps2_write_split_raml,
+                    &ram[0xa0000],
+                    MEM_MAPPING_INTERNAL,
+                    NULL);
     mem_mapping_disable(&ps2.split_mapping);
 
     if (mem_size > 8192) {
         /* Only 8 MB supported on planar, create a memory expansion card for the rest */
-            ps2_mca_mem_fffc_init(8);
+        ps2_mca_mem_fffc_init(8);
     }
 
     if (gfxcard[0] == VID_INTERNAL)
-        ps2.mb_vga = (vga_t *)device_add(&ps1vga_mca_device);
+        ps2.mb_vga = (vga_t *) device_add(&ps1vga_mca_device);
 }
 
 static void
@@ -1837,7 +1947,7 @@ ps55_mca_board_model_50v_init(void)
     /* The slot 5 is reserved for the Integrated Fixed Disk II (an internal ESDI hard drive). */
     mca_init(5);
 
-    ps2.planar_read = ps55_model_50v_read;
+    ps2.planar_read  = ps55_model_50v_read;
     ps2.planar_write = ps55_model_50tv_write;
 
     device_add(&ps2_nvr_device);
@@ -1850,17 +1960,17 @@ ps55_mca_board_model_50v_init(void)
     ps2.has_e0000_hole = 1;
 
     mem_mapping_add(&ps2.split_mapping,
-        (mem_size + 256) * 1024,
-        256 * 1024,
-        ps2_read_split_ram,
-        ps2_read_split_ramw,
-        ps2_read_split_raml,
-        ps2_write_split_ram,
-        ps2_write_split_ramw,
-        ps2_write_split_raml,
-        &ram[0xa0000],
-        MEM_MAPPING_INTERNAL,
-        NULL);
+                    (mem_size + 256) * 1024,
+                    256 * 1024,
+                    ps2_read_split_ram,
+                    ps2_read_split_ramw,
+                    ps2_read_split_raml,
+                    ps2_write_split_ram,
+                    ps2_write_split_ramw,
+                    ps2_write_split_raml,
+                    &ram[0xa0000],
+                    MEM_MAPPING_INTERNAL,
+                    NULL);
     mem_mapping_disable(&ps2.split_mapping);
 
     mem_mapping_add(&ps2.cache_mapping,
@@ -1877,13 +1987,12 @@ ps55_mca_board_model_50v_init(void)
                     NULL);
     mem_mapping_disable(&ps2.cache_mapping);
 
-    if (mem_size > 8192) {
-        /* Only 8 MB supported on planar, create a memory expansion card for the rest */
-            ps2_mca_mem_fffc_init(8);
-    }
+    /* Only 8 MB supported on planar, create a memory expansion card for the rest */
+    if (mem_size > 8192)
+        ps2_mca_mem_fffc_init(8);
 
     if (gfxcard[0] == VID_INTERNAL)
-        ps2.mb_vga = (vga_t *)device_add(&ps1vga_mca_device);
+        ps2.mb_vga = (vga_t *) device_add(&ps1vga_mca_device);
 }
 
 int
@@ -1892,7 +2001,7 @@ machine_ps55_model_50t_init(const machine_t *model)
     int ret;
 
     ret = bios_load_linear("roms/machines/ibmps55_m50t/38F6933.BIN",
-        0x000e0000, 131072, 0);
+                           0x000e0000, 131072, 0);
 
     if (bios_only || !ret)
         return ret;
@@ -1900,15 +2009,15 @@ machine_ps55_model_50t_init(const machine_t *model)
     machine_ps2_common_init(model);
 
     /*
-    * Planar ID
-    * FFFAh - PS/55 model 5551-S0x, T0x (stage 1?)
-    * FFEEh - PS/55 model 5551-S1x, T1x (stage 2?)
-    * Verification in BIOS P/N 38F6933: FBxx -> 4 slots (error), xxEE -> 5 slots (ok), others -> 8 (error)
-    * 
-    * The only difference between S and T models is the CPU speed (16 MHz vs 20 MHz).
-    * The POST measures the speed, and sets a flag in the BIOS Data Area to indicate the sub model.
-    * The VM in 86Box runs faster than the real, so the POST always determines it as the T model.
-    */
+     * Planar ID
+     * FFFAh - PS/55 model 5551-S0x, T0x (stage 1?)
+     * FFEEh - PS/55 model 5551-S1x, T1x (stage 2?)
+     * Verification in BIOS P/N 38F6933: FBxx -> 4 slots (error), xxEE -> 5 slots (ok), others -> 8 (error)
+     *
+     * The only difference between S and T models is the CPU speed (16 MHz vs 20 MHz).
+     * The POST measures the speed, and sets a flag in the BIOS Data Area to indicate the sub model.
+     * The VM in 86Box runs faster than the real, so the POST always determines it as the T model.
+     */
     ps2.planar_id = 0xffee;
     ps55_mca_board_model_50t_init();
 
@@ -1932,10 +2041,10 @@ machine_ps55_model_50v_init(const machine_t *model)
     machine_ps2_common_init(model);
 
     /*
-    * Planar ID
-    * F1FFh - PS/55 model 5551-V0x, V1x
-    * Verification in BIOS P/N 56F7416,56F7417: FBxx -> 5 slots (ok), F1xx -> 5 slots (ok), others -> 8 (error)
-    */
+     * Planar ID
+     * F1FFh - PS/55 model 5551-V0x, V1x
+     * Verification in BIOS P/N 56F7416,56F7417: FBxx -> 5 slots (ok), F1xx -> 5 slots (ok), others -> 8 (error)
+     */
     ps2.planar_id = 0xf1ff;
     ps55_mca_board_model_50v_init();
 
