@@ -8,8 +8,6 @@
  *
  *          Implementation of an SST flash chip.
  *
- *
- *
  * Authors: Miran Grca, <mgrca8@gmail.com>
  *          Jasmine Iwanek, <jriwanek@gmail.com>
  *
@@ -129,6 +127,7 @@ static char flash_path[1024];
 #define WINBOND     0xda /* Winbond Manufacturer's ID */
 #define W29C512     0xc800
 #define W29C010     0xc100
+#define W29C011A    0xc100
 #define W29C020     0x4500
 #define W29C040     0x4600
 
@@ -524,7 +523,7 @@ sst_init(const device_t *info)
         dev->is_39    = 1;
 
     dev->size = info->local & 0xffff0000;
-    if ((dev->size == 0x20000) && (strstr(machine_get_internal_name_ex(machine), "xi8088")) && !xi8088_bios_128kb())
+    if ((dev->size == 0x20000) && ((machines[machine].init == machine_xt_xi8088_init) && !xi8088_bios_128kb()))
         dev->size = 0x10000;
 
     dev->mask         = dev->size - 1;
@@ -624,6 +623,20 @@ const device_t winbond_flash_w29c010_device = {
     .config        = NULL
 };
 
+const device_t winbond_flash_w29c011a_device = {
+    .name          = "Winbond W29C011A Flash BIOS",
+    .internal_name = "winbond_flash_w29c011a",
+    .flags         = 0,
+    .local         = WINBOND | W29C011A | SIZE_1M,
+    .init          = sst_init,
+    .close         = sst_close,
+    .reset         = NULL,
+    .available     = NULL,
+    .speed_changed = NULL,
+    .force_redraw  = NULL,
+    .config        = NULL
+};
+
 const device_t winbond_flash_w29c020_device = {
     .name          = "Winbond W29C020 Flash BIOS",
     .internal_name = "winbond_flash_w29c020",
