@@ -8,8 +8,6 @@
  *
  *          Main window module.
  *
- *
- *
  * Authors: Joakim L. Gilje <jgilje@jgilje.net>
  *          Cacodemon345
  *          Teemu Korhonen
@@ -198,7 +196,11 @@ MainWindow::MainWindow(QWidget *parent)
     frameRateTimer->setInterval(1000);
     frameRateTimer->setSingleShot(false);
     connect(frameRateTimer, &QTimer::timeout, [hertz_label] {
-        hertz_label->setText(tr("%1 Hz").arg(QString::number(monitors[0].mon_actualrenderedframes.load()) + (monitors[0].mon_interlace ? "i" : "")));
+        auto hz = monitors[0].mon_actualrenderedframes.load();
+#ifdef SCREENSHOT_MODE
+        hz = ((hz + 2) / 5) * 5;
+#endif
+        hertz_label->setText(tr("%1 Hz").arg(QString::number(hz) + (monitors[0].mon_interlace ? "i" : "")));
     });
     statusBar()->addPermanentWidget(hertz_label);
     frameRateTimer->start(1000);
@@ -530,6 +532,8 @@ MainWindow::MainWindow(QWidget *parent)
 #endif
         }
         ui->stackedWidget->switchRenderer(newVidApi);
+        ui->menuOpenGL_input_scale->setEnabled(newVidApi == RendererStack::Renderer::OpenGL3);
+        ui->menuOpenGL_input_stretch_mode->setEnabled(newVidApi == RendererStack::Renderer::OpenGL3);
         if (!show_second_monitors)
             return;
         for (int i = 1; i < MONITORS_NUM; i++) {
@@ -550,6 +554,41 @@ MainWindow::MainWindow(QWidget *parent)
             break;
         }
 
+    ui->action_0_5x_2->setChecked(video_gl_input_scale < 1.0);
+    ui->action_1x_2->setChecked(video_gl_input_scale >= 1.0 && video_gl_input_scale < 1.5);
+    ui->action1_5x_2->setChecked(video_gl_input_scale >= 1.5 && video_gl_input_scale < 2.0);
+    ui->action_2x_2->setChecked(video_gl_input_scale >= 2.0 && video_gl_input_scale < 3.0);
+    ui->action_3x_2->setChecked(video_gl_input_scale >= 3.0 && video_gl_input_scale < 4.0);
+    ui->action_4x_2->setChecked(video_gl_input_scale >= 4.0 && video_gl_input_scale < 5.0);
+    ui->action_5x_2->setChecked(video_gl_input_scale >= 5.0 && video_gl_input_scale < 6.0);
+    ui->action_6x_2->setChecked(video_gl_input_scale >= 6.0 && video_gl_input_scale < 7.0);
+    ui->action_7x_2->setChecked(video_gl_input_scale >= 7.0 && video_gl_input_scale < 8.0);
+    ui->action_8x_2->setChecked(video_gl_input_scale >= 8.0);
+
+    actGroup = new QActionGroup(this);
+    actGroup->addAction(ui->action_0_5x_2);
+    actGroup->addAction(ui->action_1x_2);
+    actGroup->addAction(ui->action1_5x_2);
+    actGroup->addAction(ui->action_2x_2);
+    actGroup->addAction(ui->action_3x_2);
+    actGroup->addAction(ui->action_4x_2);
+    actGroup->addAction(ui->action_5x_2);
+    actGroup->addAction(ui->action_6x_2);
+    actGroup->addAction(ui->action_7x_2);
+    actGroup->addAction(ui->action_8x_2);
+    connect(actGroup, &QActionGroup::triggered, this, [this](QAction* action) {
+        if (action == ui->action_0_5x_2) video_gl_input_scale = 0.5;
+        if (action == ui->action_1x_2) video_gl_input_scale = 1;
+        if (action == ui->action1_5x_2) video_gl_input_scale = 1.5;
+        if (action == ui->action_2x_2) video_gl_input_scale = 2;
+        if (action == ui->action_3x_2) video_gl_input_scale = 3;
+        if (action == ui->action_4x_2) video_gl_input_scale = 4;
+        if (action == ui->action_5x_2) video_gl_input_scale = 5;
+        if (action == ui->action_6x_2) video_gl_input_scale = 6;
+        if (action == ui->action_7x_2) video_gl_input_scale = 7;
+        if (action == ui->action_8x_2) video_gl_input_scale = 8;
+    });
+
     switch (scale) {
         default:
             break;
@@ -633,6 +672,38 @@ MainWindow::MainWindow(QWidget *parent)
     actGroup->addAction(ui->actionFullScreen_keepRatio);
     actGroup->addAction(ui->actionFullScreen_int);
     actGroup->addAction(ui->actionFullScreen_int43);
+    switch (video_gl_input_scale_mode) {
+        default:
+            break;
+        case FULLSCR_SCALE_FULL:
+            ui->action_Full_screen_stretch_gl->setChecked(true);
+            break;
+        case FULLSCR_SCALE_43:
+            ui->action_4_3_gl->setChecked(true);
+            break;
+        case FULLSCR_SCALE_KEEPRATIO:
+            ui->action_Square_pixels_keep_ratio_gl->setChecked(true);
+            break;
+        case FULLSCR_SCALE_INT:
+            ui->action_Integer_scale_gl->setChecked(true);
+            break;
+        case FULLSCR_SCALE_INT43:
+            ui->action4_3_Integer_scale_gl->setChecked(true);
+            break;
+    }
+    actGroup = new QActionGroup(this);
+    actGroup->addAction(ui->action_Full_screen_stretch_gl);
+    actGroup->addAction(ui->action_4_3_gl);
+    actGroup->addAction(ui->action_Square_pixels_keep_ratio_gl);
+    actGroup->addAction(ui->action_Integer_scale_gl);
+    actGroup->addAction(ui->action4_3_Integer_scale_gl);
+    connect(actGroup, &QActionGroup::triggered, this, [this](QAction* action) {
+        if (action == ui->action_Full_screen_stretch_gl) video_gl_input_scale_mode = FULLSCR_SCALE_FULL;
+        if (action == ui->action_4_3_gl) video_gl_input_scale_mode = FULLSCR_SCALE_43;
+        if (action == ui->action_Square_pixels_keep_ratio_gl) video_gl_input_scale_mode = FULLSCR_SCALE_KEEPRATIO;
+        if (action == ui->action_Integer_scale_gl) video_gl_input_scale_mode = FULLSCR_SCALE_INT;
+        if (action == ui->action4_3_Integer_scale_gl) video_gl_input_scale_mode = FULLSCR_SCALE_INT43;
+    });
     switch (video_grayscale) {
         default:
             break;
@@ -950,6 +1021,7 @@ void MainWindow::updateShortcuts()
 void
 MainWindow::resizeEvent(QResizeEvent *event)
 {
+#ifdef MOVE_WINDOW
     //qDebug() << pos().x() + event->size().width();
     //qDebug() << pos().y() + event->size().height();
     if (vid_resize == 1 || video_fullscreen)
@@ -969,6 +1041,7 @@ MainWindow::resizeEvent(QResizeEvent *event)
         if (newY < 1) newY = 1;
     }
     move(newX, newY);
+#endif
 }
 
 void
@@ -1437,7 +1510,8 @@ MainWindow::eventFilter(QObject *receiver, QEvent *event)
 		if (event->type() == QEvent::KeyPress)
 		{
 			QKeyEvent *ke = (QKeyEvent *) event;
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("release_mouse"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("release_mouse") ||
+                (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("release_mouse"))
 			{
 				plat_mouse_capture(0);
 			}
@@ -1447,31 +1521,38 @@ MainWindow::eventFilter(QObject *receiver, QEvent *event)
 		{
 			QKeyEvent *ke = (QKeyEvent *) event;
 			
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("screenshot"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("screenshot")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("screenshot"))
 			{
 				ui->actionTake_screenshot->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("fullscreen"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("fullscreen")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("fullscreen"))
 			{
 				ui->actionFullscreen->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("hard_reset"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("hard_reset")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("hard_reset"))
 			{
 				ui->actionHard_Reset->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("send_ctrl_alt_del"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("send_ctrl_alt_del")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("send_ctrl_alt_del"))
 			{
 				ui->actionCtrl_Alt_Del->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("send_ctrl_alt_esc"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("send_ctrl_alt_esc")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("send_ctrl_alt_esc"))
 			{
 				ui->actionCtrl_Alt_Esc->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("pause"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("pause")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("pause"))
 			{
 				ui->actionPause->trigger();
 			}
-			if ((QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("mute"))
+			if ((QKeySequence)(ke->key() | (ke->modifiers() & ~Qt::KeypadModifier)) == FindAcceleratorSeq("mute")
+                || (QKeySequence)(ke->key() | ke->modifiers()) == FindAcceleratorSeq("mute"))
 			{
 				ui->actionMute_Unmute->trigger();
 			}
@@ -1655,11 +1736,12 @@ MainWindow::focusOutEvent(QFocusEvent *event)
 void
 MainWindow::on_actionResizable_window_triggered(bool checked)
 {
+    hide();
     if (checked) {
         vid_resize = 1;
-        setWindowFlag(Qt::WindowMaximizeButtonHint, true);
-        setWindowFlag(Qt::MSWindowsFixedSizeDialogHint, false);
         setFixedSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+        setWindowFlag(Qt::MSWindowsFixedSizeDialogHint, false);
+        setWindowFlag(Qt::WindowMaximizeButtonHint, true);
         for (int i = 1; i < MONITORS_NUM; i++) {
             if (monitors[i].target_buffer) {
                 renderers[i]->setWindowFlag(Qt::WindowMaximizeButtonHint, true);
@@ -1997,6 +2079,15 @@ void
 MainWindow::on_actionForce_4_3_display_ratio_triggered()
 {
     video_toggle_option(ui->actionForce_4_3_display_ratio, &force_43);
+    if (vid_resize) {
+        const auto widget = ui->stackedWidget->currentWidget();
+        ui->stackedWidget->onResize(widget->width(), widget->height());
+
+        for (int i = 1; i < MONITORS_NUM; i++) {
+            if (renderers[i])
+                renderers[i]->onResize(renderers[i]->width(), renderers[i]->height());
+        }
+    }
 }
 
 void
@@ -2062,8 +2153,9 @@ MainWindow::on_actionHiDPI_scaling_triggered()
 void
 MainWindow::on_actionHide_status_bar_triggered()
 {
-    auto w = ui->stackedWidget->width();
-    auto h = ui->stackedWidget->height();
+    auto w = ui->stackedWidget->width() * (!dpi_scale ? util::screenOfWidget(this)->devicePixelRatio() : 1.);
+    auto h = ui->stackedWidget->height() * (!dpi_scale ? util::screenOfWidget(this)->devicePixelRatio() : 1.);
+
     hide_status_bar ^= 1;
     ui->actionHide_status_bar->setChecked(hide_status_bar);
     statusBar()->setVisible(!hide_status_bar);
@@ -2075,7 +2167,7 @@ MainWindow::on_actionHide_status_bar_triggered()
     } else {
         int vid_resize_orig = vid_resize;
         vid_resize          = 0;
-        emit resizeContents(w, h);
+        emit resizeContents(vid_resize_orig ? w : monitors[0].mon_scrnsz_x, vid_resize_orig ? h : monitors[0].mon_scrnsz_y);
         vid_resize = vid_resize_orig;
         if (vid_resize == 1)
             setFixedSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
@@ -2085,8 +2177,9 @@ MainWindow::on_actionHide_status_bar_triggered()
 void
 MainWindow::on_actionHide_tool_bar_triggered()
 {
-    auto w = ui->stackedWidget->width();
-    auto h = ui->stackedWidget->height();
+    auto w = ui->stackedWidget->width() * (!dpi_scale ? util::screenOfWidget(this)->devicePixelRatio() : 1.);
+    auto h = ui->stackedWidget->height() * (!dpi_scale ? util::screenOfWidget(this)->devicePixelRatio() : 1.);
+
     hide_tool_bar ^= 1;
     ui->actionHide_tool_bar->setChecked(hide_tool_bar);
     ui->toolBar->setVisible(!hide_tool_bar);
@@ -2095,7 +2188,7 @@ MainWindow::on_actionHide_tool_bar_triggered()
     } else {
         int vid_resize_orig = vid_resize;
         vid_resize          = 0;
-        emit resizeContents(w, h);
+        emit resizeContents(vid_resize_orig ? w : monitors[0].mon_scrnsz_x, vid_resize_orig ? h : monitors[0].mon_scrnsz_y);
         vid_resize = vid_resize_orig;
         if (vid_resize == 1)
             setFixedSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
@@ -2170,7 +2263,9 @@ void
 MainWindow::on_actionPreferences_triggered()
 {
     ProgSettings progsettings(this);
-    progsettings.exec();
+    if (progsettings.exec() == QDialog::Accepted) {
+        emit vmmGlobalConfigurationChanged();
+    }
 }
 
 void
@@ -2181,8 +2276,11 @@ MainWindow::on_actionEnable_Discord_integration_triggered(bool checked)
     if (enable_discord) {
         discord_init();
         discord_update_activity(dopause);
-    } else
+        discordupdate.start(1000);
+    } else {
         discord_close();
+        discordupdate.stop();
+    }
 #endif
 }
 
@@ -2210,7 +2308,14 @@ MainWindow::changeEvent(QEvent *event)
 {
 #ifdef Q_OS_WINDOWS
     if (event->type() == QEvent::LanguageChange) {
+        auto size = this->centralWidget()->size();
         QApplication::setFont(QFont(ProgSettings::getFontName(lang_id), 9));
+        QApplication::processEvents();
+        main_window->centralWidget()->setFixedSize(size);
+        QApplication::processEvents();
+        if (vid_resize == 1) {
+            main_window->centralWidget()->setFixedSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+        }
     }
 #endif
     QWidget::changeEvent(event);
