@@ -8,8 +8,6 @@
  *
  *          Common platform functions.
  *
- *
- *
  * Authors: Joakim L. Gilje <jgilje@jgilje.net>
  *          Cacodemon345
  *          Teemu Korhonen
@@ -18,7 +16,6 @@
  *          Copyright 2021-2022 Cacodemon345
  *          Copyright 2021-2022 Teemu Korhonen
  */
-
 #ifdef __HAIKU__
 #include <OS.h>
 #endif
@@ -64,6 +61,8 @@
 #    include <sys/mman.h>
 #endif
 
+#include <sys/stat.h>
+
 #ifdef Q_OS_OPENBSD
 #    include <pthread_np.h>
 #endif
@@ -250,6 +249,21 @@ plat_dir_check(char *path)
     return fi.isDir() ? 1 : 0;
 }
 
+int
+plat_file_check(const char *path)
+{
+#ifdef _WIN32
+    auto data = QString::fromUtf8(path).toStdWString();
+    auto res = GetFileAttributesW(data.c_str());
+    return (res != INVALID_FILE_ATTRIBUTES) && !(res & FILE_ATTRIBUTE_DIRECTORY);
+#else
+    struct stat stats;
+    if (stat(path, &stats) < 0)
+        return 0;
+    return !S_ISDIR(stats.st_mode);
+#endif
+}
+
 int
 plat_getcwd(char *bufp, int max)
 {
@@ -436,6 +450,65 @@ plat_munmap(void *ptr, size_t size)
 }
 
 extern bool cpu_thread_running;
+
+#ifdef Q_OS_WINDOWS
+/* SetThreadDescription was added in 14393 and SetProcessInformation in 8. Revisit if we ever start requiring 10. */
+static void *kernel32_handle = NULL;
+static HRESULT(WINAPI *pSetThreadDescription)(HANDLE hThread, PCWSTR lpThreadDescription) = NULL;
+static HRESULT(WINAPI *pSetProcessInformation)(HANDLE hProcess, PROCESS_INFORMATION_CLASS ProcessInformationClass, LPVOID ProcessInformation, DWORD ProcessInformationSize) = NULL;
+static dllimp_t kernel32_imports[] = {
+  // clang-format off
+    { "SetThreadDescription",  &pSetThreadDescription  },
+    { "SetProcessInformation", &pSetProcessInformation },
+    { NULL,                    NULL                    }
+  // clang-format on
+};
+
+static void
+enter_pause(void)
+{
+    PROCESS_POWER_THROTTLING_STATE state{};
+    state.Version = PROCESS_POWER_THROTTLING_CURRENT_VERSION;
+    state.ControlMask = PROCESS_POWER_THROTTLING_EXECUTION_SPEED | PROCESS_POWER_THROTTLING_IGNORE_TIMER_RESOLUTION;
+    state.StateMask = PROCESS_POWER_THROTTLING_EXECUTION_SPEED | PROCESS_POWER_THROTTLING_IGNORE_TIMER_RESOLUTION;
+
+    if (!kernel32_handle) {
+        kernel32_handle = dynld_module("kernel32.dll", kernel32_imports);
+        if (!kernel32_handle) {
+            kernel32_handle = kernel32_imports; /* store dummy pointer to avoid trying again */
+            pSetThreadDescription = NULL;
+            pSetProcessInformation = NULL;
+        }
+    }
+
+    if (pSetProcessInformation) {
+        pSetProcessInformation(GetCurrentProcess(), ProcessPowerThrottling, (LPVOID)&state, sizeof(state));
+    }
+}
+
+void
+exit_pause(void)
+{
+    PROCESS_POWER_THROTTLING_STATE state{};
+    state.Version = PROCESS_POWER_THROTTLING_CURRENT_VERSION;
+    state.ControlMask = PROCESS_POWER_THROTTLING_EXECUTION_SPEED | PROCESS_POWER_THROTTLING_IGNORE_TIMER_RESOLUTION;
+    state.StateMask = 0;
+
+    if (!kernel32_handle) {
+        kernel32_handle = dynld_module("kernel32.dll", kernel32_imports);
+        if (!kernel32_handle) {
+            kernel32_handle = kernel32_imports; /* store dummy pointer to avoid trying again */
+            pSetThreadDescription = NULL;
+            pSetProcessInformation = NULL;
+        }
+    }
+
+    if (pSetProcessInformation) {
+        pSetProcessInformation(GetCurrentProcess(), ProcessPowerThrottling, (LPVOID)&state, sizeof(state));
+    }
+}
+#endif
+
 void
 plat_pause(int p)
 {
@@ -460,6 +533,14 @@ plat_pause(int p)
     if ((p == 0) && (time_sync & TIME_SYNC_ENABLED))
         nvr_time_sync();
 
+#ifdef Q_OS_WINDOWS
+    if (p) {
+        enter_pause();
+    } else {
+        exit_pause();
+    }
+#endif
+
     do_pause(p);
     if (p) {
         if (mouse_capture)
@@ -614,13 +695,8 @@ c16stombs(char dst[], const uint16_t src[], int len)
 #endif
 
 #ifdef _WIN32
-#    if defined(__amd64__) || defined(_M_X64) || defined(__aarch64__) || defined(_M_ARM64)
-#        define LIB_NAME_GS   "gsdll64.dll"
-#        define LIB_NAME_GPCL "gpcl6dll64.dll"
-#    else
-#        define LIB_NAME_GS   "gsdll32.dll"
-#        define LIB_NAME_GPCL "gpcl6dll32.dll"
-#    endif
+#    define LIB_NAME_GS          "gsdll64.dll"
+#    define LIB_NAME_GPCL        "gpcl6dll64.dll"
 #    define LIB_NAME_PCAP        "Npcap"
 #else
 #    define LIB_NAME_GS          "libgs"
@@ -830,21 +906,12 @@ void
 plat_set_thread_name(void *thread, const char *name)
 {
 #ifdef Q_OS_WINDOWS
-    /* SetThreadDescription was added in 14393. Revisit if we ever start requiring 10. */
-    static void *kernel32_handle = NULL;
-    static HRESULT(WINAPI *pSetThreadDescription)(HANDLE hThread, PCWSTR lpThreadDescription) = NULL;
-    static dllimp_t kernel32_imports[] = {
-      // clang-format off
-        { "SetThreadDescription", &pSetThreadDescription },
-        { NULL,                   NULL                   }
-      // clang-format on
-    };
-
     if (!kernel32_handle) {
         kernel32_handle = dynld_module("kernel32.dll", kernel32_imports);
         if (!kernel32_handle) {
             kernel32_handle = kernel32_imports; /* store dummy pointer to avoid trying again */
             pSetThreadDescription = NULL;
+            pSetProcessInformation = NULL;
         }
     }
 
