@@ -40,6 +40,12 @@ private slots:
     void on_checkBoxDa2_stateChanged(int state);
     void on_pushButtonConfigureDa2_clicked();
 
+    void on_radioButtonDefault_clicked();
+
+    void on_radioButtonCustom_clicked();
+
+    void on_pushButtonExportDefault_clicked();
+
 private:
     Ui::SettingsDisplay *ui;
     int                  machineId    = 0;
