@@ -8,8 +8,6 @@
  *
  *          Floppy/CD-ROM devices configuration UI module.
  *
- *
- *
  * Authors: Joakim L. Gilje <jgilje@jgilje.net>
  *          Cacodemon345
  *
@@ -32,44 +30,44 @@ extern "C" {
 #include <86box/timer.h>
 #include <86box/fdd.h>
 #include <86box/cdrom.h>
+#include <86box/fdd_audio.h>
 }
 
-#include <QStandardItemModel>
-
 #include "qt_models_common.hpp"
 #include "qt_harddrive_common.hpp"
 #include "qt_settings_bus_tracking.hpp"
 #include "qt_progsettings.hpp"
 
-static void
-setFloppyType(QAbstractItemModel *model, const QModelIndex &idx, int type)
+void
+SettingsFloppyCDROM::setFloppyType(QAbstractItemModel *model, const QModelIndex &idx, int type)
 {
     QIcon icon;
     if (type == 0)
-        icon = ProgSettings::loadIcon("/floppy_disabled.ico");
+        icon = floppy_disabled_icon;
     else if (type >= 1 && type <= 6)
-        icon = ProgSettings::loadIcon("/floppy_525.ico");
+        icon = floppy_525_icon;
     else
-        icon = ProgSettings::loadIcon("/floppy_35.ico");
+        icon = floppy_35_icon;
 
     model->setData(idx, QObject::tr(fdd_getname(type)));
     model->setData(idx, type, Qt::UserRole);
     model->setData(idx, icon, Qt::DecorationRole);
 }
 
-static void
-setCDROMBus(QAbstractItemModel *model, const QModelIndex &idx, uint8_t bus, uint8_t channel)
+void
+SettingsFloppyCDROM::setCDROMBus(QAbstractItemModel *model, const QModelIndex &idx, uint8_t bus, uint8_t channel)
 {
     QIcon icon;
 
     switch (bus) {
         case CDROM_BUS_DISABLED:
-            icon = ProgSettings::loadIcon("/cdrom_disabled.ico");
+            icon = cdrom_disabled_icon;
             break;
         case CDROM_BUS_ATAPI:
         case CDROM_BUS_SCSI:
         case CDROM_BUS_MITSUMI:
-            icon = ProgSettings::loadIcon("/cdrom.ico");
+        case CDROM_BUS_MKE:
+            icon = cdrom_icon;
             break;
     }
 
@@ -90,6 +88,14 @@ setCDROMSpeed(QAbstractItemModel *model, const QModelIndex &idx, uint8_t speed)
     model->setData(i, speed, Qt::UserRole);
 }
 
+static QString
+CDROMName(int type)
+{
+    char temp[512];
+    cdrom_get_name(type, temp);
+    return QObject::tr((const char *) temp);
+}
+
 static void
 setCDROMType(QAbstractItemModel *model, const QModelIndex &idx, int type)
 {
@@ -97,7 +103,7 @@ setCDROMType(QAbstractItemModel *model, const QModelIndex &idx, int type)
     if (idx.siblingAtColumn(0).data(Qt::UserRole).toUInt() == CDROM_BUS_DISABLED)
         model->setData(i, QCoreApplication::translate("", "None"));
     else if (idx.siblingAtColumn(0).data(Qt::UserRole).toUInt() != CDROM_BUS_MITSUMI)
-        model->setData(i, QObject::tr(cdrom_getname(type)));
+        model->setData(i, CDROMName(type));
     model->setData(i, type, Qt::UserRole);
 }
 
@@ -107,6 +113,10 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
 {
     ui->setupUi(this);
 
+    floppy_disabled_icon = QIcon(":/settings/qt/icons/floppy_disabled.ico");
+    floppy_525_icon = QIcon(":/settings/qt/icons/floppy_525.ico");
+    floppy_35_icon = QIcon(":/settings/qt/icons/floppy_35.ico");
+
     auto *model = ui->comboBoxFloppyType->model();
     int   i     = 0;
     while (true) {
@@ -118,13 +128,14 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
         ++i;
     }
 
-    model = new QStandardItemModel(0, 3, this);
+    model = new QStandardItemModel(0, 4, this);
     ui->tableViewFloppy->setModel(model);
     model->setHeaderData(0, Qt::Horizontal, tr("Type"));
     model->setHeaderData(1, Qt::Horizontal, tr("Turbo"));
     model->setHeaderData(2, Qt::Horizontal, tr("Check BPB"));
+    model->setHeaderData(3, Qt::Horizontal, tr("Audio"));
 
-    model->insertRows(0, FDD_NUM);
+model->insertRows(0, FDD_NUM);
     /* Floppy drives category */
     for (int i = 0; i < FDD_NUM; i++) {
         auto idx  = model->index(i, 0);
@@ -132,6 +143,25 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
         setFloppyType(model, idx, type);
         model->setData(idx.siblingAtColumn(1), fdd_get_turbo(i) > 0 ? tr("On") : tr("Off"));
         model->setData(idx.siblingAtColumn(2), fdd_get_check_bpb(i) > 0 ? tr("On") : tr("Off"));
+
+        int     prof = fdd_get_audio_profile(i);
+        QString profName;
+
+#ifndef DISABLE_FDD_AUDIO
+        // Get the profile name from the configuration system
+        const char *name = fdd_audio_get_profile_internal_name(prof);
+        if (name) {
+            profName = QString(name);
+        } else {
+            profName = tr("None");
+        }
+#else
+        profName = tr("None");
+#endif
+
+        auto audioIdx = model->index(i, 3);
+        model->setData(audioIdx, profName);
+        model->setData(audioIdx, prof, Qt::UserRole);
     }
 
     ui->tableViewFloppy->resizeColumnsToContents();
@@ -139,9 +169,30 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
 
     connect(ui->tableViewFloppy->selectionModel(), &QItemSelectionModel::currentRowChanged,
             this, &SettingsFloppyCDROM::onFloppyRowChanged);
-    ui->tableViewFloppy->setCurrentIndex(model->index(0, 0));
+    
+#ifndef DISABLE_FDD_AUDIO
+    ui->comboBoxFloppyAudio->setVisible(true);
+    int profile_count = fdd_audio_get_profile_count();
+    for (int i = 0; i < profile_count; i++) {
+        const char *name = fdd_audio_get_profile_name(i);
+        if (name) {
+            ui->comboBoxFloppyAudio->addItem(name, i);
+        }
+    }
+    ui->comboBoxFloppyAudio->setSizeAdjustPolicy(QComboBox::AdjustToContents);
+#else
+    ui->comboBoxFloppyAudio->setVisible(false);
+#endif
 
-    Harddrives::populateRemovableBuses(ui->comboBoxBus->model());
+    // Set initial selection and trigger the row changed event to update controls
+    ui->tableViewFloppy->setCurrentIndex(model->index(0, 0));
+    // Manually trigger the row changed event to ensure audio selection is updated
+    onFloppyRowChanged(model->index(0, 0));
+
+    cdrom_disabled_icon = QIcon(":/settings/qt/icons/cdrom_disabled.ico");
+    cdrom_icon = QIcon(":/settings/qt/icons/cdrom.ico");
+
+    Harddrives::populateCDROMBuses(ui->comboBoxBus->model());
     model = ui->comboBoxSpeed->model();
     for (int i = 0; i < 72; i++)
         Models::AddEntry(model, QString("%1x").arg(i + 1), i + 1);
@@ -156,13 +207,18 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
         auto idx = model->index(i, 0);
         int type = cdrom_get_type(i);
         setCDROMBus(model, idx, cdrom[i].bus_type, cdrom[i].res);
-        setCDROMSpeed(model, idx.siblingAtColumn(1), cdrom[i].speed);
         setCDROMType(model, idx.siblingAtColumn(2), type);
-        if (cdrom[i].bus_type == CDROM_BUS_ATAPI)
+        int speed = cdrom_get_speed(type);
+        if (speed == -1)
+            setCDROMSpeed(model, idx.siblingAtColumn(1), cdrom[i].speed);
+        else
+            setCDROMSpeed(model, idx.siblingAtColumn(1), speed);
+        if (cdrom[i].bus_type == CDROM_BUS_MKE)
+            Harddrives::busTrackClass->device_track(1, DEV_CDROM, cdrom[i].bus_type, cdrom[i].mke_channel);
+        else if (cdrom[i].bus_type == CDROM_BUS_ATAPI)
             Harddrives::busTrackClass->device_track(1, DEV_CDROM, cdrom[i].bus_type, cdrom[i].ide_channel);
         else if (cdrom[i].bus_type == CDROM_BUS_SCSI)
-            Harddrives::busTrackClass->device_track(1, DEV_CDROM, cdrom[i].bus_type,
-                                                    cdrom[i].scsi_device_id);
+            Harddrives::busTrackClass->device_track(1, DEV_CDROM, cdrom[i].bus_type, cdrom[i].scsi_device_id);
         else if (cdrom[i].bus_type == CDROM_BUS_MITSUMI)
             Harddrives::busTrackClass->device_track(1, DEV_CDROM, cdrom[i].bus_type, 0);
     }
@@ -183,12 +239,13 @@ SettingsFloppyCDROM::SettingsFloppyCDROM(QWidget *parent)
     int selectedTypeRow      = 0;
     int eligibleRows         = 0;
     while (cdrom_drive_types[j].bus_type != BUS_TYPE_NONE) {
-        if (((bus_type == CDROM_BUS_ATAPI) || (bus_type == CDROM_BUS_SCSI)) &&
+        if (((bus_type == CDROM_BUS_MKE) || (bus_type == CDROM_BUS_ATAPI) ||
+             (bus_type == CDROM_BUS_SCSI)) &&
             ((cdrom_drive_types[j].bus_type == bus_type) ||
-             (cdrom_drive_types[j].bus_type == BUS_TYPE_BOTH))) {
-            QString name = tr(cdrom_getname(j));
+             ((cdrom_drive_types[j].bus_type == BUS_TYPE_BOTH) && (bus_type != BUS_TYPE_MKE)))) {
+            QString name = CDROMName(j);
             Models::AddEntry(modelType, name, j);
-            if ((cdrom[cdromIdx].bus_type == bus_type) && (cdrom[cdromIdx].type == j))
+            if (cdrom[cdromIdx].type == j)
                 selectedTypeRow = eligibleRows;
             ++eligibleRows;
         }
@@ -213,6 +270,9 @@ SettingsFloppyCDROM::save()
         fdd_set_type(i, model->index(i, 0).data(Qt::UserRole).toInt());
         fdd_set_turbo(i, model->index(i, 1).data() == tr("On") ? 1 : 0);
         fdd_set_check_bpb(i, model->index(i, 2).data() == tr("On") ? 1 : 0);
+#ifndef DISABLE_FDD_AUDIO
+        fdd_set_audio_profile(i, model->index(i, 3).data(Qt::UserRole).toInt());
+#endif
     }
 
     /* Removable devices category */
@@ -220,7 +280,7 @@ SettingsFloppyCDROM::save()
     for (int i = 0; i < CDROM_NUM; i++) {
         cdrom[i].priv = NULL;
         cdrom[i].ops = NULL;
-        cdrom[i].image = NULL;
+        cdrom[i].local = NULL;
         cdrom[i].insert = NULL;
         cdrom[i].close = NULL;
         cdrom[i].get_volume = NULL;
@@ -230,6 +290,12 @@ SettingsFloppyCDROM::save()
         cdrom[i].speed = model->index(i, 1).data(Qt::UserRole).toUInt();
         cdrom_set_type(i, model->index(i, 2).data(Qt::UserRole).toInt());
     }
+
+#ifdef DISABLE_FDD_AUDIO
+    fdd_sounds_enabled = 0;
+#else
+    fdd_sounds_enabled = 1;
+#endif
 }
 
 void
@@ -239,15 +305,18 @@ SettingsFloppyCDROM::onFloppyRowChanged(const QModelIndex &current)
     ui->comboBoxFloppyType->setCurrentIndex(type);
     ui->checkBoxTurboTimings->setChecked(current.siblingAtColumn(1).data() == tr("On"));
     ui->checkBoxCheckBPB->setChecked(current.siblingAtColumn(2).data() == tr("On"));
+
+    int prof = current.siblingAtColumn(3).data(Qt::UserRole).toInt();
+    int comboIndex = ui->comboBoxFloppyAudio->findData(prof);
+    ui->comboBoxFloppyAudio->setCurrentIndex(comboIndex);
 }
 
 void
 SettingsFloppyCDROM::onCDROMRowChanged(const QModelIndex &current)
 {
-    uint8_t bus     = current.siblingAtColumn(0).data(Qt::UserRole).toUInt();
-    uint8_t channel = current.siblingAtColumn(0).data(Qt::UserRole + 1).toUInt();
-    uint8_t speed   = current.siblingAtColumn(1).data(Qt::UserRole).toUInt();
-    int type        = current.siblingAtColumn(2).data(Qt::UserRole).toInt();
+    uint8_t  bus     = current.siblingAtColumn(0).data(Qt::UserRole).toUInt();
+    uint8_t  channel = current.siblingAtColumn(0).data(Qt::UserRole + 1).toUInt();
+    uint32_t type    = current.siblingAtColumn(2).data(Qt::UserRole).toUInt();
 
     ui->comboBoxBus->setCurrentIndex(-1);
     auto* model = ui->comboBoxBus->model();
@@ -260,8 +329,38 @@ SettingsFloppyCDROM::onCDROMRowChanged(const QModelIndex &current)
     if (!match.isEmpty())
         ui->comboBoxChannel->setCurrentIndex(match.first().row());
 
+    int     speed   = cdrom_get_speed(type);
+    if (speed == -1) {
+        speed   = current.siblingAtColumn(1).data(Qt::UserRole).toUInt();
+        ui->comboBoxSpeed->setEnabled((bus == CDROM_BUS_DISABLED) ? false : true);
+    } else
+        ui->comboBoxSpeed->setEnabled(false);
     ui->comboBoxSpeed->setCurrentIndex(speed == 0 ? 7 : speed - 1);
-    ui->comboBoxCDROMType->setCurrentIndex(type);
+
+    auto *modelType  = ui->comboBoxCDROMType->model();
+    int   removeRows = modelType->rowCount();
+
+    uint32_t j               = 0;
+    int selectedTypeRow      = 0;
+    int eligibleRows         = 0;
+    while (cdrom_drive_types[j].bus_type != BUS_TYPE_NONE) {
+        if (((bus == CDROM_BUS_MKE) || (bus == CDROM_BUS_ATAPI) ||
+             (bus == CDROM_BUS_SCSI)) &&
+            ((cdrom_drive_types[j].bus_type == bus) ||
+             ((cdrom_drive_types[j].bus_type == BUS_TYPE_BOTH) && (bus != BUS_TYPE_MKE)))) {
+            QString name = CDROMName(j);
+            Models::AddEntry(modelType, name, j);
+            if (type == j)
+                selectedTypeRow = eligibleRows;
+            ++eligibleRows;
+        }
+        ++j;
+    }
+    modelType->removeRows(0, removeRows);
+    ui->comboBoxCDROMType->setEnabled(eligibleRows > 1);
+    ui->comboBoxCDROMType->setCurrentIndex(-1);
+    ui->comboBoxCDROMType->setCurrentIndex(selectedTypeRow);
+
     enableCurrentlySelectedChannel();
 }
 
@@ -269,7 +368,7 @@ void
 SettingsFloppyCDROM::on_checkBoxTurboTimings_stateChanged(int arg1)
 {
     auto idx = ui->tableViewFloppy->selectionModel()->currentIndex();
-    ui->tableViewFloppy->model()->setData(idx.siblingAtColumn(1), arg1 == Qt::Checked ?
+    ui->tableViewFloppy->model()->setData(idx.siblingAtColumn(1), arg1 == Qt::Checked ? 
                                           tr("On") : tr("Off"));
 }
 
@@ -277,10 +376,11 @@ void
 SettingsFloppyCDROM::on_checkBoxCheckBPB_stateChanged(int arg1)
 {
     auto idx = ui->tableViewFloppy->selectionModel()->currentIndex();
-    ui->tableViewFloppy->model()->setData(idx.siblingAtColumn(2), arg1 == Qt::Checked ?
+    ui->tableViewFloppy->model()->setData(idx.siblingAtColumn(2), arg1 == Qt::Checked ? 
                                           tr("On") : tr("Off"));
 }
 
+
 void
 SettingsFloppyCDROM::on_comboBoxFloppyType_activated(int index)
 {
@@ -288,6 +388,30 @@ SettingsFloppyCDROM::on_comboBoxFloppyType_activated(int index)
                   ui->tableViewFloppy->selectionModel()->currentIndex(), index);
 }
 
+void
+SettingsFloppyCDROM::on_comboBoxFloppyAudio_activated(int)
+{
+    auto    idx  = ui->tableViewFloppy->selectionModel()->currentIndex();
+    int     prof = ui->comboBoxFloppyAudio->currentData().toInt();
+    QString profName;
+
+#ifndef DISABLE_FDD_AUDIO
+    // Get the profile name from the configuration system
+    const char *name = fdd_audio_get_profile_internal_name(prof);
+    if (name) {
+        profName = name;
+    } else {
+        profName = tr("None");
+    }
+#else
+    profName = tr("None");
+#endif
+
+    auto audioIdx = idx.siblingAtColumn(3);
+    ui->tableViewFloppy->model()->setData(audioIdx, profName);
+    ui->tableViewFloppy->model()->setData(audioIdx, prof, Qt::UserRole);
+}
+
 void SettingsFloppyCDROM::reloadBusChannels() {
     auto selected = ui->comboBoxChannel->currentIndex();
     Harddrives::populateBusChannels(ui->comboBoxChannel->model(), ui->comboBoxBus->currentData().toInt(), Harddrives::busTrackClass);
@@ -319,14 +443,16 @@ SettingsFloppyCDROM::on_comboBoxSpeed_activated(int index)
 void
 SettingsFloppyCDROM::on_comboBoxBus_activated(int)
 {
-    auto i = ui->tableViewCDROM->selectionModel()->currentIndex().siblingAtColumn(0);
+    auto    i        = ui->tableViewCDROM->selectionModel()->currentIndex().siblingAtColumn(0);
     uint8_t bus_type = ui->comboBoxBus->currentData().toUInt();
-    int cdromIdx     = ui->tableViewCDROM->selectionModel()->currentIndex().data().toInt();
+    int     cdromIdx = ui->tableViewCDROM->selectionModel()->currentIndex().data().toInt();
 
     Harddrives::busTrackClass->device_track(0, DEV_CDROM, ui->tableViewCDROM->model()->data(i,
                                             Qt::UserRole).toInt(), ui->tableViewCDROM->model()->data(i,
                                             Qt::UserRole + 1).toInt());
-    if (bus_type == CDROM_BUS_ATAPI)
+    if (bus_type == CDROM_BUS_MKE)
+        ui->comboBoxChannel->setCurrentIndex(Harddrives::busTrackClass->next_free_mke_channel());
+    else if (bus_type == CDROM_BUS_ATAPI)
         ui->comboBoxChannel->setCurrentIndex(Harddrives::busTrackClass->next_free_ide_channel());
     else if (bus_type == CDROM_BUS_SCSI)
         ui->comboBoxChannel->setCurrentIndex(Harddrives::busTrackClass->next_free_scsi_id());
@@ -344,16 +470,17 @@ SettingsFloppyCDROM::on_comboBoxBus_activated(int)
     auto *modelType  = ui->comboBoxCDROMType->model();
     int   removeRows = modelType->rowCount();
 
-    uint32_t j               = 0;
-    int selectedTypeRow      = 0;
-    int eligibleRows         = 0;
+    uint32_t j          = 0;
+    int selectedTypeRow = 0;
+    int eligibleRows    = 0;
     while (cdrom_drive_types[j].bus_type != BUS_TYPE_NONE) {
-        if (((bus_type == CDROM_BUS_ATAPI) || (bus_type == CDROM_BUS_SCSI)) &&
+        if (((bus_type == CDROM_BUS_MKE) || (bus_type == CDROM_BUS_ATAPI) ||
+             (bus_type == CDROM_BUS_SCSI)) &&
             ((cdrom_drive_types[j].bus_type == bus_type) ||
-             (cdrom_drive_types[j].bus_type == BUS_TYPE_BOTH))) {
-            QString name = tr(cdrom_getname(j));
+             ((cdrom_drive_types[j].bus_type == BUS_TYPE_BOTH) && (bus_type != BUS_TYPE_MKE)))) {
+            QString name = CDROMName(j);
             Models::AddEntry(modelType, name, j);
-            if ((cdrom[cdromIdx].bus_type == bus_type) && (cdrom[cdromIdx].type == j))
+            if (cdrom[cdromIdx].type == j)
                 selectedTypeRow = eligibleRows;
             ++eligibleRows;
         }
@@ -367,6 +494,20 @@ SettingsFloppyCDROM::on_comboBoxBus_activated(int)
     setCDROMType(ui->tableViewCDROM->model(),
                  ui->tableViewCDROM->selectionModel()->currentIndex(),
                  ui->comboBoxCDROMType->currentData().toUInt());
+
+    int speed = cdrom_get_speed(ui->comboBoxCDROMType->currentData().toUInt());
+    if ((speed == -1) && (bus_type != CDROM_BUS_MITSUMI)) {
+        speed = ui->comboBoxSpeed->currentData().toUInt();
+        ui->comboBoxSpeed->setEnabled(bus_type != CDROM_BUS_DISABLED);
+    } else {
+        ui->comboBoxSpeed->setEnabled(false);
+        if (bus_type == CDROM_BUS_MITSUMI) // temp hack
+            speed = 0; 
+    }
+    ui->comboBoxSpeed->setCurrentIndex(speed == 0 ? 7 : speed - 1);
+    setCDROMSpeed(ui->tableViewCDROM->model(),
+                  ui->tableViewCDROM->selectionModel()->currentIndex(),
+                  speed);
     emit cdromChannelChanged();
 }
 
@@ -374,11 +515,10 @@ void
 SettingsFloppyCDROM::enableCurrentlySelectedChannel()
 {
     const auto *item_model = qobject_cast<QStandardItemModel*>(ui->comboBoxChannel->model());
-    const auto index = ui->comboBoxChannel->currentIndex();
-    auto *item = item_model->item(index);
-    if(item) {
+    const auto  index      = ui->comboBoxChannel->currentIndex();
+    auto       *item       = item_model->item(index);
+    if(item)
         item->setEnabled(true);
-    }
 }
 
 void
@@ -401,9 +541,22 @@ SettingsFloppyCDROM::on_comboBoxChannel_activated(int)
 void
 SettingsFloppyCDROM::on_comboBoxCDROMType_activated(int)
 {
+    int type = ui->comboBoxCDROMType->currentData().toUInt();
+
     setCDROMType(ui->tableViewCDROM->model(),
                  ui->tableViewCDROM->selectionModel()->currentIndex(),
-                 ui->comboBoxCDROMType->currentData().toUInt());
+                 type);
     ui->tableViewCDROM->resizeColumnsToContents();
     ui->tableViewCDROM->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
+
+    int speed = cdrom_get_speed(type);
+    if (speed == -1) {
+        speed = ui->comboBoxSpeed->currentData().toUInt();
+        ui->comboBoxSpeed->setEnabled(true);
+    } else
+        ui->comboBoxSpeed->setEnabled(false);
+    ui->comboBoxSpeed->setCurrentIndex(speed == 0 ? 7 : speed - 1);
+
+    auto idx = ui->tableViewCDROM->selectionModel()->currentIndex();
+    setCDROMSpeed(ui->tableViewCDROM->model(), idx.siblingAtColumn(1), speed);
 }
