@@ -1,20 +1,17 @@
 /*
-* 86Box	A hypervisor and IBM PC system emulator that specializes in
-*		running old operating systems and software designed for IBM
-*		PC systems and compatibles from 1981 through fairly recent
-*		system designs based on the PCI bus.
-*
-*		This file is part of the 86Box distribution.
-*
-*		86Box VM manager main module
-*
-*
-*
-* Authors:	cold-brewed
-*
-*		Copyright 2024 cold-brewed
-*/
-
+ * 86Box    A hypervisor and IBM PC system emulator that specializes in
+ *          running old operating systems and software designed for IBM
+ *          PC systems and compatibles from 1981 through fairly recent
+ *          system designs based on the PCI bus.
+ *
+ *          This file is part of the 86Box distribution.
+ *
+ *          86Box VM manager main module
+ *
+ * Authors: cold-brewed
+ *
+ *          Copyright 2024 cold-brewed
+ */
 #include <QDirIterator>
 #include <QLabel>
 #include <QAbstractListModel>
@@ -31,10 +28,13 @@
 #include <atomic>
 
 #include "qt_vmmanager_main.hpp"
+#include "qt_vmmanager_mainwindow.hpp"
 #include "ui_qt_vmmanager_main.h"
 #include "qt_vmmanager_model.hpp"
 #include "qt_vmmanager_addmachine.hpp"
 
+extern VMManagerMainWindow* vmm_main_window;
+
 // https://stackoverflow.com/a/36460740
 bool copyPath(QString sourceDir, QString destinationDir, bool overWriteDirectory)
 {
@@ -106,6 +106,53 @@ VMManagerMain::VMManagerMain(QWidget *parent) :
         if (indexAt.isValid()) {
             QMenu contextMenu(tr("Context Menu"), ui->listView);
 
+            QAction startAction(tr("&Start"));
+            contextMenu.addAction(&startAction);
+            connect(&startAction, &QAction::triggered, [this] {
+                selected_sysconfig->startButtonPressed();
+            });
+            startAction.setEnabled(selected_sysconfig->process->state() == QProcess::NotRunning);
+            startAction.setVisible(selected_sysconfig->process->state() == QProcess::NotRunning);
+
+            QAction pauseAction(tr("&Pause"));
+            contextMenu.addAction(&pauseAction);
+            connect(&pauseAction, &QAction::triggered, [this] {
+                selected_sysconfig->pauseButtonPressed();
+            });
+            pauseAction.setEnabled(selected_sysconfig->process->state() == QProcess::Running);
+            pauseAction.setVisible(selected_sysconfig->process->state() == QProcess::Running);
+            if (selected_sysconfig->getProcessStatus() != VMManagerSystem::ProcessStatus::Running)
+                pauseAction.setText(tr("Re&sume"));
+
+            QAction resetAction(tr("&Hard reset"));
+            contextMenu.addAction(&resetAction);
+            connect(&resetAction, &QAction::triggered, [this] {
+                selected_sysconfig->restartButtonPressed();
+            });
+            resetAction.setEnabled(selected_sysconfig->process->state() == QProcess::Running);
+
+            QAction forceShutdownAction(tr("&Force shutdown"));
+            contextMenu.addAction(&forceShutdownAction);
+            connect(&forceShutdownAction, &QAction::triggered, [this] {
+                selected_sysconfig->shutdownForceButtonPressed();
+            });
+            forceShutdownAction.setEnabled(selected_sysconfig->process->state() == QProcess::Running);
+
+            QAction cadAction(tr("&Ctrl+Alt+Del"));
+            contextMenu.addAction(&cadAction);
+            connect(&cadAction, &QAction::triggered, [this] {
+                selected_sysconfig->cadButtonPressed();
+            });
+            cadAction.setEnabled(selected_sysconfig->process->state() == QProcess::Running);
+
+            contextMenu.addSeparator();
+
+            QAction settingsAction(tr("&Settings..."));
+            contextMenu.addAction(&settingsAction);
+            connect(&settingsAction, &QAction::triggered, [this] {
+                selected_sysconfig->launchSettings();
+            });
+
             QAction nameChangeAction(tr("Change &display name..."));
             contextMenu.addAction(&nameChangeAction);
             // Use a lambda to call a function so indexAt can be passed
@@ -304,7 +351,7 @@ illegal_chars:
                 }
             });
 
-            QAction openPrinterFolderAction(tr("Open &printer tray..."));
+            QAction openPrinterFolderAction(tr("Open p&rinter tray..."));
             contextMenu.addAction(&openPrinterFolderAction);
             connect(&openPrinterFolderAction, &QAction::triggered, [indexAt] {
                 if (const auto printerDir = indexAt.data(VMManagerModel::Roles::ConfigDir).toString() + QString("/printer/"); !printerDir.isEmpty()) {
@@ -340,7 +387,7 @@ illegal_chars:
         } else {
             QMenu contextMenu(tr("Context Menu"), ui->listView);
 
-            QAction newMachineAction(tr("New machine..."));
+            QAction newMachineAction(tr("&New machine..."));
             contextMenu.addAction(&newMachineAction);
             connect(&newMachineAction, &QAction::triggered, this, &VMManagerMain::newMachineWizard);
 
@@ -348,6 +395,10 @@ illegal_chars:
         }
     });
 
+    connect(vm_model, &VMManagerModel::globalConfigurationChanged, this, [] () {
+        vmm_main_window->updateSettings();
+    });
+
     // Initial default details view
     vm_details = new VMManagerDetails(ui->detailsArea);
     ui->detailsArea->layout()->addWidget(vm_details);
@@ -400,6 +451,12 @@ VMManagerMain::~VMManagerMain() {
     delete vm_model;
 }
 
+void
+VMManagerMain::updateGlobalSettings()
+{
+    vmm_main_window->updateSettings();
+}
+
 void
 VMManagerMain::currentSelectionChanged(const QModelIndex &current,
                              const QModelIndex &previous)
@@ -727,6 +784,18 @@ VMManagerMain::machineCountString(QString states) const
     return tr("VMs: %1").arg(states);
 }
 
+QList<int>
+VMManagerMain::getPaneSizes() const
+{
+    return ui->splitter->sizes();
+}
+
+void
+VMManagerMain::setPaneSizes(const QList<int> &sizes)
+{
+    ui->splitter->setSizes(sizes);
+}
+
 void
 VMManagerMain::modelDataChange()
 {
@@ -768,6 +837,10 @@ VMManagerMain::onPreferencesUpdated()
     if (oldRegexSearch != regexSearch) {
         ui->searchBar->clear();
     }
+
+    if (vm_model) {
+        vm_model->sendGlobalConfigurationChanged();
+    }
 }
 
 void
