@@ -46,6 +46,7 @@
 #define HAVE_STDARG_H
 #include <86box/86box.h>
 #include <86box/sound.h>
+#include "cpu.h"
 #include <86box/timer.h>
 #include <86box/device.h>
 #include <86box/snd_opl.h>
@@ -68,9 +69,6 @@
 
 #define RSM_FRAC    10
 
-// #define OPL_FREQ FREQ_48000
-#define OPL_FREQ FREQ_49716
-
 // Channel types
 enum {
     ch_2op  = 0,
@@ -1276,7 +1274,7 @@ OPL3_Reset(opl3_chip *chip, uint32_t samplerate)
     }
 
     chip->noise        = 1;
-    chip->rateratio    = (samplerate << RSM_FRAC) / 49716;
+    chip->rateratio    = (samplerate << RSM_FRAC) / FREQ_49716;
     chip->tremoloshift = 4;
     chip->vibshift     = 1;
 
@@ -1462,6 +1460,15 @@ OPL3_GenerateStream(opl3_chip *chip, int32_t *sndptr, uint32_t numsamples)
     }
 }
 
+void
+OPL3_GenerateResampledStream(opl3_chip *chip, int32_t *sndptr, uint32_t numsamples)
+{
+    for (uint_fast32_t i = 0; i < numsamples; i++) {
+        OPL3_GenerateResampled(chip, sndptr);
+        sndptr += 2;
+    }
+}
+
 static void
 nuked_timer_tick(nuked_drv_t *dev, int tmr)
 {
@@ -1527,32 +1534,6 @@ nuked_drv_set_do_cycles(void *priv, int8_t do_cycles)
         dev->flags &= ~FLAG_CYCLES;
 }
 
-static void *
-nuked_drv_init(const device_t *info)
-{
-    nuked_drv_t *dev = (nuked_drv_t *) calloc(1, sizeof(nuked_drv_t));
-    dev->flags       = FLAG_CYCLES;
-    if (info->local == FM_YMF262)
-        dev->flags |= FLAG_OPL3;
-    else
-        dev->status = 0x06;
-
-    /* Initialize the NukedOPL object. */
-    OPL3_Reset(&dev->opl, OPL_FREQ);
-
-    timer_add(&dev->timers[0], nuked_timer_1, dev, 0);
-    timer_add(&dev->timers[1], nuked_timer_2, dev, 0);
-
-    return dev;
-}
-
-static void
-nuked_drv_close(void *priv)
-{
-    nuked_drv_t *dev = (nuked_drv_t *) priv;
-    free(dev);
-}
-
 static int32_t *
 nuked_drv_update(void *priv)
 {
@@ -1562,8 +1543,8 @@ nuked_drv_update(void *priv)
         return dev->buffer;
 
     OPL3_GenerateStream(&dev->opl,
-                          &dev->buffer[dev->pos * 2],
-                          music_pos_global - dev->pos);
+                        &dev->buffer[dev->pos * 2],
+                        music_pos_global - dev->pos);
 
     for (; dev->pos < music_pos_global; dev->pos++) {
         dev->buffer[dev->pos * 2] /= 2;
@@ -1573,6 +1554,26 @@ nuked_drv_update(void *priv)
     return dev->buffer;
 }
 
+static int32_t *
+nuked_drv_update_48k(void *priv)
+{
+    nuked_drv_t *dev = (nuked_drv_t *) priv;
+
+    if (dev->pos >= sound_pos_global)
+        return dev->buffer;
+
+    OPL3_GenerateResampledStream(&dev->opl,
+                                 &dev->buffer[dev->pos * 2],
+                                 sound_pos_global - dev->pos);
+
+    for (; dev->pos < sound_pos_global; dev->pos++) {
+        dev->buffer[dev->pos * 2] /= 2;
+        dev->buffer[(dev->pos * 2) + 1] /= 2;
+    }
+
+    return dev->buffer;
+}
+
 static uint8_t
 nuked_drv_read(uint16_t port, void *priv)
 {
@@ -1581,7 +1582,7 @@ nuked_drv_read(uint16_t port, void *priv)
     if (dev->flags & FLAG_CYCLES)
         cycles -= ((int) (isa_timing * 8));
 
-    nuked_drv_update(dev);
+    dev->update(dev);
 
     uint8_t ret = 0xff;
 
@@ -1600,7 +1601,8 @@ static void
 nuked_drv_write(uint16_t port, uint8_t val, void *priv)
 {
     nuked_drv_t *dev = (nuked_drv_t *) priv;
-    nuked_drv_update(dev);
+
+    dev->update(dev);
 
     if ((port & 0x0001) == 0x0001) {
         OPL3_WriteRegBuffered(&dev->opl, dev->port, val);
@@ -1651,6 +1653,40 @@ nuked_drv_reset_buffer(void *priv)
     dev->pos = 0;
 }
 
+static void
+nuked_drv_close(void *priv)
+{
+    nuked_drv_t *dev = (nuked_drv_t *) priv;
+    free(dev);
+}
+
+static void *
+nuked_drv_init(const device_t *info)
+{
+    nuked_drv_t *dev = (nuked_drv_t *) calloc(1, sizeof(nuked_drv_t));
+    dev->flags       = FLAG_CYCLES;
+    if ((info->local & FM_TYPE_MASK) == FM_YMF262)
+        dev->flags |= FLAG_OPL3;
+    else
+        dev->status = 0x06;
+
+    dev->is_48k      = !!(info->local & FM_FORCE_48K);
+
+    /* Initialize the NukedOPL object. */
+    if (dev->is_48k) {
+        dev->update      = nuked_drv_update_48k;
+        OPL3_Reset(&dev->opl, FREQ_48000);
+    } else {
+        dev->update      = nuked_drv_update;
+        OPL3_Reset(&dev->opl, FREQ_49716);
+    }
+
+    timer_add(&dev->timers[0], nuked_timer_1, dev, 0);
+    timer_add(&dev->timers[1], nuked_timer_2, dev, 0);
+
+    return dev;
+}
+
 const device_t ym3812_nuked_device = {
     .name          = "Yamaha YM3812 OPL2 (NUKED)",
     .internal_name = "ym3812_nuked",
@@ -1659,7 +1695,7 @@ const device_t ym3812_nuked_device = {
     .init          = nuked_drv_init,
     .close         = nuked_drv_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL ,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1673,18 +1709,28 @@ const device_t ymf262_nuked_device = {
     .init          = nuked_drv_init,
     .close         = nuked_drv_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
 const fm_drv_t nuked_opl_drv = {
-    &nuked_drv_read,
-    &nuked_drv_write,
-    &nuked_drv_update,
-    &nuked_drv_reset_buffer,
-    &nuked_drv_set_do_cycles,
-    NULL,
-    NULL,
+    .read          = &nuked_drv_read,
+    .write         = &nuked_drv_write,
+    .update        = &nuked_drv_update,
+    .reset_buffer  = &nuked_drv_reset_buffer,
+    .set_do_cycles = &nuked_drv_set_do_cycles,
+    .priv          = NULL,
+    .generate      = NULL,
+};
+
+const fm_drv_t nuked_opl_drv_48k = {
+    .read          = &nuked_drv_read,
+    .write         = &nuked_drv_write,
+    .update        = &nuked_drv_update_48k,
+    .reset_buffer  = &nuked_drv_reset_buffer,
+    .set_do_cycles = &nuked_drv_set_do_cycles,
+    .priv          = NULL,
+    .generate      = NULL,
 };
