@@ -386,9 +386,9 @@ std::string opl_registers_base<Revision>::log_keyon(uint32_t choffs, uint32_t op
 	uint32_t opnum = (opoffs & 31) - 2 * ((opoffs & 31) / 8) + 18 * bitfield(opoffs, 8);
 
 	char buffer[256];
-	char *end = &buffer[0];
+	int end = 0;
 
-	end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, "%2u.%02u freq=%04X fb=%u alg=%X mul=%X tl=%02X ksr=%u ns=%u ksl=%u adr=%X/%X/%X sl=%X sus=%u",
+	end += snprintf(&buffer[end], sizeof(buffer) - end, "%2u.%02u freq=%04X fb=%u alg=%X mul=%X tl=%02X ksr=%u ns=%u ksl=%u adr=%X/%X/%X sl=%X sus=%u",
 		chnum, opnum,
 		ch_block_freq(choffs),
 		ch_feedback(choffs),
@@ -405,25 +405,25 @@ std::string opl_registers_base<Revision>::log_keyon(uint32_t choffs, uint32_t op
 		op_eg_sustain(opoffs));
 
 	if (OUTPUTS > 1)
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " out=%c%c%c%c",
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " out=%c%c%c%c",
 			ch_output_0(choffs) ? 'L' : '-',
 			ch_output_1(choffs) ? 'R' : '-',
 			ch_output_2(choffs) ? '0' : '-',
 			ch_output_3(choffs) ? '1' : '-');
 	if (op_lfo_am_enable(opoffs) != 0)
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " am=%u", lfo_am_depth());
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " am=%u", lfo_am_depth());
 	if (op_lfo_pm_enable(opoffs) != 0)
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " pm=%u", lfo_pm_depth());
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " pm=%u", lfo_pm_depth());
 	if (waveform_enable() && op_waveform(opoffs) != 0)
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " wf=%u", op_waveform(opoffs));
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " wf=%u", op_waveform(opoffs));
 	if (is_rhythm(choffs))
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " rhy=1");
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " rhy=1");
 	if (DYNAMIC_OPS)
 	{
 		operator_mapping map;
 		operator_map(map);
 		if (bitfield(map.chan[chnum], 16, 8) != 0xff)
-			end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " 4op");
+			end += snprintf(&buffer[end], sizeof(buffer) - end, " 4op");
 	}
 
 	return buffer;
@@ -685,9 +685,9 @@ std::string opll_registers::log_keyon(uint32_t choffs, uint32_t opoffs)
 	uint32_t opnum = opoffs;
 
 	char buffer[256];
-	char *end = &buffer[0];
+	int end = 0;
 
-	end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, "%u.%02u freq=%04X inst=%X fb=%u mul=%X",
+	end += snprintf(&buffer[end], sizeof(buffer) - end, "%u.%02u freq=%04X inst=%X fb=%u mul=%X",
 		chnum, opnum,
 		ch_block_freq(choffs),
 		ch_instrument(choffs),
@@ -695,11 +695,11 @@ std::string opll_registers::log_keyon(uint32_t choffs, uint32_t opoffs)
 		op_multiple(opoffs));
 
 	if (bitfield(opoffs, 0) == 1 || (is_rhythm(choffs) && choffs >= 6))
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " vol=%X", op_volume(opoffs));
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " vol=%X", op_volume(opoffs));
 	else
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " tl=%02X", ch_total_level(choffs));
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " tl=%02X", ch_total_level(choffs));
 
-	end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " ksr=%u ksl=%u adr=%X/%X/%X sl=%X sus=%u/%u",
+	end += snprintf(&buffer[end], sizeof(buffer) - end, " ksr=%u ksl=%u adr=%X/%X/%X sl=%X sus=%u/%u",
 		op_ksr(opoffs),
 		op_ksl(opoffs),
 		op_attack_rate(opoffs),
@@ -710,13 +710,13 @@ std::string opll_registers::log_keyon(uint32_t choffs, uint32_t opoffs)
 		ch_sustain(choffs));
 
 	if (op_lfo_am_enable(opoffs))
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " am=1");
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " am=1");
 	if (op_lfo_pm_enable(opoffs))
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " pm=1");
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " pm=1");
 	if (op_waveform(opoffs) != 0)
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " wf=1");
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " wf=1");
 	if (is_rhythm(choffs))
-		end += snprintf(end, SNPRINTF_BUFFER_SIZE_CALC, " rhy=1");
+		end += snprintf(&buffer[end], sizeof(buffer) - end, " rhy=1");
 
 	return buffer;
 }
@@ -1716,15 +1716,15 @@ uint8_t ymf278b::read_status()
 uint8_t ymf278b::read_data_pcm()
 {
 	// read from PCM
-	if (bitfield(m_address, 9) != 0)
-	{
-		uint8_t result = m_pcm.read(m_address & 0xff);
-		if ((m_address & 0xff) == 0x02)
-			result |= 0x20;
-
-		return result;
+	if (bitfield(m_address, 9) != 0) {
+		auto ret = m_pcm.read(m_address & 0xff);
+		if (m_address == 0x202) {
+			ret &= ~0xe0;
+			ret |= 0x20;
+		}
+		return ret;
 	}
-	return 0;
+    return 0;
 }
 
 
