@@ -290,6 +290,8 @@ plat_get_string(int i)
             return L"Hardware not available";
         case STRING_MONITOR_SLEEP:
             return L"Monitor in sleep mode";
+        case STRING_EDID_TOO_LARGE:
+            return "EDID file \"%ls\" is too large.";
     }
     return L"";
 }
@@ -349,7 +351,7 @@ plat_put_backslash(char *s)
 
 /* Return the last element of a pathname. */
 char *
-plat_get_basename(const char *path)
+path_get_basename(const char *path)
 {
     int c = (int) strlen(path);
 
@@ -403,11 +405,19 @@ path_append_filename(char *dest, const char *s1, const char *s2)
 int
 plat_dir_check(char *path)
 {
-    struct stat dummy;
-    if (stat(path, &dummy) < 0) {
+    struct stat stats;
+    if (stat(path, &stats) < 0)
         return 0;
-    }
-    return S_ISDIR(dummy.st_mode);
+    return S_ISDIR(stats.st_mode);
+}
+
+int
+plat_file_check(const char *path)
+{
+    struct stat stats;
+    if (stat(path, &stats) < 0)
+        return 0;
+    return !S_ISDIR(stats.st_mode);
 }
 
 int
@@ -875,19 +885,21 @@ plat_init_rom_paths(void)
 void
 plat_get_global_config_dir(char *outbuf, const size_t len)
 {
-    char *prefPath = SDL_GetPrefPath(NULL, "86Box");
-    strncpy(outbuf, prefPath, len);
-    path_slash(outbuf);
-    SDL_free(prefPath);
+    return plat_get_global_data_dir(outbuf, len);
 }
 
 void
 plat_get_global_data_dir(char *outbuf, const size_t len)
 {
-    char *prefPath = SDL_GetPrefPath(NULL, "86Box");
-    strncpy(outbuf, prefPath, len);
+    if (portable_mode) {
+        strncpy(outbuf, exe_path, len);
+    } else {
+        char *prefPath = SDL_GetPrefPath(NULL, "86Box");
+        strncpy(outbuf, prefPath, len);
+        SDL_free(prefPath);
+    }
+
     path_slash(outbuf);
-    SDL_free(prefPath);
 }
 
 void
@@ -1034,12 +1046,8 @@ monitor_thread(UNUSED(void *param))
 #        define EMU_GIT_HASH "0000000"
 #    endif
 
-#    if defined(__arm__) || defined(__TARGET_ARCH_ARM)
-#        define ARCH_STR "arm"
-#    elif defined(__aarch64__) || defined(_M_ARM64)
+#    if defined(__aarch64__) || defined(_M_ARM64)
 #        define ARCH_STR "arm64"
-#    elif defined(__i386) || defined(__i386__) || defined(_M_IX86)
-#        define ARCH_STR "i386"
 #    elif defined(__x86_64) || defined(__x86_64__) || defined(__amd64) || defined(_M_X64)
 #        define ARCH_STR "x86_64"
 #    else
@@ -1416,6 +1424,7 @@ main(int argc, char **argv)
         f_rl_callback_handler_remove();
     return 0;
 }
+
 char *
 plat_vidapi_name(UNUSED(int i))
 {
