@@ -8,8 +8,6 @@
  *
  *          Emulation of the Tseng Labs ET3000.
  *
- *
- *
  * Authors: Miran Grca, <mgrca8@gmail.com>
  *
  *          Copyright 2016-2018 Miran Grca.
@@ -424,7 +422,7 @@ et3000_out(uint16_t addr, uint8_t val, void *priv)
 static void
 et3000_recalctimings(svga_t *svga)
 {
-    svga->ma_latch |= (svga->crtc[0x23] & 2) << 15;
+    svga->memaddr_latch |= (svga->crtc[0x23] & 2) << 15;
     if (svga->crtc[0x25] & 1)
         svga->vblankstart |= 0x400;
     if (svga->crtc[0x25] & 2)
@@ -439,7 +437,7 @@ et3000_recalctimings(svga_t *svga)
     svga->interlace = !!(svga->crtc[0x25] & 0x80);
 
     if (svga->attrregs[0x16] & 0x10) {
-        svga->ma_latch <<= (1 << 0);
+        svga->memaddr_latch <<= (1 << 0);
         svga->rowoffset <<= (1 << 0);
         switch (svga->gdcreg[5] & 0x60) {
             case 0x00:
@@ -476,6 +474,20 @@ et3000_recalctimings(svga_t *svga)
             svga->clock = (cpuclock * (double) (1ULL << 32)) / 36000000.0;
             break;
     }
+
+    if (svga->render == svga_render_4bpp_highres)
+        svga->render = svga_render_4bpp_tseng_highres;
+}
+
+static int
+et3000_line_compare(svga_t* svga)
+{
+    if (svga->split > svga->vsyncstart) {
+        /* Don't do line compare if we're already in vertical retrace. */
+        /* This makes picture bouncing effect work on Copper demo. */
+        return 0;
+    }
+    return 1;
 }
 
 static void *
@@ -511,6 +523,7 @@ et3000_init(const device_t *info)
     dev->svga.miscout = 1;
 
     dev->svga.packed_chain4 = 1;
+    dev->svga.line_compare  = et3000_line_compare;
 
     return dev;
 }
@@ -548,17 +561,24 @@ et3000_available(void)
 }
 
 static const device_config_t et3000_config[] = {
-    { .name        = "memory",
-     .description = "Memory size",
-     .type        = CONFIG_SELECTION,
-     .default_int = 512,
-     .selection   = {
-          { .description = "256 KB",
-              .value       = 256 },
-          { .description = "512 KB",
-              .value       = 512 },
-          { .description = "" } } },
-    { .type = CONFIG_END }
+  // clang-format off
+    {
+        .name           = "memory",
+        .description    = "Memory size",
+        .type           = CONFIG_SELECTION,
+        .default_string = NULL,
+        .default_int    = 512,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = {
+            { .description = "256 KB", .value = 256 },
+            { .description = "512 KB", .value = 512 },
+            { .description = ""                     }
+        },
+        .bios           = { { 0 } }
+    },
+    { .name = "", .description = "", .type = CONFIG_END }
+  // clang-format on
 };
 
 const device_t et3000_isa_device = {
@@ -569,7 +589,7 @@ const device_t et3000_isa_device = {
     .init          = et3000_init,
     .close         = et3000_close,
     .reset         = NULL,
-    { .available = et3000_available },
+    .available     = et3000_available,
     .speed_changed = et3000_speed_changed,
     .force_redraw  = et3000_force_redraw,
     .config        = et3000_config
