@@ -8,8 +8,6 @@
  *
  *          Video 7 VGA 1024i emulation.
  *
- *
- *
  * Authors: Sarah Walker, <https://pcem-emulator.co.uk/>
  *          Miran Grca, <mgrca8@gmail.com>
  *
@@ -51,6 +49,7 @@ typedef struct ht216_t {
 
     uint32_t vram_mask, linear_base;
     uint8_t  adjust_cursor, monitor_type;
+    uint8_t  clk_sel;
 
     int ext_reg_enable;
     int isabus;
@@ -102,7 +101,8 @@ void    ht216_out(uint16_t addr, uint8_t val, void *priv);
 uint8_t ht216_in(uint16_t addr, void *priv);
 
 #define BIOS_G2_GC205_PATH              "roms/video/video7/BIOS.BIN"
-#define BIOS_VIDEO7_VGA_1024I_PATH      "roms/video/video7/Video Seven VGA 1024i - BIOS - v2.19 - 435-0062-05 - U17 - 27C256.BIN"
+#define BIOS_VIDEO7_VGA_1024I_219_PATH  "roms/video/video7/Video Seven VGA 1024i - BIOS - v2.19 - 435-0062-05 - U17 - 27C256.BIN"
+#define BIOS_VIDEO7_VGA_1024I_700_PATH  "roms/video/video7/Headland Video7 VGA 1024i v7.0 32x8 (IP) NMC27C256B@DIP28.BIN"
 #define BIOS_RADIUS_SVGA_MULTIVIEW_PATH "roms/video/video7/U18.BIN"
 #define BIOS_HT216_32_PATH              "roms/video/video7/HT21632.BIN"
 
@@ -134,6 +134,7 @@ dword_remap(svga_t *svga, uint32_t in_addr)
 {
     if (svga->packed_chain4)
         return in_addr;
+
     return ((in_addr & 0xfffc) << 2) | ((in_addr & 0x30000) >> 14) | (in_addr & ~0x3ffff);
 }
 
@@ -184,8 +185,8 @@ ht216_out(uint16_t addr, uint8_t val, void *priv)
     switch (addr) {
         case 0x3c2:
             /*Bit 17 of the display memory address, only active on odd/even modes, has no effect on graphics modes.*/
+            ht216->clk_sel = (ht216->clk_sel & ~0x03) | ((val & 0x0c) >> 2);
             ht216->misc   = val;
-            svga->miscout = val;
             ht216_log("HT216 misc val = %02x, mode = 0, chain4 = %x\n", val, svga->chain4);
             ht216_recalc_bank_regs(ht216, 0);
             ht216_remap(ht216);
@@ -253,10 +254,11 @@ ht216_out(uint16_t addr, uint8_t val, void *priv)
                           ht216->ht_regs[0xfc], ht216->ht_regs[0xfd], ht216->ht_regs[0xfe], ht216->ht_regs[0xff]);
                 return;
 #endif
-            } else if (svga->seqaddr >= 0x80 && ht216->ext_reg_enable) {
+            } else if ((svga->seqaddr >= 0x80) && ht216->ext_reg_enable) {
                 old                                  = ht216->ht_regs[svga->seqaddr & 0xff];
                 ht216->ht_regs[svga->seqaddr & 0xff] = val;
 
+                ht216_log("SeqAddr=%02x, val=%02x.\n", svga->seqaddr & 0xff, val);
                 switch (svga->seqaddr & 0xff) {
                     case 0x83:
                         svga->attraddr = val & 0x1f;
@@ -293,7 +295,29 @@ ht216_out(uint16_t addr, uint8_t val, void *priv)
                         break;
 
                     case 0xa4:
+                        if (ht216->id == 0x7861)
+                            ht216->clk_sel = (val >> 2) & 0x0f;
+                        else {
+                            if (svga->getclock == ics1494_getclock) {
+                                if (val & 0x10)
+                                    val &= ~0x10;
+                                else if (!(val & 0x10))
+                                    val |= 0x10;
+                            }
+                            ht216->clk_sel = (val >> 2) & 0x07;
+                        }
+                        svga->miscout = (svga->miscout & ~0x0c) | ((ht216->clk_sel & 0x03) << 2);
+                        svga->fullchange = changeframecount;
+                        svga_recalctimings(svga);
+                        break;
                     case 0xf8:
+                        if (ht216->id != 0x7861) {
+                            if ((val & 0x06) == 0x06)
+                                ht216->clk_sel = (val >> 5) & 0x07;
+                        } else {
+                            if ((val & 0x05) == 0x05)
+                                ht216->clk_sel = (val >> 4) & 0x0f;
+                        }
                         svga->fullchange = changeframecount;
                         svga_recalctimings(svga);
                         break;
@@ -557,6 +581,7 @@ ht216_in(uint16_t addr, void *priv)
         case 0x3c9:
             if (ht216->id == 0x7152)
                 return sc1148x_ramdac_in(addr, 0, svga->ramdac, svga);
+
             return svga_in(addr, svga);
 
         case 0x3cb:
@@ -564,9 +589,6 @@ ht216_in(uint16_t addr, void *priv)
                 return ht216->reg_3cb;
             break;
 
-        case 0x3cc:
-            return svga->miscout;
-
         case 0x3D4:
             return svga->crtcreg;
         case 0x3D5:
@@ -630,8 +652,7 @@ ht216_recalctimings(svga_t *svga)
     ht216_t   *ht216      = (ht216_t *) svga->priv;
     ibm8514_t *dev        = (ibm8514_t *) svga->dev8514;
     mach_t    *mach       = (mach_t *) svga->ext8514;
-    int      high_res_256 = 0;
-
+    int        high_res_256 = 0;
 
     if (ht216->id == 0x7861) {
         if (ht216->ht_regs[0xe0] & 0x20) {
@@ -642,23 +663,8 @@ ht216_recalctimings(svga_t *svga)
         }
     }
 
-    switch ((((((svga->miscout >> 2) & 3) || ((ht216->ht_regs[0xa4] >> 2) & 3)) | ((ht216->ht_regs[0xa4] >> 2) & 4)) || ((ht216->ht_regs[0xf8] >> 5) & 0x0f)) | ((ht216->ht_regs[0xf8] << 1) & 8)) {
-        case 0:
-        case 1:
-            break;
-        case 4:
-            svga->clock = (cpuclock * (double) (1ULL << 32)) / 50350000.0;
-            break;
-        case 5:
-            svga->clock = (cpuclock * (double) (1ULL << 32)) / 65000000.0;
-            break;
-        case 7:
-            svga->clock = (cpuclock * (double) (1ULL << 32)) / 40000000.0;
-            break;
-        default:
-            svga->clock = (cpuclock * (double) (1ULL << 32)) / 36000000.0;
-            break;
-    }
+    svga->clock = (cpuclock * (double) (1ULL << 32)) / svga->getclock(ht216->clk_sel, svga->clock_gen);
+    ht216_log("ClkSel V7=%02x, regf8=%02x, rega4=%02x, miscout=%x, vidclock=%02x.\n", ht216->clk_sel, ht216->ht_regs[0xf8], ht216->ht_regs[0xa4], (svga->miscout >> 2) & 0x03, svga->vidclock);
 
     svga->memaddr_latch |= ((ht216->ht_regs[0xf6] & 0x30) << 12);
 
@@ -699,7 +705,6 @@ ht216_recalctimings(svga_t *svga)
                     if (high_res_256) {
                         svga->hdisp >>= 1;
                         svga->dots_per_clock >>= 1;
-                        svga->clock /= 2;
                         ht216->adjust_cursor = 1;
                     }
                     svga->render = svga_render_8bpp_highres;
@@ -707,7 +712,6 @@ ht216_recalctimings(svga_t *svga)
                     if (high_res_256) {
                         svga->hdisp >>= 1;
                         svga->dots_per_clock >>= 1;
-                        svga->clock /= 2;
                         ht216->adjust_cursor = 1;
                         svga->render         = svga_render_8bpp_highres;
                     } else {
@@ -731,7 +735,6 @@ ht216_recalctimings(svga_t *svga)
                 svga->rowoffset <<= 1;
                 svga->hdisp >>= 1;
                 svga->dots_per_clock >>= 1;
-                svga->clock /= 2;
                 if ((svga->crtc[0x17] & 0x60) == 0x20) /*Would result in a garbled screen with trailing cursor glitches*/
                     svga->crtc[0x17] |= 0x40;
                 svga->render = svga_render_15bpp_highres;
@@ -1515,10 +1518,16 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
 {
     ht216_t *ht216 = malloc(sizeof(ht216_t));
     svga_t  *svga;
+    const char *bios_ver = NULL;
+    const char *fn = NULL;
 
     memset(ht216, 0, sizeof(ht216_t));
     svga = &ht216->svga;
 
+    ht216->id = info->local;
+    ht216->isabus = (info->flags & DEVICE_ISA) || (info->flags & DEVICE_ISA16);
+    ht216->mca = (info->flags & DEVICE_MCA);
+
     if (info->flags & DEVICE_VLB)
         video_inform(VIDEO_FLAG_TYPE_SPECIAL, &timing_v7vga_vlb);
     else if (info->flags & DEVICE_MCA)
@@ -1537,7 +1546,9 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
             rom_init(&ht216->bios_rom, BIOS_G2_GC205_PATH, 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
             break;
         case 2:
-            rom_init(&ht216->bios_rom, BIOS_VIDEO7_VGA_1024I_PATH, 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
+            bios_ver = (char *) device_get_config_bios("bios_ver");
+            fn = (char *) device_get_bios_file(info, bios_ver, 0);
+            rom_init(&ht216->bios_rom, fn, 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
             break;
         case 3:
             ht216->monitor_type = device_get_config_int("monitor_type");
@@ -1585,13 +1596,20 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
             break;
     }
 
+    svga->bpp     = 8;
+    svga->miscout = 1;
     svga->hwcursor.cur_ysize = 32;
     ht216->vram_mask         = mem_size - 1;
     svga->decode_mask        = mem_size - 1;
 
-    if (has_rom == 4)
+    if (ht216->id == 0x7152) {
         svga->ramdac = device_add(&sc11484_nors2_ramdac_device);
-
+        svga->clock_gen = device_add(&ics1494m_540_radius_ht209_device);
+        svga->getclock  = ics1494_getclock;
+    } else {
+        svga->clock_gen = device_add(&icd2047_20_device);
+        svga->getclock  = icd2047_getclock;
+    }
     svga->read = ht216_read;
     svga->readw = NULL;
     svga->readl = NULL;
@@ -1609,16 +1627,9 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
     mem_mapping_set_p(&svga->mapping, ht216);
     mem_mapping_disable(&ht216->linear_mapping);
 
-    ht216->id     = info->local;
-    ht216->isabus = (info->flags & DEVICE_ISA) || (info->flags & DEVICE_ISA16);
-    ht216->mca    = (info->flags & DEVICE_MCA);
-
     io_sethandler(0x03c0, 0x0020, ht216_in, NULL, NULL, ht216_out, NULL, NULL, ht216);
     io_sethandler(0x46e8, 0x0001, ht216_in, NULL, NULL, ht216_out, NULL, NULL, ht216);
 
-    svga->bpp     = 8;
-    svga->miscout = 1;
-
     if (ht216->id == 0x7861)
         ht216->ht_regs[0xb4] = 0x08; /*32-bit DRAM bus*/
 
@@ -1683,7 +1694,7 @@ g2_gc205_available(void)
 static int
 v7_vga_1024i_available(void)
 {
-    return rom_present(BIOS_VIDEO7_VGA_1024I_PATH);
+    return rom_present(BIOS_VIDEO7_VGA_1024I_219_PATH);
 }
 
 static int
@@ -1726,6 +1737,37 @@ ht216_force_redraw(void *priv)
 
 // clang-format off
 static const device_config_t v7_vga_1024i_config[] = {
+    {
+        .name           = "bios_ver",
+        .description    = "BIOS Revision",
+        .type           = CONFIG_BIOS,
+        .default_string = "v2_19",
+        .default_int    = 0,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = { { 0 } },
+        .bios           = {
+            {
+                .name          = "Version 2.19",
+                .internal_name = "v2_19",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 1,
+                .local         = 0,
+                .size          = 32768,
+                .files         = { BIOS_VIDEO7_VGA_1024I_219_PATH, "" }
+            },
+            {
+                .name          = "Version 7.00",
+                .internal_name = "v7_00",
+                .bios_type     = BIOS_NORMAL,
+                .files_no      = 1,
+                .local         = 0,
+                .size          = 32768,
+                .files         = { BIOS_VIDEO7_VGA_1024I_700_PATH, "" }
+            },
+            { .files_no = 0 }
+        }
+    },
     {
         .name           = "memory",
         .description    = "Memory size",
