@@ -8,8 +8,6 @@
  *
  *          Define all known video cards.
  *
- *
- *
  * Authors: Miran Grca, <mgrca8@gmail.com>
  *          Fred N. van Kempen, <decwiz@yahoo.com>
  *
@@ -136,8 +134,8 @@ video_cards[] = {
     { .device = &s3_phoenix_86c801_isa_device,                  .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &s3_spea_mirage_86c801_isa_device,              .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &s3_winner1000_805_isa_device,                  .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32_device,                              .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32i_isa_device,                         .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32_machspeed_vga_gui_2400s_isa_device,  .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32i_axis_microdevice_isa_device,        .flags = VIDEO_FLAG_TYPE_NONE },
     /* MCA */
     { .device = &mach32_mca_device,                             .flags = VIDEO_FLAG_TYPE_8514 },
     { .device = &gd5426_mca_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
@@ -147,12 +145,13 @@ video_cards[] = {
     /* VLB */
     { .device = &mach32_vlb_device,                             .flags = VIDEO_FLAG_TYPE_8514 },
     { .device = &mach64gx_vlb_device,                           .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32i_vlb_device,                         .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32_machspeed_vga_gui_2400s_vlb_device,  .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32i_hercules_dynamite_pro_vlb_device,   .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &et4000w32p_videomagic_revb_vlb_device,         .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_revc_vlb_device,                    .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_cardex_vlb_device,                  .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_vlb_device,                         .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_noncardex_vlb_device,               .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_cardex_revc_vlb_device,             .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_generic_revd_vlb_device,            .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_cardex_revd_vlb_device,             .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_diamond_revd_vlb_device,            .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &gd5424_vlb_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &gd5426_vlb_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &gd5428_vlb_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
@@ -186,6 +185,8 @@ video_cards[] = {
     /* PCI */
     { .device = &mach32_pci_device,                             .flags = VIDEO_FLAG_TYPE_8514 },
     { .device = &mach64gx_pci_device,                           .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &mach64ct_device,                               .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &mach64vt_device,                               .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &mach64vt2_device,                              .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &bochs_svga_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &chips_69000_device,                            .flags = VIDEO_FLAG_TYPE_NONE },
@@ -196,11 +197,10 @@ video_cards[] = {
     { .device = &gd5446_pci_device,                             .flags = VIDEO_FLAG_TYPE_SECONDARY },
     { .device = &gd5446_stb_pci_device,                         .flags = VIDEO_FLAG_TYPE_SECONDARY },
     { .device = &gd5480_pci_device,                             .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_videomagic_revb_pci_device,         .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_revc_pci_device,                    .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_cardex_pci_device,                  .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_noncardex_pci_device,               .flags = VIDEO_FLAG_TYPE_NONE },
-    { .device = &et4000w32p_pci_device,                         .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_cardex_revc_pci_device,             .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_generic_revd_pci_device,            .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_cardex_revd_pci_device,             .flags = VIDEO_FLAG_TYPE_NONE },
+    { .device = &et4000w32p_diamond_revd_pci_device,            .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &s3_elsa_winner1000_86c928_pci_device,          .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &s3_spea_mercury_lite_86c928_pci_device,        .flags = VIDEO_FLAG_TYPE_NONE },
     { .device = &s3_diamond_stealth64_964_pci_device,           .flags = VIDEO_FLAG_TYPE_NONE },
